(herald "IADH: unified model (UM)" (bound 20) (limit 2000)
  (algebra diffie-hellman))

(comment "CPSA 3.6.11")
(comment "All input read from tst/iadh_um.scm")
(comment "Strand count bounded at 20")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (lp ep expt) (self peer name) (key data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ep)) (send key))
    (uniq-gen e)
    (fn-of
      (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e ep))) key)))
    (neq (ep (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) lp))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA lA eB lB rndx)
    (lp ep lp-0 ep-0 expt))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp lp) (ep ep))
  (defstrand participant 5 (key key) (self B) (peer D) (l lB) (e eB)
    (lp lp-0) (ep ep-0))
  (absent (eB (exp (gen) lB)) (eB (exp (gen) lp-0)) (eA (exp (gen) lA))
    (eA (exp (gen) lp)))
  (fn-of
    (foo
      ((hash (exp (gen) (mul lB lp-0)) (exp (gen) (mul eB ep-0))) key)
      ((hash (exp (gen) (mul lA lp)) (exp (gen) (mul eA ep))) key)))
  (neq (ep-0 (one)) (eA eB) (ep (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) ep)) (send key))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp (gen) lp-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep-0)) (send key)))
  (label 0)
  (unrealized (0 0) (0 1) (1 0) (1 1))
  (preskeleton)
  (origs)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA lB rndx) (lp expt) (eA eB rndx)
    (ep expt))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) lB lp)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l lB) (e eB)
    (lp lp) (ep ep))
  (precedes ((1 2) (0 3)))
  (absent (eB (exp (gen) lB)) (eB (exp (gen) lp)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) lB lp))))
  (fn-of
    (foo ((hash (exp (gen) (mul lB lp)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) lB lp)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp (gen) lp) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key)))
  (label 1)
  (parent 0)
  (unrealized (0 0) (0 1) (0 3) (1 0) (1 1))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA rndx) (lp expt) (eA eB rndx)
    (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) lp l)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp lp) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) lp)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) lp l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul lp l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l) B (privk B)) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) lp l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) lp) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 2)
  (parent 1)
  (unrealized (0 0) (0 1) (0 3) (1 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (lA eA eB rndx) (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) l l)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l))))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 3)
  (parent 2)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (lA eA eB rndx) (ep expt)
    (l l-0 rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l lA) (e eA)
    (lp (mul (rec lA) l l-0)) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) lA))
    (eA (exp (gen) (mul (rec lA) l l-0))))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 4)
  (parent 2)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C name) (eA eB rndx) (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer C) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk C) l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 5)
  (parent 3)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l l-0) (e eA)
    (lp (mul l l (rec l-0))) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) (mul l l (rec l-0)))))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 6)
  (parent 3)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer C) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk C) (privk D) l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 7)
  (parent 4)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B C D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer C) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 8)
  (parent 4)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B C D name) (eA eB rndx) (ep expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self A) (peer C) (l l-1) (e eA)
    (lp (mul l l-0 (rec l-1))) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)) ((4 0) (0 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-1))
    (eA (exp (gen) (mul l l-0 (rec l-1)))))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 9)
  (parent 4)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (ep expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 10)
  (parent 5)
  (unrealized (0 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 11)
  (parent 7)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0))
    ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 12)
  (parent 8)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen eB l e)
  (operation nonce-test (displaced 3 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 13)
  (parent 10)
  (unrealized (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eA))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq (eA (one)) (eA e) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l e)
  (operation nonce-test (displaced 3 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 14)
  (parent 10)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C B) (D B) (lA l) (lB l) (lp l) (eA eA)
        (eB e) (ep eA))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 2) (0 3)) ((3 2) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 15)
  (parent 10)
  (unrealized (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 3 2 ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 16)
  (parent 10)
  (unrealized (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 17)
  (parent 10)
  (unrealized (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (ep expt) (l rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
    (exp (gen) (mul (rec eA) eB ep)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))))
  (label 18)
  (parent 10)
  (unrealized (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 19)
  (parent 11)
  (unrealized (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eA))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq (eA (one)) (eA e) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 20)
  (parent 11)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C D) (D D) (lA l) (lB l) (lp l-0) (eA eA)
        (eB e) (ep eA))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 21)
  (parent 11)
  (unrealized (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 22)
  (parent 11)
  (unrealized (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 23)
  (parent 11)
  (unrealized (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 24)
  (parent 11)
  (unrealized (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx)
    (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
    (exp (gen) (mul (rec eA) eB ep)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))))
  (label 25)
  (parent 11)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 26)
  (parent 12)
  (unrealized (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eA))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq (eA (one)) (eA e) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eA)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 27)
  (parent 12)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A D) (B B) (C B) (D D) (lA l-0) (lB l) (lp l-0)
        (eA eA) (eB e) (ep eA))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3)) ((4 2) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 28)
  (parent 12)
  (unrealized (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 29)
  (parent 12)
  (unrealized (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 30)
  (parent 12)
  (unrealized (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 31)
  (parent 12)
  (unrealized (1 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (ep expt) (l l-0 rndx)
    (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB ep)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep ep))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB ep))) key)))
  (neq (ep (one)) (eA eB) ((mul (rec eA) eB ep) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
    (exp (gen) (mul (rec eA) eB ep)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB ep)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) ep)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))
      (send (cat (exp (gen) (mul (rec eA) eB ep (rec w))) w))))
  (label 32)
  (parent 12)
  (unrealized (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (rec eB)) e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen eB l e)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eB)) e))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (rec eB)) e))
      (send (cat (exp (gen) (rec eB)) e))))
  (label 33)
  (parent 13)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul e e)) eB))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen eB l e)
  (operation nonce-test (added-listener (cat (exp (gen) (mul e e)) eB))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul e e)) eB))
      (send (cat (exp (gen) (mul e e)) eB))))
  (label 34)
  (parent 13)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB e)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB))) e))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB))) e))
      (send (cat (exp (gen) (mul eA (rec eB))) e))))
  (label 35)
  (parent 15)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA e)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB e)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eA e)) eB))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA e)) eB))
      (send (cat (exp (gen) (mul eA e)) eB))))
  (label 36)
  (parent 15)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul (rec eB) e)) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 2) (0 3)) ((3 2) (4 0)) ((4 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB e)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) e)) eA))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eB) e)) eA))
      (send (cat (exp (gen) (mul (rec eB) e)) eA))))
  (label 37)
  (parent 15)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA l)) eB))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eA l)) eB))
    (exp (gen) (mul eA (rec eB) l)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA l)) eB))
      (send (cat (exp (gen) (mul eA l)) eB))))
  (label 38)
  (parent 16)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul (rec eB) l)) eA))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (uniq-gen eA eB l)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l)) eA))
    (exp (gen) (mul eA (rec eB) l)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul (rec eB) l)) eA))
      (send (cat (exp (gen) (mul (rec eB) l)) eA))))
  (label 39)
  (parent 16)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB))) l-0))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0))))
  (label 40)
  (parent 17)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) (mul eA l-0)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) eB))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) (mul eA l-0)) eB))
      (send (cat (exp (gen) (mul eA l-0)) eB))))
  (label 41)
  (parent 17)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eB) l-0)) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 3)) ((3 0) (4 0)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l-0)) eA))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eB) l-0)) eA))
      (send (cat (exp (gen) (mul (rec eB) l-0)) eA))))
  (label 42)
  (parent 17)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (ep (mul eA (rec eB) w)) (w-0 w))
    (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 43)
  (parent 18)
  (unrealized (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 44)
  (parent 18)
  (unrealized (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 45)
  (parent 18)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 46)
  (parent 18)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((mul eA (rec eB) w l) (one)) (eA eB) ((mul w l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 47)
  (parent 18)
  (unrealized (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 48)
  (parent 18)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (rec eB)) e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eB)) e))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (rec eB)) e))
      (send (cat (exp (gen) (rec eB)) e))))
  (label 49)
  (parent 19)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul e e)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener (cat (exp (gen) (mul e e)) eB))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul e e)) eB))
      (send (cat (exp (gen) (mul e e)) eB))))
  (label 50)
  (parent 19)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) e))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB))) e))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB))) e))
      (send (cat (exp (gen) (mul eA (rec eB))) e))))
  (label 51)
  (parent 21)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA e)) eB))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eA e)) eB))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA e)) eB))
      (send (cat (exp (gen) (mul eA e)) eB))))
  (label 52)
  (parent 21)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul (rec eB) e)) eA))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) e)) eA))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eB) e)) eA))
      (send (cat (exp (gen) (mul (rec eB) e)) eA))))
  (label 53)
  (parent 21)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eA l-0)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) eB))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eA l-0)) eB))
      (send (cat (exp (gen) (mul eA l-0)) eB))))
  (label 54)
  (parent 22)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eB) l-0)) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l-0)) eA))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eB) l-0)) eA))
      (send (cat (exp (gen) (mul (rec eB) l-0)) eA))))
  (label 55)
  (parent 22)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB))) l-0))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0))))
  (label 56)
  (parent 23)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA l-0)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) eB))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA l-0)) eB))
      (send (cat (exp (gen) (mul eA l-0)) eB))))
  (label 57)
  (parent 23)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eB) l-0)) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l-0)) eA))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eB) l-0)) eA))
      (send (cat (exp (gen) (mul (rec eB) l-0)) eA))))
  (label 58)
  (parent 23)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB))) l-1))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1))))
  (label 59)
  (parent 24)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) eB))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) eB))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA l-1)) eB))
      (send (cat (exp (gen) (mul eA l-1)) eB))))
  (label 60)
  (parent 24)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec eB) l-1)) eA))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l-1)) eA))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eB) l-1)) eA))
      (send (cat (exp (gen) (mul (rec eB) l-1)) eA))))
  (label 61)
  (parent 24)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (ep (mul eA (rec eB) w)) (w-0 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 62)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 63)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 64)
  (parent 25)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 65)
  (parent 25)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 66)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 67)
  (parent 25)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 68)
  (parent 25)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (rec eB)) e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eB)) e))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (rec eB)) e))
      (send (cat (exp (gen) (rec eB)) e))))
  (label 69)
  (parent 26)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul e e)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener (cat (exp (gen) (mul e e)) eB))
    (exp (gen) (mul (rec eB) e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul e e)) eB))
      (send (cat (exp (gen) (mul e e)) eB))))
  (label 70)
  (parent 26)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) e))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB))) e))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB))) e))
      (send (cat (exp (gen) (mul eA (rec eB))) e))))
  (label 71)
  (parent 28)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA e)) eB))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eA e)) eB))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA e)) eB))
      (send (cat (exp (gen) (mul eA e)) eB))))
  (label 72)
  (parent 28)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul (rec eB) e)) eA))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (0 3))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) e)) eA))
    (exp (gen) (mul eA (rec eB) e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eB) e)) eA))
      (send (cat (exp (gen) (mul (rec eB) e)) eA))))
  (label 73)
  (parent 28)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eA l-0)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) eB))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eA l-0)) eB))
      (send (cat (exp (gen) (mul eA l-0)) eB))))
  (label 74)
  (parent 29)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eB) l-0)) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l-0)) eA))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eB) l-0)) eA))
      (send (cat (exp (gen) (mul (rec eB) l-0)) eA))))
  (label 75)
  (parent 29)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA l-0)) eB))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-0)) eB))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA l-0)) eB))
      (send (cat (exp (gen) (mul eA l-0)) eB))))
  (label 76)
  (parent 30)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul (rec eB) l-0)) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l-0)) eA))
    (exp (gen) (mul eA (rec eB) l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eB) l-0)) eA))
      (send (cat (exp (gen) (mul (rec eB) l-0)) eA))))
  (label 77)
  (parent 30)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB))) l-1))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1))))
  (label 78)
  (parent 31)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA l-1)) eB))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA l-1)) eB))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA l-1)) eB))
      (send (cat (exp (gen) (mul eA l-1)) eB))))
  (label 79)
  (parent 31)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul (rec eB) l-1)) eA))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eB) l-1)) eA))
    (exp (gen) (mul eA (rec eB) l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eB) l-1)) eA))
      (send (cat (exp (gen) (mul (rec eB) l-1)) eA))))
  (label 80)
  (parent 31)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (ep (mul eA (rec eB) w)) (w-0 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 81)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 82)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 83)
  (parent 32)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 84)
  (parent 32)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 85)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 86)
  (parent 32)
  (unrealized (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 87)
  (parent 32)
  (unrealized (1 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 5 ltx-gen 2) l-0 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 88)
  (parent 40)
  (unrealized (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) eB))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) eB)) (send (cat (gen) eB))))
  (label 89)
  (parent 43)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 90)
  (parent 43)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 91)
  (parent 43)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq (l (one)) (eA eB) ((mul (rec eA) eB l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB l)))
      (send (cat (gen) (mul (rec eA) eB l)))))
  (label 92)
  (parent 43)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 93)
  (parent 43)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx) (w w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))))
  (label 94)
  (parent 43)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e))))))
  (label 95)
  (parent 44)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (e e-0) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 96)
  (parent 44)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e eB) ((mul eB (rec e) e-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 97)
  (parent 44)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq (l (one)) (e eB) ((mul eB (rec e) l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l)))))
  (label 98)
  (parent 44)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 99)
  (parent 44)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eB) w e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))))
  (label 100)
  (parent 44)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) l)
  (uniq-gen eA l e)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 101)
  (parent 45)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C B) (D B) (lA l) (lB l) (lp l) (eA eA)
        (eB e) (ep (mul eA w)))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eB e l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e-0 eB) (eB (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 102)
  (parent 46)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA e l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (eA e-0) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 103)
  (parent 46)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA eB l-0 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 5 4 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 104)
  (parent 46)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0))
    ((5 2) (1 3)) ((5 2) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (eA eB) ((mul (rec eA) eB e-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 105)
  (parent 46)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 106)
  (parent 46)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0))
    ((5 0) (1 3)) ((5 0) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 107)
  (parent 46)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))))
  (label 108)
  (parent 46)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l))))))
  (label 109)
  (parent 47)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 110)
  (parent 47)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (1 3)) ((4 2) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 111)
  (parent 47)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) key)))
  (neq (l (one)) (eA eB) ((mul (rec eA) eB l) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB)))
      (send (cat (exp (gen) l) (mul (rec eA) eB)))))
  (label 112)
  (parent 47)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 113)
  (parent 47)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l))) key)))
  (neq ((mul eA (rec eB) w l) (one)) (eA eB) ((mul w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l (rec w-0))) w-0))))
  (label 114)
  (parent 47)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 115)
  (parent 48)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 116)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0))
    ((5 2) (1 3)) ((5 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 117)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 118)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 4 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 119)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (1 3)) ((4 0) (3 0))
    ((5 0) (1 3)) ((5 0) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) l)
  (precur (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 120)
  (parent 48)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 121)
  (parent 48)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 1)) ((4 0) (1 1))
    ((4 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 5 ltx-gen 2) l-0 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 122)
  (parent 56)
  (unrealized (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 123)
  (parent 59)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) eB))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) eB)) (send (cat (gen) eB))))
  (label 124)
  (parent 62)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 125)
  (parent 62)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 126)
  (parent 62)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 127)
  (parent 62)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 128)
  (parent 62)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 129)
  (parent 62)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))))
  (label 130)
  (parent 62)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e))))))
  (label 131)
  (parent 63)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (e e-0) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 132)
  (parent 63)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e eB) ((mul eB (rec e) e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 133)
  (parent 63)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 134)
  (parent 63)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 135)
  (parent 63)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (e eB) ((mul eB (rec e) l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 136)
  (parent 63)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eB) w e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))))
  (label 137)
  (parent 63)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 0)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen eA l l-0 e)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 138)
  (parent 64)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A B) (B B) (C D) (D D) (lA l) (lB l) (lp l-0) (eA eA)
        (eB e) (ep (mul eA w)))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eB e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e-0 eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 139)
  (parent 65)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (eA e-0) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 140)
  (parent 65)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB l-1 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 141)
  (parent 65)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (eA eB) ((mul (rec eA) eB e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 142)
  (parent 65)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 143)
  (parent 65)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 144)
  (parent 65)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 145)
  (parent 65)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))))
  (label 146)
  (parent 65)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l))))))
  (label 147)
  (parent 66)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 148)
  (parent 66)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 149)
  (parent 66)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 150)
  (parent 66)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 151)
  (parent 66)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 152)
  (parent 66)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 153)
  (parent 66)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0))))))
  (label 154)
  (parent 67)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 155)
  (parent 67)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 156)
  (parent 67)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 157)
  (parent 67)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 158)
  (parent 67)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 159)
  (parent 67)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 160)
  (parent 67)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 161)
  (parent 68)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 162)
  (parent 68)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 163)
  (parent 68)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 164)
  (parent 68)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 165)
  (parent 68)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 166)
  (parent 68)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 167)
  (parent 68)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))))
  (label 168)
  (parent 68)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 169)
  (parent 78)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) eB))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) eB)) (send (cat (gen) eB))))
  (label 170)
  (parent 81)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 171)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 172)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 173)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 174)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 175)
  (parent 81)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) key)))
  (neq ((mul eA (rec eB) w) (one)) (eA eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w (rec w-0))) w-0))))
  (label 176)
  (parent 81)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e))))))
  (label 177)
  (parent 82)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (e e-0) ((mul (rec e) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 178)
  (parent 82)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e eB) ((mul eB (rec e) e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 179)
  (parent 82)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 180)
  (parent 82)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (e eB) ((mul eB (rec e) l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-0)))))
  (label 181)
  (parent 82)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (e eB) ((mul eB (rec e) l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 182)
  (parent 82)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul (rec eB) w e e) (one)) (e eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eB) w e e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eB) w e e (rec w-0))) w-0))))
  (label 183)
  (parent 82)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA w)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA w) (one)) (eA e) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA l l-0 e)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 184)
  (parent 83)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((key key) (A D) (B B) (C B) (D D) (lA l-0) (lB l) (lp l-0)
        (eA eA) (eB e) (ep (mul eA w)))))
  (origs))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eB e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (e-0 eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e))))
      (send (cat (exp (gen) e) (mul eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 185)
  (parent 84)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (eA e l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0))) key)))
  (neq (e-0 (one)) (eA e-0) ((mul (rec eA) e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 186)
  (parent 84)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB l-1 rndx) (lp expt)
    (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 187)
  (parent 84)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e-0))) key)))
  (neq (e-0 (one)) (eA eB) ((mul (rec eA) eB e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 188)
  (parent 84)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 189)
  (parent 84)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 190)
  (parent 84)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 191)
  (parent 84)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) key)))
  (neq ((mul eA (rec eB) w e) (one)) (eA eB) ((mul w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w e)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w e (rec w-0))) w-0))))
  (label 192)
  (parent 84)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0))))))
  (label 193)
  (parent 85)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))))
  (label 194)
  (parent 85)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 195)
  (parent 85)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 196)
  (parent 85)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 197)
  (parent 85)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 198)
  (parent 85)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 199)
  (parent 85)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l))))))
  (label 200)
  (parent 86)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) e e)))))
  (label 201)
  (parent 86)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 2) (1 3)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 202)
  (parent 86)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) l-0)))))
  (label 203)
  (parent 86)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 204)
  (parent 86)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 205)
  (parent 86)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) key)))
  (neq ((mul eA (rec eB) w l-0) (one)) (eA eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-0)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-0 (rec w-0))) w-0))))
  (label 206)
  (parent 86)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep eB))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (e eB) (eB (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l))))
      (send (cat (exp (gen) l) (mul eB (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 207)
  (parent 87)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e))) key)))
  (neq (e (one)) (eA e) ((mul (rec eA) e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 208)
  (parent 87)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 2) (1 3)) ((6 2) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) key)))
  (neq (e (one)) (eA eB) ((mul (rec eA) eB e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 209)
  (parent 87)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 210)
  (parent 87)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 211)
  (parent 87)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 212)
  (parent 87)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-2))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 0) (4 0)) ((6 0) (1 3)) ((6 0) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) key)))
  (neq (l-2 (one)) (eA eB) ((mul (rec eA) eB l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 213)
  (parent 87)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (w-0 expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) key)))
  (neq ((mul eA (rec eB) w l-1) (one)) (eA eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
    (exp (gen) (mul eA (rec eB) w l-1)) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul eA (rec eB) w l-1 (rec w-0))) w-0))))
  (label 214)
  (parent 87)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) eA) eB))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener (cat (exp (gen) eA) eB))
    (exp (gen) (mul eA (rec eB))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) eA) eB)) (send (cat (exp (gen) eA) eB))))
  (label 215)
  (parent 88)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) (rec eB)) eA))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 1) (3 0))
    ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eB)) eA))
    (exp (gen) (mul eA (rec eB))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) (rec eB)) eA))
      (send (cat (exp (gen) (rec eB)) eA))))
  (label 216)
  (parent 88)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB w)) (w-1 w))
    (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 217)
  (parent 94)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 218)
  (parent 94)
  (unrealized (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 219)
  (parent 94)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w e)))
      (send (cat (gen) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 220)
  (parent 94)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w l)))
      (send (cat (gen) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 221)
  (parent 94)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 222)
  (parent 94)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w-0 (mul eB (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 223)
  (parent 100)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w (rec e))))
      (send (cat (exp (gen) e) (mul eB w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 224)
  (parent 100)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((mul w e-0) (one)) (e e-0) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 225)
  (parent 100)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (e eB) ((mul eB (rec e) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 226)
  (parent 100)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (e eB) ((mul eB (rec e) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 227)
  (parent 100)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 228)
  (parent 100)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec e) w))
      (e-0 e) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 229)
  (parent 108)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e-0)))
        key)))
  (neq ((mul w e-0) (one)) (e-0 eB) ((mul eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 230)
  (parent 108)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((mul w e-0) (one)) (eA e-0) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 231)
  (parent 108)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA eB rndx) (w expt) (l-0 rndx)
    (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 232)
  (parent 108)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (eA eB) ((mul (rec eA) eB w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 233)
  (parent 108)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0)))
        key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 234)
  (parent 108)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 235)
  (parent 108)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l) w))
      (l-0 l) (w-1 w)) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 236)
  (parent 114)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w (rec l))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w (rec l))))
      (send (cat (exp (gen) l) (mul eB w (rec l)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 237)
  (parent 114)
  (unrealized (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 238)
  (parent 114)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 239)
  (parent 114)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 240)
  (parent 114)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 241)
  (parent 114)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 242)
  (parent 121)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 243)
  (parent 121)
  (unrealized (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 244)
  (parent 121)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 245)
  (parent 121)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0)))
        key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 246)
  (parent 121)
  (unrealized (0 3) (3 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 247)
  (parent 121)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 248)
  (parent 121)
  (unrealized (0 3) (3 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) eA) eB))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 1)) ((4 0) (1 1))
    ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener (cat (exp (gen) eA) eB))
    (exp (gen) (mul eA (rec eB))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) eA) eB)) (send (cat (exp (gen) eA) eB))))
  (label 249)
  (parent 122)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener (cat (exp (gen) (rec eB)) eA))
  (precedes ((0 2) (5 0)) ((1 2) (0 3)) ((1 2) (5 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 1)) ((4 0) (1 1))
    ((4 1) (3 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eB)) eA))
    (exp (gen) (mul eA (rec eB))) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-0))
      (send (cat (exp (gen) (mul eA (rec eB))) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv (cat (exp (gen) (rec eB)) eA))
      (send (cat (exp (gen) (rec eB)) eA))))
  (label 250)
  (parent 122)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) eA) eB))
  (precedes ((0 2) (6 0)) ((1 2) (0 3)) ((1 2) (6 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener (cat (exp (gen) eA) eB))
    (exp (gen) (mul eA (rec eB))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) eA) eB)) (send (cat (exp (gen) eA) eB))))
  (label 251)
  (parent 123)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) (rec eB)) eA))
  (precedes ((0 2) (6 0)) ((1 2) (0 3)) ((1 2) (6 0)) ((2 0) (0 0))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eB)) eA))
    (exp (gen) (mul eA (rec eB))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) (rec eB)) eA))
      (send (cat (exp (gen) (rec eB)) eA))))
  (label 252)
  (parent 123)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 253)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 254)
  (parent 130)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 255)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w e)))
      (send (cat (gen) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 256)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 257)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 258)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-1)))
      (send (cat (gen) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 259)
  (parent 130)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w-0 (mul eB (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 260)
  (parent 137)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w (rec e))))
      (send (cat (exp (gen) e) (mul eB w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 261)
  (parent 137)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((mul w e-0) (one)) (e e-0) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 262)
  (parent 137)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (e eB) ((mul eB (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 263)
  (parent 137)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 264)
  (parent 137)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 265)
  (parent 137)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (e eB) ((mul eB (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 266)
  (parent 137)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec e) w))
      (e-0 e) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 267)
  (parent 146)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e-0)))
        key)))
  (neq ((mul w e-0) (one)) (e-0 eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 268)
  (parent 146)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((mul w e-0) (one)) (eA e-0) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 269)
  (parent 146)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 270)
  (parent 146)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (eA eB) ((mul (rec eA) eB w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 271)
  (parent 146)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 272)
  (parent 146)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 273)
  (parent 146)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 274)
  (parent 146)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 275)
  (parent 153)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB w (rec l))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB w (rec l))))
      (send (cat (exp (gen) l) (mul eB w (rec l)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 276)
  (parent 153)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 277)
  (parent 153)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 278)
  (parent 153)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 279)
  (parent 153)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 280)
  (parent 153)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 281)
  (parent 153)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 282)
  (parent 160)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w (rec l-0))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB w (rec l-0)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 283)
  (parent 160)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 284)
  (parent 160)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 285)
  (parent 160)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 286)
  (parent 160)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 287)
  (parent 160)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 288)
  (parent 160)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx) (w expt))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 289)
  (parent 168)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 290)
  (parent 168)
  (unrealized (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 291)
  (parent 168)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 292)
  (parent 168)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 293)
  (parent 168)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 294)
  (parent 168)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 295)
  (parent 168)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 296)
  (parent 168)
  (unrealized (0 3) (4 0))
  (comment "9 in cohort - 9 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) eA) eB))
  (precedes ((0 2) (6 0)) ((1 2) (0 3)) ((1 2) (6 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener (cat (exp (gen) eA) eB))
    (exp (gen) (mul eA (rec eB))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) eA) eB)) (send (cat (exp (gen) eA) eB))))
  (label 297)
  (parent 169)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eA (rec eB))) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) (rec eB)) eA))
  (precedes ((0 2) (6 0)) ((1 2) (0 3)) ((1 2) (6 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eB)) eA))
    (exp (gen) (mul eA (rec eB))) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eA (rec eB))) l-1))
      (send (cat (exp (gen) (mul eA (rec eB))) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) (rec eB)) eA))
      (send (cat (exp (gen) (rec eB)) eA))))
  (label 298)
  (parent 169)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 299)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 300)
  (parent 176)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e e)))
      (send (cat (gen) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 301)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w e)))
      (send (cat (gen) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 302)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 303)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-0)))
      (send (cat (gen) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 304)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w l-1)))
      (send (cat (gen) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 305)
  (parent 176)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w-0 (mul eB (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 306)
  (parent 183)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w (rec e))))
      (send (cat (exp (gen) e) (mul eB w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 307)
  (parent 183)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0))) key)))
  (neq ((mul w e-0) (one)) (e e-0) ((mul (rec e) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0 e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 308)
  (parent 183)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (e eB) ((mul eB (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 309)
  (parent 183)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 310)
  (parent 183)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (e eB) ((mul eB (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 311)
  (parent 183)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (e eB) ((mul eB (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 312)
  (parent 183)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec e) w))
      (e-0 e) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 313)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e-0)))
        key)))
  (neq ((mul w e-0) (one)) (e-0 eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 314)
  (parent 192)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))
        key)))
  (neq ((mul w e-0) (one)) (eA e-0) ((mul (rec eA) w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 315)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 316)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0))) key)))
  (neq ((mul w e-0) (one)) (eA eB) ((mul (rec eA) eB w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 317)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 318)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 319)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 320)
  (parent 192)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 321)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w (rec l-0))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eB w (rec l-0)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 322)
  (parent 199)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 323)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 324)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 325)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 326)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 327)
  (parent 199)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 328)
  (parent 206)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w (rec l))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB w (rec l))))
      (send (cat (exp (gen) l) (mul eB w (rec l)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 329)
  (parent 206)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 330)
  (parent 206)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 331)
  (parent 206)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 332)
  (parent 206)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 333)
  (parent 206)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 334)
  (parent 206)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx) (w expt))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul (rec eA) eB (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 335)
  (parent 214)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 l-1 e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 336)
  (parent 214)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e))) key)))
  (neq ((mul w e) (one)) (eA e) ((mul (rec eA) w e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 337)
  (parent 214)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 338)
  (parent 214)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 339)
  (parent 214)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 340)
  (parent 214)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 341)
  (parent 214)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 342)
  (parent 214)
  (unrealized (0 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 343)
  (parent 217)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eB eA)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 344)
  (parent 217)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 345)
  (parent 217)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 346)
  (parent 218)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 347)
  (parent 218)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener eB) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eB) (send eB)))
  (label 348)
  (parent 218)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec e) l-0) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 349)
  (parent 219)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 350)
  (parent 219)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 351)
  (parent 219)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 352)
  (parent 220)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 353)
  (parent 220)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 354)
  (parent 220)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 355)
  (parent 220)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 356)
  (parent 221)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l))) (send (cat (gen) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 357)
  (parent 221)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB))))))
  (label 358)
  (parent 221)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul (rec eA) eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (added-absence l (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 359)
  (parent 221)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 360)
  (parent 222)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 361)
  (parent 222)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 362)
  (parent 222)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 363)
  (parent 222)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 364)
  (parent 222)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 365)
  (parent 223)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (gen) (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (gen) (mul (rec eB) e e l-0)))
      (send (cat (gen) (mul (rec eB) e e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 366)
  (parent 223)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 367)
  (parent 223)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 368)
  (parent 223)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) e)) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 369)
  (parent 224)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 370)
  (parent 224)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul w (rec e)))
    (mul w (rec e)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 371)
  (parent 224)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 372)
  (parent 224)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 373)
  (parent 225)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e e-0 l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul e e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e e (rec e-0) l-0) (one)) (e e-0) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 374)
  (parent 225)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 375)
  (parent 225)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 376)
  (parent 225)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul (rec eB) e e (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 377)
  (parent 226)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 378)
  (parent 226)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (e rndx)
    (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 379)
  (parent 226)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 380)
  (parent 226)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 381)
  (parent 226)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec eB) e e (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l)))
      (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l) (mul (rec eB) e e (rec l))))
      (send (cat (exp (gen) l) (mul (rec eB) e e (rec l))))))
  (label 382)
  (parent 227)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB e l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 383)
  (parent 227)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (e eB) ((mul (rec e) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l))
    (mul (rec e) (rec e) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB))))))
  (label 384)
  (parent 227)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w e e)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul eB w l)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l e e))) key)))
  (neq ((mul w l e e) (one)) (e eB) ((mul eB w l e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-absence e (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l)))
      (send (cat (exp (gen) e) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w e e)))
      (send (cat (exp (gen) l) (mul w e e)))))
  (label 385)
  (parent 227)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB e rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (added-absence l (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l))))))
  (label 386)
  (parent 227)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 387)
  (parent 228)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 388)
  (parent 228)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 389)
  (parent 228)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 390)
  (parent 228)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 391)
  (parent 228)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 392)
  (parent 228)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 393)
  (parent 229)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-1)))
      (send (cat (gen) (mul eA (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 394)
  (parent 229)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 395)
  (parent 229)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA e rndx)
    (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 396)
  (parent 229)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 397)
  (parent 229)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (l-0 e eB e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep e-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul e (rec eB) e-0) (one)) (e eB) (e-0 (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eB e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e-0) (w (mul (rec eB) e-0))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0)))))
  (label 398)
  (parent 230)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 e eB e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul e-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul e e-0 l-1)))
        key)))
  (neq ((mul e (rec eB) e-0 l-1) (one)) (e eB) ((mul e-0 l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eB e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 399)
  (parent 230)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w))
    (mul (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB))))))
  (label 400)
  (parent 230)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 401)
  (parent 230)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (l-0 eA e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 402)
  (parent 231)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 eA e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul eA e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA e (rec e-0) l-1) (one)) (eA e-0) ((mul e l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 403)
  (parent 231)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 404)
  (parent 231)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 405)
  (parent 231)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 406)
  (parent 231)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (e eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e eA))) key)))
  (neq ((mul e eA (rec eB)) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 407)
  (parent 232)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (e eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e eA l-1))) key)))
  (neq ((mul e eA (rec eB) l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 408)
  (parent 232)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eB rndx) (w expt) (l-0 rndx)
    (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 409)
  (parent 232)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA rndx) (w expt) (l-0 rndx)
    (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 410)
  (parent 232)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul eA (rec eB) e (rec e-0))) (e-2 e-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 411)
  (parent 233)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 412)
  (parent 233)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0 eA))) key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w e-0))
    (mul eB (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 413)
  (parent 233)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 414)
  (parent 233)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0))) key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 415)
  (parent 233)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 416)
  (parent 233)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-0))) (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0))))))
  (label 417)
  (parent 234)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 418)
  (parent 234)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-0))
    (mul eB (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 419)
  (parent 234)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-0))
    (mul (rec eA) (rec e) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 420)
  (parent 234)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB rndx)
    (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w e)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 e)))
        key)))
  (neq ((mul w l-0 e) (one)) (eA eB) ((mul (rec eA) eB w l-0 e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w e)))
      (send (cat (exp (gen) l-0) (mul w e)))))
  (label 421)
  (parent 234)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 422)
  (parent 234)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 423)
  (parent 235)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 424)
  (parent 235)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 425)
  (parent 235)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 426)
  (parent 235)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 427)
  (parent 235)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 e))) key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 428)
  (parent 235)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 429)
  (parent 235)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-0 l) (w (mul eA (rec eB) l)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l)))
      (send (cat (gen) (mul eA (rec eB) l)))))
  (label 430)
  (parent 236)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l l-0)))
      (send (cat (gen) (mul eA (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 431)
  (parent 236)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB (rec l) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w))
    (mul eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 432)
  (parent 236)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 433)
  (parent 236)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul (rec eA) eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (added-absence l (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l))) (send (cat (gen) (mul w l)))))
  (label 434)
  (parent 236)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e eB l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l))) key)))
  (neq ((mul e (rec eB) l) (one)) (e eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e eB l)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l)) (l-0 l)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l)))
      (send (cat (exp (gen) e) (mul (rec eB) l)))))
  (label 435)
  (parent 237)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (e eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l l-0))) key)))
  (neq ((mul e (rec eB) l l-0) (one)) (e eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e eB l l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 436)
  (parent 237)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-absence eB (mul w (rec l)))
    (mul w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 437)
  (parent 237)
  (unrealized (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l))) key)))
  (neq ((mul w e l) (one)) (e eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l)
  (operation nonce-test (added-absence l (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w)))
      (send (cat (exp (gen) l) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 438)
  (parent 237)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA l (rec e)) (one)) (eA e) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l (rec e) (rec e))) (l-0 l)
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e))))))
  (label 439)
  (parent 238)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA l (rec e) l-0) (one)) (eA e) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l (rec e) l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 440)
  (parent 238)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 441)
  (parent 238)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (e l rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (l (mul (rec eA) w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l))) key)))
  (neq ((mul w e l) (one)) (eA e) ((mul (rec eA) w e e l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA e l)
  (operation nonce-test (added-absence l (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l)))
      (send (cat (exp (gen) e) (mul w l)))))
  (label 442)
  (parent 238)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 443)
  (parent 238)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0)
      (w (mul eA (rec eB) l-0 (rec e))) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 444)
  (parent 239)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 445)
  (parent 239)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB l-0 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eA (mul eB (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eB l-0 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w e))
    (mul eB (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 446)
  (parent 239)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA l-0 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (eB (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w e))
    (mul (rec eA) (rec l-0) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 447)
  (parent 239)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB rndx)
    (w expt) (e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (l-0 (mul (rec eA) eB w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (eA eB) ((mul (rec eA) eB w e l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 448)
  (parent 239)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)))
  (absent (e (mul (rec eA) eB (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 449)
  (parent 239)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l eA))) key)))
  (neq ((mul l eA (rec eB)) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB))))
      (send (cat (exp (gen) l) (mul eA (rec eB))))))
  (label 450)
  (parent 240)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l eA l-0))) key)))
  (neq ((mul l eA (rec eB) l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-strand ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 451)
  (parent 240)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w)))
      (send (cat (exp (gen) l) (mul eB w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))))
  (label 452)
  (parent 240)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB))))))
  (label 453)
  (parent 240)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 454)
  (parent 241)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 455)
  (parent 241)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 456)
  (parent 241)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 457)
  (parent 241)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 458)
  (parent 241)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 459)
  (parent 241)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 460)
  (parent 241)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 461)
  (parent 242)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 462)
  (parent 242)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1))) key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 463)
  (parent 242)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 464)
  (parent 242)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 465)
  (parent 242)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 466)
  (parent 242)
  (unrealized (0 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (l-1 l-0) (w (mul (rec eB) l-0))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 467)
  (parent 243)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-0))) key)))
  (neq ((mul e (rec eB) l-0 l-0) (one)) (e eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 468)
  (parent 243)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 469)
  (parent 243)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w))
    (mul (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 470)
  (parent 243)
  (unrealized (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 471)
  (parent 243)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul eA l-0 (rec e) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 472)
  (parent 244)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) (eA e) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0 l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 473)
  (parent 244)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1))) key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 474)
  (parent 244)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 475)
  (parent 244)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 476)
  (parent 244)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 477)
  (parent 244)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 478)
  (parent 245)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1))) key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 479)
  (parent 245)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer self-1 name) (l l-0 rndx)
    (lp expt) (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2))) key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 480)
  (parent 245)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 481)
  (parent 245)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 482)
  (parent 245)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 483)
  (parent 245)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 484)
  (parent 245)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 485)
  (parent 246)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 486)
  (parent 246)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l l-1)))
        key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 487)
  (parent 246)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 488)
  (parent 246)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 489)
  (parent 246)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0))) key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 490)
  (parent 246)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 491)
  (parent 246)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 492)
  (parent 247)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (4 0)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-0))) key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 493)
  (parent 247)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 494)
  (parent 247)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 495)
  (parent 247)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 496)
  (parent 247)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 497)
  (parent 248)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (4 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1))) key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 498)
  (parent 248)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-1))) key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 499)
  (parent 248)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (3 0)))
  (absent (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-0 l-2))) key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 500)
  (parent 248)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 501)
  (parent 248)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 502)
  (parent 248)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 503)
  (parent 248)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 504)
  (parent 248)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 505)
  (parent 253)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 506)
  (parent 253)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 507)
  (parent 253)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 508)
  (parent 253)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 509)
  (parent 254)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 510)
  (parent 254)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 511)
  (parent 254)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eB) (send eB)))
  (label 512)
  (parent 254)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec e) l-0) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 513)
  (parent 255)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec e) l-1) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 514)
  (parent 255)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 515)
  (parent 255)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 516)
  (parent 255)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 517)
  (parent 256)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 518)
  (parent 256)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 519)
  (parent 256)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 520)
  (parent 256)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 521)
  (parent 256)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 522)
  (parent 257)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 523)
  (parent 257)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 524)
  (parent 257)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 525)
  (parent 257)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 526)
  (parent 257)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 527)
  (parent 258)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 528)
  (parent 258)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 529)
  (parent 258)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 530)
  (parent 258)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 531)
  (parent 258)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 532)
  (parent 259)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 533)
  (parent 259)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 534)
  (parent 259)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 535)
  (parent 259)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 536)
  (parent 259)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 537)
  (parent 259)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 538)
  (parent 260)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (gen) (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (gen) (mul (rec eB) e e l-0)))
      (send (cat (gen) (mul (rec eB) e e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 539)
  (parent 260)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (gen) (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (gen) (mul (rec eB) e e l-1)))
      (send (cat (gen) (mul (rec eB) e e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 540)
  (parent 260)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 541)
  (parent 260)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 542)
  (parent 260)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) e)) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 543)
  (parent 261)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 544)
  (parent 261)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 545)
  (parent 261)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 546)
  (parent 261)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 547)
  (parent 261)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 548)
  (parent 262)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e e-0 l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e e (rec e-0) l-0) (one)) (e e-0) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 549)
  (parent 262)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e e (rec e-0) l-1) (one)) (e e-0) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 550)
  (parent 262)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 551)
  (parent 262)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 552)
  (parent 262)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul (rec eB) e e (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 553)
  (parent 263)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 554)
  (parent 263)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 555)
  (parent 263)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 556)
  (parent 263)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 557)
  (parent 263)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 558)
  (parent 263)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 559)
  (parent 264)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB e l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB e l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) e e (rec l) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 560)
  (parent 264)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 561)
  (parent 264)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 562)
  (parent 264)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 563)
  (parent 264)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 564)
  (parent 264)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 565)
  (parent 265)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul (rec eB) e e l-0) (one)) (e eB) ((mul e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 566)
  (parent 265)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 567)
  (parent 265)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 568)
  (parent 265)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 569)
  (parent 265)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 570)
  (parent 265)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 571)
  (parent 266)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 572)
  (parent 266)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 573)
  (parent 266)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 574)
  (parent 266)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 575)
  (parent 266)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 576)
  (parent 266)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 577)
  (parent 266)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 578)
  (parent 267)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-1)))
      (send (cat (gen) (mul eA (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 579)
  (parent 267)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-2)))
      (send (cat (gen) (mul eA (rec eB) e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 580)
  (parent 267)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 581)
  (parent 267)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 582)
  (parent 267)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 583)
  (parent 267)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep e-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul e (rec eB) e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul e (rec eB) e-0) (one)) (e eB) (e-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e-0) (w (mul (rec eB) e-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0)))))
  (label 584)
  (parent 268)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 e eB e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e e) (lp l-1) (ep (mul e-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul e (rec eB) e-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-1)))
        key)))
  (neq ((mul e (rec eB) e-0 l-1) (one)) (e eB) ((mul e-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 e eB e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) l-1)) (send (cat (exp (gen) e-0) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 585)
  (parent 268)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul e-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul e (rec eB) e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))
        key)))
  (neq ((mul e (rec eB) e-0 l-2) (one)) (e eB) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 586)
  (parent 268)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB))))))
  (label 587)
  (parent 268)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 588)
  (parent 268)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 589)
  (parent 269)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (l-0 eA e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e e-0) (lp l-1) (ep (mul eA e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA e (rec e-0) l-1) (one)) (eA e-0) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l-0 eA e e-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 590)
  (parent 269)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))
        key)))
  (neq ((mul eA e (rec e-0) l-2) (one)) (eA e-0) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 591)
  (parent 269)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 592)
  (parent 269)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 593)
  (parent 269)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 594)
  (parent 269)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA))) key)))
  (neq ((mul e eA (rec eB)) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 595)
  (parent 270)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (e eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e eA l-1))) key)))
  (neq ((mul e eA (rec eB) l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eA eB l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 596)
  (parent 270)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (e eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2))) key)))
  (neq ((mul e eA (rec eB) l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 597)
  (parent 270)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 598)
  (parent 270)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 599)
  (parent 270)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul eA (rec eB) e (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 600)
  (parent 271)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 601)
  (parent 271)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (lp expt) (l-2 rndx) (lp-0 expt)
    (eA eB e e-0 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 602)
  (parent 271)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w e-0))
    (mul eB (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 603)
  (parent 271)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 604)
  (parent 271)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 605)
  (parent 271)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 606)
  (parent 271)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 607)
  (parent 272)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-1)))
        key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB e l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 608)
  (parent 272)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 609)
  (parent 272)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 610)
  (parent 272)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 611)
  (parent 272)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 612)
  (parent 272)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 613)
  (parent 272)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 614)
  (parent 273)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-1))) key)))
  (neq ((mul eA (rec eB) e l-1) (one)) (eA eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 615)
  (parent 273)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 616)
  (parent 273)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 617)
  (parent 273)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 618)
  (parent 273)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 619)
  (parent 273)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 620)
  (parent 273)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 621)
  (parent 274)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 622)
  (parent 274)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 623)
  (parent 274)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 624)
  (parent 274)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-2))
    (mul eB (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 625)
  (parent 274)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 626)
  (parent 274)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-2))
    (mul (rec eA) eB w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 627)
  (parent 274)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 628)
  (parent 274)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 629)
  (parent 275)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l l-0)))
      (send (cat (gen) (mul eA (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 630)
  (parent 275)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 631)
  (parent 275)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 632)
  (parent 275)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 633)
  (parent 275)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 634)
  (parent 275)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 635)
  (parent 276)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (e eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l l-0))) key)))
  (neq ((mul e (rec eB) l l-0) (one)) (e eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen e eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 636)
  (parent 276)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 637)
  (parent 276)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 638)
  (parent 276)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 639)
  (parent 276)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 640)
  (parent 277)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eA l (rec e) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA l (rec e) l-0) (one)) (eA e) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l (rec e) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA l (rec e) (rec e) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 641)
  (parent 277)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 642)
  (parent 277)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 643)
  (parent 277)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 644)
  (parent 277)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 645)
  (parent 277)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 646)
  (parent 278)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l rndx) (lp expt)
    (eA eB l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-0 (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 647)
  (parent 278)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 648)
  (parent 278)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 649)
  (parent 278)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 650)
  (parent 278)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 651)
  (parent 278)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 652)
  (parent 278)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 653)
  (parent 279)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-0))) key)))
  (neq ((mul l eA (rec eB) l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 654)
  (parent 279)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 655)
  (parent 279)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 656)
  (parent 279)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 657)
  (parent 279)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 658)
  (parent 280)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-0))) key)))
  (neq ((mul eA (rec eB) l l-0) (one)) (eA eB) ((mul l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 659)
  (parent 280)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 660)
  (parent 280)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 661)
  (parent 280)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 662)
  (parent 280)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 663)
  (parent 280)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 664)
  (parent 280)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 665)
  (parent 281)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 666)
  (parent 281)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 667)
  (parent 281)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 668)
  (parent 281)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 669)
  (parent 281)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 670)
  (parent 281)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 671)
  (parent 281)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 672)
  (parent 281)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 673)
  (parent 282)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 674)
  (parent 282)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 675)
  (parent 282)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 676)
  (parent 282)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 677)
  (parent 282)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 678)
  (parent 282)
  (unrealized (0 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 679)
  (parent 283)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-0))) key)))
  (neq ((mul e (rec eB) l-0 l-0) (one)) (e eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 680)
  (parent 283)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 681)
  (parent 283)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 682)
  (parent 283)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 683)
  (parent 283)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 684)
  (parent 284)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec e) l-0 l-0) (one)) (eA e) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 685)
  (parent 284)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 686)
  (parent 284)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 687)
  (parent 284)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 688)
  (parent 284)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 689)
  (parent 284)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 690)
  (parent 285)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1 l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 691)
  (parent 285)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 692)
  (parent 285)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 693)
  (parent 285)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 694)
  (parent 285)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 695)
  (parent 285)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 696)
  (parent 285)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 697)
  (parent 286)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0 l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 698)
  (parent 286)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 699)
  (parent 286)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 700)
  (parent 286)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 701)
  (parent 286)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 702)
  (parent 286)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 703)
  (parent 286)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 704)
  (parent 287)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-0)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-0) (one)) (eA eB) ((mul l-0 l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-0)) (send (cat (exp (gen) l-0) l-0)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 705)
  (parent 287)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 706)
  (parent 287)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 707)
  (parent 287)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 708)
  (parent 287)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 709)
  (parent 288)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 710)
  (parent 288)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 711)
  (parent 288)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 712)
  (parent 288)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 713)
  (parent 288)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 714)
  (parent 288)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 715)
  (parent 288)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 716)
  (parent 288)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1)))))
  (label 717)
  (parent 289)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 718)
  (parent 289)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 719)
  (parent 289)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-2)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 720)
  (parent 289)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 721)
  (parent 289)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 722)
  (parent 289)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 723)
  (parent 289)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (l-2 l-1) (w (mul (rec eB) l-1))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1)))))
  (label 724)
  (parent 290)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 725)
  (parent 290)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1))) key)))
  (neq ((mul e (rec eB) l-1 l-1) (one)) (e eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 726)
  (parent 290)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e (rec eB) l-1 l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 727)
  (parent 290)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 728)
  (parent 290)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 729)
  (parent 290)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA l-1 (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 730)
  (parent 291)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 731)
  (parent 291)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) (eA e) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 732)
  (parent 291)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 733)
  (parent 291)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 734)
  (parent 291)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 735)
  (parent 291)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 736)
  (parent 291)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2)
      (w (mul eA (rec eB) l-2 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 737)
  (parent 292)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer self-1 name) (l l-0 rndx)
    (lp expt) (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-2)) (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 738)
  (parent 292)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 739)
  (parent 292)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 740)
  (parent 292)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-2) w e))
    (mul eB (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 741)
  (parent 292)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 742)
  (parent 292)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 743)
  (parent 292)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-2) w))
    (mul (rec eA) eB (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 744)
  (parent 292)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 745)
  (parent 293)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l-1)) (eA (exp (gen) l-0))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l l-1)))
        key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 746)
  (parent 293)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 747)
  (parent 293)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 748)
  (parent 293)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 749)
  (parent 293)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 750)
  (parent 293)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 751)
  (parent 293)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 752)
  (parent 293)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 753)
  (parent 294)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 754)
  (parent 294)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 755)
  (parent 294)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 756)
  (parent 294)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 757)
  (parent 294)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 758)
  (parent 294)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 759)
  (parent 294)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 760)
  (parent 294)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB))))))
  (label 761)
  (parent 295)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 762)
  (parent 295)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 763)
  (parent 295)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 764)
  (parent 295)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 765)
  (parent 295)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 766)
  (parent 295)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-1)
      (w (mul eA (rec eB) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 767)
  (parent 296)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 0) (0 1)) ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-2)) (eA (exp (gen) l))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 768)
  (parent 296)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 769)
  (parent 296)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 770)
  (parent 296)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 771)
  (parent 296)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w l-2))
    (mul eB (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 772)
  (parent 296)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 773)
  (parent 296)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 774)
  (parent 296)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (added-absence l-2 (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 775)
  (parent 296)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 776)
  (parent 299)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 777)
  (parent 299)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 778)
  (parent 299)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 779)
  (parent 300)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 780)
  (parent 300)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (e eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eB) (send eB)))
  (label 781)
  (parent 300)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec e) l-1) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 782)
  (parent 301)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 783)
  (parent 301)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 784)
  (parent 301)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 785)
  (parent 302)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 786)
  (parent 302)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 787)
  (parent 302)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 788)
  (parent 302)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 789)
  (parent 303)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 790)
  (parent 303)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 791)
  (parent 303)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 792)
  (parent 303)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 793)
  (parent 304)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 794)
  (parent 304)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 795)
  (parent 304)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 796)
  (parent 304)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 797)
  (parent 305)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 798)
  (parent 305)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 799)
  (parent 305)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 800)
  (parent 305)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 801)
  (parent 305)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 802)
  (parent 306)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (gen) (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (gen) (mul (rec eB) e e l-1)))
      (send (cat (gen) (mul (rec eB) e e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 803)
  (parent 306)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 804)
  (parent 306)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 805)
  (parent 306)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) e)) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) e)))
      (send (cat (exp (gen) e) (mul (rec eB) e)))))
  (label 806)
  (parent 307)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 807)
  (parent 307)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 808)
  (parent 307)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 809)
  (parent 307)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul e e (rec e-0)) (one)) (e e-0) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 810)
  (parent 308)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e e-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul e e (rec e-0) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e e (rec e-0) l-1) (one)) (e e-0) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 811)
  (parent 308)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 812)
  (parent 308)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 813)
  (parent 308)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul (rec eB) e e (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 814)
  (parent 309)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eB) e e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 815)
  (parent 309)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 816)
  (parent 309)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 817)
  (parent 309)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-absence e-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))))
  (label 818)
  (parent 309)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 819)
  (parent 310)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 820)
  (parent 310)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 821)
  (parent 310)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 822)
  (parent 310)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 823)
  (parent 310)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0))))))
  (label 824)
  (parent 311)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB e l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 825)
  (parent 311)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 826)
  (parent 311)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 827)
  (parent 311)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-absence l-0 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 828)
  (parent 311)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 829)
  (parent 312)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 830)
  (parent 312)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 831)
  (parent 312)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 832)
  (parent 312)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 833)
  (parent 312)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-absence l-1 (mul eB (rec e) (rec e) w))
    (mul eB (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 834)
  (parent 312)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e)))))
  (label 835)
  (parent 313)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul eA (rec eB) e l-2)))
      (send (cat (gen) (mul eA (rec eB) e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 836)
  (parent 313)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 837)
  (parent 313)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 838)
  (parent 313)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e)))))
  (label 839)
  (parent 313)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep e-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) key)))
  (neq ((mul e (rec eB) e-0) (one)) (e eB) (e-0 (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e-0) (w (mul (rec eB) e-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0)))))
  (label 840)
  (parent 314)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 e eB e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul e-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul e (rec eB) e-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l-1)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))
        key)))
  (neq ((mul e (rec eB) e-0 l-2) (one)) (e eB) ((mul e-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eB e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) e-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eB) e-0 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 841)
  (parent 314)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB))))))
  (label 842)
  (parent 314)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 843)
  (parent 314)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA e (rec e-0)) (one)) (eA e-0) (e (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul eA e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0))))))
  (label 844)
  (parent 315)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l rndx) (lp expt)
    (l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul eA e (rec e-0) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l-1)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e l-2)))
        key)))
  (neq ((mul eA e (rec e-0) l-2) (one)) (eA e-0) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA e e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA e (rec e-0) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eA e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 845)
  (parent 315)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-absence eA (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))))
  (label 846)
  (parent 315)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))))
  (label 847)
  (parent 315)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 848)
  (parent 315)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA))) key)))
  (neq ((mul e eA (rec eB)) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB))))))
  (label 849)
  (parent 316)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (e eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eA l-2))) key)))
  (neq ((mul e eA (rec eB) l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 850)
  (parent 316)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 851)
  (parent 316)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 852)
  (parent 316)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul eA (rec eB) e (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 853)
  (parent 317)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 self-1 name)
    (l l-0 l-1 rndx) (lp expt) (l-2 rndx) (lp-0 expt)
    (eA eB e e-0 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e-0 (exp (gen) l-2)) (e-0 (exp (gen) lp-0))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul eA (rec eB) e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 854)
  (parent 317)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w e-0))
    (mul eB (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 855)
  (parent 317)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w e-0))
    (mul (rec eA) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 856)
  (parent 317)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))))
  (label 857)
  (parent 317)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-absence e-0 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 858)
  (parent 317)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 859)
  (parent 318)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 860)
  (parent 318)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 861)
  (parent 318)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 862)
  (parent 318)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 863)
  (parent 318)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 864)
  (parent 318)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1))))))
  (label 865)
  (parent 319)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 866)
  (parent 319)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-1))
    (mul eB (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 867)
  (parent 319)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-1))
    (mul (rec eA) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 868)
  (parent 319)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))))
  (label 869)
  (parent 319)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 870)
  (parent 319)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul eA (rec eB) e (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 871)
  (parent 320)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 872)
  (parent 320)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 873)
  (parent 320)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w l-2))
    (mul eB (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 874)
  (parent 320)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w l-2))
    (mul (rec eA) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 875)
  (parent 320)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w l-2))
    (mul (rec eA) eB w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 876)
  (parent 320)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB (rec e) w))
    (mul (rec eA) eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 877)
  (parent 320)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 878)
  (parent 321)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 879)
  (parent 321)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 880)
  (parent 321)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 881)
  (parent 321)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 882)
  (parent 321)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 883)
  (parent 322)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 884)
  (parent 322)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 885)
  (parent 322)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 886)
  (parent 322)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 887)
  (parent 323)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-0 (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 888)
  (parent 323)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l) e e))
    (mul w (rec l) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 889)
  (parent 323)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 890)
  (parent 323)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l)))
    (mul (rec eA) w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 891)
  (parent 323)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 892)
  (parent 324)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 893)
  (parent 324)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 894)
  (parent 324)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 895)
  (parent 324)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 896)
  (parent 324)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 897)
  (parent 324)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 898)
  (parent 325)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 899)
  (parent 325)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 900)
  (parent 325)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 901)
  (parent 325)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 902)
  (parent 326)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 903)
  (parent 326)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 904)
  (parent 326)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 905)
  (parent 326)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 906)
  (parent 326)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 907)
  (parent 326)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 908)
  (parent 327)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 909)
  (parent 327)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 910)
  (parent 327)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 911)
  (parent 327)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 912)
  (parent 327)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 913)
  (parent 327)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 914)
  (parent 327)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 915)
  (parent 328)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-0 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 916)
  (parent 328)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 917)
  (parent 328)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 918)
  (parent 328)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 919)
  (parent 328)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e eB l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e (rec eB) l-0) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul (rec eB) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0)))))
  (label 920)
  (parent 329)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l e eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e (rec eB) l-0 l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 921)
  (parent 329)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-absence eB (mul w (rec l)))
    (mul w (rec l)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 922)
  (parent 329)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-absence l-0 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 923)
  (parent 329)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA l-0 (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e))))))
  (label 924)
  (parent 330)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-0 (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA l-0 (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-0 (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 925)
  (parent 330)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-absence eA (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 926)
  (parent 330)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 927)
  (parent 330)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w (rec l-0)))
    (mul (rec eA) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 928)
  (parent 330)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul eA (rec eB) l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 929)
  (parent 331)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 930)
  (parent 331)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w e))
    (mul eB (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 931)
  (parent 331)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w e))
    (mul (rec eA) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 932)
  (parent 331)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 933)
  (parent 331)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 934)
  (parent 331)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l))) key)))
  (neq ((mul eA (rec eB) l) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul eA (rec eB) l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0))))))
  (label 935)
  (parent 332)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l l-1))) key)))
  (neq ((mul eA (rec eB) l l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 936)
  (parent 332)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w l-0))
    (mul eB (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 937)
  (parent 332)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w l-0))
    (mul (rec eA) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 938)
  (parent 332)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w l))
    (mul (rec eA) eB w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 939)
  (parent 332)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB (rec l) w))
    (mul (rec eA) eB (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 940)
  (parent 332)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 941)
  (parent 333)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 942)
  (parent 333)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))))
  (label 943)
  (parent 333)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB))))))
  (label 944)
  (parent 333)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 945)
  (parent 334)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 946)
  (parent 334)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 947)
  (parent 334)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 948)
  (parent 334)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 949)
  (parent 334)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 950)
  (parent 334)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 951)
  (parent 334)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1)))))
  (label 952)
  (parent 335)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 953)
  (parent 335)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul eA (rec eB) l-1 l-2)))
      (send (cat (gen) (mul eA (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 954)
  (parent 335)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA)))))
  (label 955)
  (parent 335)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))))
  (label 956)
  (parent 335)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 957)
  (parent 335)
  (unrealized (0 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e (rec eB) l-1) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test
    (contracted (eB-0 eB) (l-2 l-1) (w (mul (rec eB) l-1))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1)))))
  (label 958)
  (parent 336)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1))) key)))
  (neq ((mul e (rec eB) l-1 l-1) (one)) (e eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 959)
  (parent 336)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e (rec eB) l-1 l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eB) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 960)
  (parent 336)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-absence eB (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB))))))
  (label 961)
  (parent 336)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 962)
  (parent 336)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA l-1 (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul eA l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e))))))
  (label 963)
  (parent 337)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA (rec e) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec e) l-1 l-1) (one)) (eA e) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 964)
  (parent 337)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eA l-1 (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA l-1 (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA l-1 (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 965)
  (parent 337)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-absence eA (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))))
  (label 966)
  (parent 337)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) w e e))
    (mul (rec eA) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 967)
  (parent 337)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 968)
  (parent 337)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2)
      (w (mul eA (rec eB) l-2 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 969)
  (parent 338)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 970)
  (parent 338)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer self-1 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 e l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3)))
      (send (cat (exp (gen) e) (mul eA (rec eB) l-2 (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 971)
  (parent 338)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-2) w e))
    (mul eB (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 972)
  (parent 338)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-2) w e))
    (mul (rec eA) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 973)
  (parent 338)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w e))
    (mul (rec eA) eB w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 974)
  (parent 338)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB (rec l-2) w))
    (mul (rec eA) eB (rec l-2) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))))
  (label 975)
  (parent 338)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 976)
  (parent 339)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 977)
  (parent 339)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 978)
  (parent 339)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 979)
  (parent 339)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 980)
  (parent 339)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 981)
  (parent 339)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 982)
  (parent 339)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul eA (rec eB) l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1))))))
  (label 983)
  (parent 340)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 984)
  (parent 340)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-0 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-0 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 985)
  (parent 340)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w l-1))
    (mul eB (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 986)
  (parent 340)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-0) w l-1))
    (mul (rec eA) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 987)
  (parent 340)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w l-0))
    (mul (rec eA) eB w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 988)
  (parent 340)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (added-absence l-1 (mul (rec eA) eB (rec l-0) w))
    (mul (rec eA) eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 989)
  (parent 340)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB))))))
  (label 990)
  (parent 341)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 991)
  (parent 341)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 992)
  (parent 341)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))))
  (label 993)
  (parent 341)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB))))))
  (label 994)
  (parent 341)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-1)
      (w (mul eA (rec eB) l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 995)
  (parent 342)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 996)
  (parent 342)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 997)
  (parent 342)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (absent (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 998)
  (parent 342)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w l-2))
    (mul eB (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 999)
  (parent 342)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test
    (added-absence eB (mul (rec eA) (rec l-1) w l-2))
    (mul (rec eA) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1000)
  (parent 342)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w l-1))
    (mul (rec eA) eB w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 1001)
  (parent 342)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (added-absence l-2 (mul (rec eA) eB (rec l-1) w))
    (mul (rec eA) eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 1002)
  (parent 342)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eB eA)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1003)
  (parent 344)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (uniq-gen l eA eB)
  (precur (4 0) (3 0))
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1004)
  (parent 345)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1005)
  (parent 347)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1006)
  (parent 350)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1007)
  (parent 351)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1008)
  (parent 353)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1009)
  (parent 354)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1010)
  (parent 355)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l))) (send (cat (gen) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 1011)
  (parent 357)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) w l)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l)))
      (send (cat (gen) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1012)
  (parent 358)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eA) (send eA)))
  (label 1013)
  (parent 362)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1014)
  (parent 363)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1015)
  (parent 364)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (l e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1016)
  (parent 367)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (l eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1017)
  (parent 368)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1018)
  (parent 371)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener e) (mul w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1019)
  (parent 372)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1020)
  (parent 375)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1021)
  (parent 376)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (e rndx)
    (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1022)
  (parent 379)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB rndx)
    (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e-0)
    (lp l) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1023)
  (parent 380)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1024)
  (parent 381)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (e rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w l)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (e eB) ((mul (rec e) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen e l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1025)
  (parent 384)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul eB w l)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l e e))) key)))
  (neq ((mul w l e e) (one)) (e eB) ((mul eB w l e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l)))
      (send (cat (exp (gen) e) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w e e)))
      (send (cat (exp (gen) l) (mul w e e)))) ((recv e) (send e)))
  (label 1026)
  (parent 385)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1027)
  (parent 390)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv e) (send e)))
  (label 1028)
  (parent 391)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1029)
  (parent 392)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eB e rndx)
    (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1030)
  (parent 395)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt) (eA e rndx)
    (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1031)
  (parent 396)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1032)
  (parent 397)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1033)
  (parent 400)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) lp)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1034)
  (parent 401)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l-0 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1035)
  (parent 404)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1036)
  (parent 405)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e (exp (gen) l)) (e (exp (gen) lp)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1037)
  (parent 406)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eB rndx) (w expt) (l-0 rndx)
    (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1038)
  (parent 409)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l eA rndx) (w expt) (l-0 rndx)
    (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1039)
  (parent 410)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e-0 eA))) key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1040)
  (parent 413)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1041)
  (parent 414)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (7 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-0)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e-0))) key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1042)
  (parent 415)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 peer-0 name) (l l-0 rndx)
    (lp expt) (l-1 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-1) (e e-0)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1043)
  (parent 416)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB e rndx)
    (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1044)
  (parent 419)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1045)
  (parent 420)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA eB rndx)
    (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) l-0) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) eB w l-0)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 e)))
        key)))
  (neq ((mul w l-0 e) (one)) (eA eB) ((mul (rec eA) eB w l-0 e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) (mul w e)))
      (send (cat (exp (gen) l-0) (mul w e)))) ((recv e) (send e)))
  (label 1046)
  (parent 421)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1047)
  (parent 426)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1048)
  (parent 427)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (7 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 e))) key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1049)
  (parent 428)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1050)
  (parent 429)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB l rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB (rec l) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1051)
  (parent 432)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA l rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1052)
  (parent 433)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1053)
  (parent 437)
  (unrealized (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (w expt) (l e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1054)
  (parent 441)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1055)
  (parent 443)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eB l-0 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eB l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1056)
  (parent 446)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt) (eA l-0 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l-0) w e)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1057)
  (parent 447)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer name) (l rndx) (lp expt)
    (eA eB l-0 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (6 0)) ((6 1) (4 0)))
  (absent (e (mul (rec eA) eB (rec l-0) w)) (e (exp (gen) l))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1058)
  (parent 449)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eB rndx) (w expt) (l eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB w)))
  (deflistener (cat (exp (gen) l) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l eA))) key)))
  (neq ((mul w l eA) (one)) (eA eB) ((mul eB w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB w)))
      (send (cat (exp (gen) l) (mul eB w))))
    ((recv (cat (exp (gen) l) (mul w eA)))
      (send (cat (exp (gen) l) (mul w eA)))) ((recv eA) (send eA)))
  (label 1059)
  (parent 452)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B name) (eA rndx) (w expt) (l eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l))) key)))
  (neq ((mul w l (rec eB)) (one)) (eA eB) ((mul (rec eA) w l) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w)))
      (send (cat (exp (gen) l) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l) (mul w (rec eB))))
      (send (cat (exp (gen) l) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1060)
  (parent 453)
  (unrealized (0 3) (3 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eA) (send eA)))
  (label 1061)
  (parent 457)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv eB) (send eB)))
  (label 1062)
  (parent 458)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (6 0)) ((6 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-0) (send l-0)))
  (label 1063)
  (parent 460)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1064)
  (parent 464)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1065)
  (parent 465)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1066)
  (parent 466)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 1067)
  (parent 466)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1068)
  (parent 470)
  (unrealized (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) eB))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 eB) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e l-0))) key)))
  (neq ((mul e l-0) (one)) (e eB) ((mul eB l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) eB)) (send (cat (exp (gen) l-0) eB)))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1069)
  (parent 471)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1070)
  (parent 471)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1071)
  (parent 475)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e l-0) (one)) (eA e) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1072)
  (parent 476)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1073)
  (parent 476)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1074)
  (parent 477)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1075)
  (parent 481)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1076)
  (parent 482)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA eB) ((mul (rec eA) eB e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1077)
  (parent 483)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (7 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1)))
  (label 1078)
  (parent 483)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 2) (3 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1079)
  (parent 484)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1080)
  (parent 488)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1081)
  (parent 489)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((mul l l-0) (one)) (eA eB) ((mul (rec eA) eB l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1082)
  (parent 490)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (6 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0))) key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1083)
  (parent 490)
  (unrealized (0 3) (3 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (6 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1084)
  (parent 495)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1085)
  (parent 496)
  (unrealized (0 3) (3 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1 eA))) key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1086)
  (parent 501)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1087)
  (parent 502)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (3 0)) ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0 l-1))) key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1088)
  (parent 503)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (7 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1089)
  (parent 503)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 0) (3 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-1) (send l-1)))
  (label 1090)
  (parent 504)
  (unrealized (0 3) (3 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1091)
  (parent 507)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1092)
  (parent 508)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1093)
  (parent 511)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1094)
  (parent 515)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1095)
  (parent 516)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1096)
  (parent 519)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1097)
  (parent 520)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1098)
  (parent 521)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1099)
  (parent 524)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1100)
  (parent 525)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1101)
  (parent 529)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1102)
  (parent 530)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1103)
  (parent 531)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1104)
  (parent 535)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1105)
  (parent 536)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1106)
  (parent 537)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1107)
  (parent 541)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1108)
  (parent 542)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1109)
  (parent 546)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1110)
  (parent 547)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1111)
  (parent 551)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l)) (e (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1112)
  (parent 552)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1113)
  (parent 556)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1114)
  (parent 557)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1115)
  (parent 558)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1116)
  (parent 562)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1117)
  (parent 563)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1118)
  (parent 568)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1119)
  (parent 569)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1120)
  (parent 570)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1121)
  (parent 575)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1122)
  (parent 576)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1123)
  (parent 577)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1124)
  (parent 581)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1125)
  (parent 582)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1126)
  (parent 583)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1127)
  (parent 587)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1128)
  (parent 588)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1129)
  (parent 592)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1130)
  (parent 593)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1131)
  (parent 594)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1132)
  (parent 598)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1133)
  (parent 599)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1134)
  (parent 603)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1135)
  (parent 604)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1136)
  (parent 605)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1137)
  (parent 606)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1138)
  (parent 610)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1139)
  (parent 611)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1140)
  (parent 612)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1141)
  (parent 617)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1142)
  (parent 618)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1143)
  (parent 619)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0))
    ((6 1) (1 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 1144)
  (parent 620)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1145)
  (parent 625)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1146)
  (parent 626)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1147)
  (parent 627)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1148)
  (parent 628)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1149)
  (parent 632)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1150)
  (parent 633)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1151)
  (parent 638)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1152)
  (parent 643)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1153)
  (parent 645)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1154)
  (parent 649)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1155)
  (parent 650)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1156)
  (parent 652)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1157)
  (parent 656)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1158)
  (parent 657)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1159)
  (parent 661)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1160)
  (parent 662)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w (rec l-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0)))
  (label 1161)
  (parent 664)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1162)
  (parent 669)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1163)
  (parent 670)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1164)
  (parent 672)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1165)
  (parent 676)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1166)
  (parent 677)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep l-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) key)))
  (neq (l-0 (one)) (eA eB) ((mul (rec eA) eB l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB))))
    ((recv (cat (gen) l-0)) (send (cat (gen) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1167)
  (parent 678)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0)))
  (label 1168)
  (parent 678)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1169)
  (parent 682)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) eB))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 eB) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-0))) key)))
  (neq ((mul e l-0) (one)) (e eB) ((mul eB l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) eB)) (send (cat (exp (gen) l-0) eB)))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1170)
  (parent 683)
  (unrealized (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1171)
  (parent 683)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1172)
  (parent 687)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-0))) key)))
  (neq ((mul e l-0) (one)) (eA e) ((mul (rec eA) e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1173)
  (parent 688)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1174)
  (parent 688)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1175)
  (parent 689)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1176)
  (parent 693)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1177)
  (parent 694)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA eB) ((mul (rec eA) eB e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1178)
  (parent 695)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0))
    ((6 2) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1)))
  (label 1179)
  (parent 695)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1180)
  (parent 696)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1181)
  (parent 700)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1182)
  (parent 701)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (0 1)) ((5 0) (1 1)) ((5 1) (4 0)))
  (absent (l-0 (mul (rec eA) eB l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-0))) key)))
  (neq ((mul l l-0) (one)) (eA eB) ((mul (rec eA) eB l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1183)
  (parent 702)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (6 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener l-0) (mul w l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0)))
  (label 1184)
  (parent 702)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1185)
  (parent 707)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1186)
  (parent 708)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1187)
  (parent 713)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1188)
  (parent 714)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 0) (0 1)) ((6 0) (1 1))
    ((6 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-2 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1189)
  (parent 715)
  (unrealized (0 3) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0))
    ((6 0) (5 0)) ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1190)
  (parent 715)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1191)
  (parent 716)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1192)
  (parent 721)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1193)
  (parent 722)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1194)
  (parent 723)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1195)
  (parent 723)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1196)
  (parent 728)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) eB))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 eB) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (e eB) ((mul eB l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) eB)) (send (cat (exp (gen) l-1) eB)))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1197)
  (parent 729)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1198)
  (parent 729)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1199)
  (parent 734)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA e) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1200)
  (parent 735)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1201)
  (parent 735)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l self) (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1202)
  (parent 736)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1203)
  (parent 741)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1204)
  (parent 742)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-2))) key)))
  (neq ((mul e l-2) (one)) (eA eB) ((mul (rec eA) eB e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1205)
  (parent 743)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2)))
  (label 1206)
  (parent 743)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1207)
  (parent 744)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1208)
  (parent 749)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1209)
  (parent 750)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1210)
  (parent 751)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1211)
  (parent 751)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1212)
  (parent 757)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1213)
  (parent 758)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1214)
  (parent 759)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1215)
  (parent 759)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (7 0)) ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0))
    ((6 1) (1 3)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1)))
  (label 1216)
  (parent 760)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1217)
  (parent 765)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1218)
  (parent 766)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1219)
  (parent 772)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1220)
  (parent 773)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((mul l-1 l-2) (one)) (eA eB) ((mul (rec eA) eB l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1221)
  (parent 774)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1222)
  (parent 774)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1223)
  (parent 775)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eB eA)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1224)
  (parent 777)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen l l-0 eA eB)
  (precur (5 0) (4 0))
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1225)
  (parent 778)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1226)
  (parent 780)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1227)
  (parent 783)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1228)
  (parent 784)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w e))) (send (cat (gen) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1229)
  (parent 786)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w e)))
      (send (cat (gen) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1230)
  (parent 787)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1231)
  (parent 788)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1232)
  (parent 790)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1233)
  (parent 791)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w l-0)))
      (send (cat (gen) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1234)
  (parent 794)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-0)))
      (send (cat (gen) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1235)
  (parent 795)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB w l-1)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w l-1)))
      (send (cat (gen) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1236)
  (parent 799)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w l-1)))
      (send (cat (gen) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1237)
  (parent 800)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1238)
  (parent 801)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1239)
  (parent 804)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l l-0 eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 1240)
  (parent 805)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec e))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1241)
  (parent 808)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 1242)
  (parent 809)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l) (ep (mul w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul w e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (e-0 (exp (gen) l-0)) (e-0 (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 e) ((mul w e e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 1243)
  (parent 812)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (e rndx) (w expt) (l l-0 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e-0)
    (lp l-0) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e-0 (mul (rec e) (rec e) w)) (e-0 (exp (gen) l))
    (e-0 (exp (gen) l-0)) (e (exp (gen) l-0)) (e (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (e e-0) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen e l l-0 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1244)
  (parent 813)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w e-0)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e eB) ((mul (rec e) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1245)
  (parent 816)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e-0)
    (lp l-0) (ep (mul eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0 e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB w e)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0 e-0)))
        key)))
  (neq ((mul w e e-0 e-0) (one)) (e-0 eB) ((mul eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w e)))
      (send (cat (exp (gen) e-0) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1246)
  (parent 817)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e-0 (mul eB (rec e) (rec e) w)) (e-0 (exp (gen) l-1))
    (e-0 (exp (gen) lp)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1247)
  (parent 818)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1248)
  (parent 821)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1249)
  (parent 822)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul (rec e) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-0)) (e (exp (gen) l-0))
    (e (exp (gen) l)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (e eB) ((mul (rec e) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1250)
  (parent 826)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l) (ep (mul eB w l-0 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul eB w l-0)) (e (exp (gen) l-0)) (e (exp (gen) l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 e e)))
        key)))
  (neq ((mul w l-0 e e) (one)) (e eB) ((mul eB w l-0 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0 e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-0)))
      (send (cat (exp (gen) e) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 1251)
  (parent 827)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w l-1)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (e eB) ((mul (rec e) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1252)
  (parent 832)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (e (mul eB w l-1)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 e e)))
        key)))
  (neq ((mul w l-1 e e) (one)) (e eB) ((mul eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w l-1)))
      (send (cat (exp (gen) e) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 1253)
  (parent 833)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1254)
  (parent 834)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eB e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1255)
  (parent 837)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA e rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1256)
  (parent 838)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 l-1 rndx) (lp expt)
    (eA eB rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (gen) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (gen) (mul w e))) (send (cat (gen) (mul w e))))
    ((recv e) (send e)))
  (label 1257)
  (parent 839)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e-0)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (e-0 eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1258)
  (parent 842)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eB rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul eB w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (e-0 (exp (gen) l)) (e-0 (exp (gen) lp)) (eB (exp (gen) l-1))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (e eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eB l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e-0))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eB w)))
      (send (cat (exp (gen) e-0) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1259)
  (parent 843)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (e rndx)
    (w expt) (l-0 l-1 e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e-0 e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) w e-0 e-0)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA e-0) ((mul w e-0 e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen e l-0 l-1 e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1260)
  (parent 846)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA rndx)
    (w expt) (l-0 l-1 e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) w e e)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (e-0 (exp (gen) l)) (e-0 (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA e) ((mul (rec eA) w e e e-0) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1 e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) w e e))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0)))) ((recv e-0) (send e-0)))
  (label 1261)
  (parent 847)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l rndx) (lp expt) (eA e rndx)
    (w expt) (l-0 l-1 e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e-0)
    (lp l-1) (ep (mul w (rec e-0))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l) (e e) (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e-0 (mul (rec eA) (rec e) w)) (e-0 (exp (gen) l-0))
    (e-0 (exp (gen) l-1)) (e (exp (gen) l)) (e (exp (gen) lp))
    (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e-0)) (one)) (eA e-0) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA e l-0 l-1 e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 1262)
  (parent 848)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eB rndx) (w expt)
    (l-1 rndx) (lp expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1263)
  (parent 851)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 eA rndx) (w expt)
    (l-1 rndx) (lp expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) w)))
      (send (cat (exp (gen) e) (mul (rec eA) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1264)
  (parent 852)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e-0 eA)))
        key)))
  (neq ((mul w e-0 eA) (one)) (eA eB) ((mul eB w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e e-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eB (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eA)))
      (send (cat (exp (gen) e-0) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 1265)
  (parent 855)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w e-0)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) key)))
  (neq ((mul w e-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e e-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eB))))
      (send (cat (exp (gen) e-0) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 1266)
  (parent 856)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB rndx) (w expt) (e e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e e-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e-0)
    (lp lp))
  (deflistener (cat (exp (gen) e) (mul w e-0)))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB w e)) (e (exp (gen) l-2))
    (e (exp (gen) lp-0)) (e-0 (exp (gen) l-1)) (e-0 (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e-0)))
        key)))
  (neq ((mul w e e-0) (one)) (eA eB) ((mul (rec eA) eB w e e-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e e-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB w e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul w e-0)))
      (send (cat (exp (gen) e) (mul w e-0))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e))) ((recv e-0) (send e-0)))
  (label 1267)
  (parent 857)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 peer-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 rndx) (lp-0 expt) (eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (l l-2) (e e-0)
    (lp lp-0))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e-0 (mul (rec eA) eB (rec e) w)) (e-0 (exp (gen) l-2))
    (e-0 (exp (gen) lp-0)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e e-0)
  (operation nonce-test (added-listener e-0) (mul w (rec e-0)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp-0) peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv e-0) (send e-0)))
  (label 1268)
  (parent 858)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1269)
  (parent 861)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1270)
  (parent 862)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1271)
  (parent 863)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1272)
  (parent 867)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1273)
  (parent 868)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-1)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 e)))
        key)))
  (neq ((mul w l-1 e) (one)) (eA eB) ((mul (rec eA) eB w l-1 e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w e)))
      (send (cat (exp (gen) l-1) (mul w e)))) ((recv e) (send e)))
  (label 1274)
  (parent 869)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eB (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1275)
  (parent 874)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1276)
  (parent 875)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2 e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w e)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB w l-2)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 e)))
        key)))
  (neq ((mul w l-2 e) (one)) (eA eB) ((mul (rec eA) eB w l-2 e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2 e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w e)))
      (send (cat (exp (gen) l-2) (mul w e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv e) (send e)))
  (label 1277)
  (parent 876)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1278)
  (parent 877)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1279)
  (parent 880)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1280)
  (parent 881)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l-0))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1281)
  (parent 885)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul w (rec l) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l) e e)))
      (send (cat (exp (gen) l) (mul w (rec l) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1282)
  (parent 889)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) w (rec l))))
      (send (cat (exp (gen) l) (mul (rec eA) w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1283)
  (parent 891)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1284)
  (parent 894)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1285)
  (parent 895)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1286)
  (parent 897)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1287)
  (parent 900)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1288)
  (parent 901)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1289)
  (parent 904)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1290)
  (parent 905)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1291)
  (parent 911)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1292)
  (parent 912)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1293)
  (parent 914)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1294)
  (parent 917)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1295)
  (parent 918)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul w (rec l))))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul w (rec l))) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul w (rec l))))
      (send (cat (exp (gen) l) (mul w (rec l)))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1296)
  (parent 922)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (w expt) (l l-0 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul w (rec l-0) e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eA))) key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1297)
  (parent 926)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w (rec l-0))) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1298)
  (parent 928)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eB l-1 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1299)
  (parent 931)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA l-1 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-1) w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1300)
  (parent 932)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer name) (l l-0 rndx) (lp expt)
    (eA eB l-1 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (7 0))
    ((7 1) (5 0)))
  (absent (e (mul (rec eA) eB (rec l-1) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1301)
  (parent 934)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eB (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1302)
  (parent 937)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1303)
  (parent 938)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w eA)))
      (send (cat (exp (gen) l-0) (mul w eA)))) ((recv eA) (send eA)))
  (label 1304)
  (parent 943)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1305)
  (parent 944)
  (unrealized (0 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 1306)
  (parent 948)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 1307)
  (parent 949)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 1308)
  (parent 951)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eA))) (send (cat (gen) (mul w eA))))
    ((recv eA) (send eA)))
  (label 1309)
  (parent 955)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eB))))
      (send (cat (gen) (mul w (rec eB))))) ((recv eB) (send eB)))
  (label 1310)
  (parent 956)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep l-1))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) key)))
  (neq (l-1 (one)) (eA eB) ((mul (rec eA) eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1311)
  (parent 957)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 1312)
  (parent 957)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e e)
    (lp l-1) (ep w))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l-0) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec l) w)) (e (exp (gen) l-0)) (e (exp (gen) l-1))
    (eB (exp (gen) l-1)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (e eB) (w (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) w)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1313)
  (parent 961)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) eB))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 eB) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-1))) key)))
  (neq ((mul e l-1) (one)) (e eB) ((mul eB l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) eB)) (send (cat (exp (gen) l-1) eB)))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1314)
  (parent 962)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1315)
  (parent 962)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l rndx) (w expt)
    (l-0 l-1 e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul w e e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul (rec l) w e e)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e e eA)))
        key)))
  (neq ((mul w e eA) (one)) (eA e) ((mul w e e) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA)))) ((recv eA) (send eA)))
  (label 1316)
  (parent 966)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) e e)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul e l-1) (one)) (eA e) ((mul (rec eA) e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1317)
  (parent 967)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1318)
  (parent 967)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA l rndx) (w expt)
    (l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e e)
    (lp l-1) (ep (mul w (rec e))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) (rec l) w)) (e (exp (gen) l-0))
    (e (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec e)) (one)) (eA e) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 1319)
  (parent 968)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eA)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e eA))) key)))
  (neq ((mul w e eA) (one)) (eA eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 e eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eB (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eA)))
      (send (cat (exp (gen) e) (mul w eA))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eA) (send eA)))
  (label 1320)
  (parent 972)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eB))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-2) w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) key)))
  (neq ((mul w e (rec eB)) (one)) (eA eB) ((mul (rec eA) w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 e eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eB))))
      (send (cat (exp (gen) e) (mul w (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv eB) (send eB)))
  (label 1321)
  (parent 973)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e l-2))) key)))
  (neq ((mul e l-2) (one)) (eA eB) ((mul (rec eA) eB e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1322)
  (parent 974)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2)))
  (label 1323)
  (parent 974)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 peer name) (l l-0 l-1 rndx)
    (lp expt) (eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 2) (4 0)) ((7 2) (8 0)) ((8 1) (6 0)))
  (absent (e (mul (rec eA) eB (rec l-2) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec e)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB (rec l-2) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv e) (send e)))
  (label 1324)
  (parent 975)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1325)
  (parent 979)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l-1))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 B) (l-0 self) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1326)
  (parent 980)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1327)
  (parent 981)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 B) (l-1 self) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1328)
  (parent 981)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1329)
  (parent 986)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1330)
  (parent 987)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1)))
        key)))
  (neq ((mul l-0 l-1) (one)) (eA eB) ((mul (rec eA) eB l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1331)
  (parent 988)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 D) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 1332)
  (parent 988)
  (unrealized (0 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eA)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eA)))
      (send (cat (exp (gen) l-1) (mul w eA)))) ((recv eA) (send eA)))
  (label 1333)
  (parent 993)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eB))))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec eB)))))
    ((recv eB) (send eB)))
  (label 1334)
  (parent 994)
  (unrealized (0 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eA)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2 eA)))
        key)))
  (neq ((mul w l-2 eA) (one)) (eA eB) ((mul eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eA)))
      (send (cat (exp (gen) l-2) (mul w eA))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 1335)
  (parent 999)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eB))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w l-2)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) key)))
  (neq ((mul w l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec eB)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec eB)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 1336)
  (parent 1000)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2)))
        key)))
  (neq ((mul l-1 l-2) (one)) (eA eB) ((mul (rec eA) eB l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1337)
  (parent 1001)
  (unrealized (0 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1338)
  (parent 1001)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 1339)
  (parent 1002)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1340)
  (parent 1015)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1341)
  (parent 1029)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1342)
  (parent 1050)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1343)
  (parent 1063)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1344)
  (parent 1067)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1345)
  (parent 1070)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1346)
  (parent 1073)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1347)
  (parent 1078)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0))) key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1348)
  (parent 1083)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1349)
  (parent 1089)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1350)
  (parent 1090)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1351)
  (parent 1103)
  (unrealized (0 3) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1352)
  (parent 1106)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1353)
  (parent 1120)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1354)
  (parent 1123)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1355)
  (parent 1144)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1356)
  (parent 1148)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB (rec l) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1357)
  (parent 1161)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1358)
  (parent 1164)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1359)
  (parent 1168)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1360)
  (parent 1171)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) key)))
  (neq ((mul w e l-0) (one)) (eA e) ((mul (rec eA) w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l e l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1361)
  (parent 1174)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (eA eB) ((mul (rec eA) eB w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1362)
  (parent 1179)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0)))
        key)))
  (neq ((mul w l l-0) (one)) (eA eB) ((mul (rec eA) eB w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 3 7 ltx-gen 2) l-0 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1363)
  (parent 1184)
  (unrealized (0 3) (3 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1364)
  (parent 1190)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1365)
  (parent 1191)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1366)
  (parent 1195)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1367)
  (parent 1198)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1368)
  (parent 1201)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1369)
  (parent 1206)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1370)
  (parent 1211)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1371)
  (parent 1215)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1372)
  (parent 1216)
  (unrealized (0 3) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1373)
  (parent 1222)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1374)
  (parent 1223)
  (unrealized (0 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) eB w)))
      (send (cat (gen) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1375)
  (parent 1238)
  (unrealized (0 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB e rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB (rec e) (rec e) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (e eB) ((mul eB (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1376)
  (parent 1254)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec e) w)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1377)
  (parent 1278)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1378)
  (parent 1293)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep w))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1379)
  (parent 1308)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1380)
  (parent 1312)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1381)
  (parent 1315)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) key)))
  (neq ((mul w e l-1) (one)) (eA e) ((mul (rec eA) w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1382)
  (parent 1318)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2)))
        key)))
  (neq ((mul w e l-2) (one)) (eA eB) ((mul (rec eA) eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB e l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1383)
  (parent 1323)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1384)
  (parent 1328)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1)))
        key)))
  (neq ((mul w l-0 l-1) (one)) (eA eB)
    ((mul (rec eA) eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1385)
  (parent 1332)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2)))
        key)))
  (neq ((mul w l-1 l-2) (one)) (eA eB)
    ((mul (rec eA) eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA))
      (recv (exp (gen) (mul (rec eA) eB w l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1386)
  (parent 1338)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 rndx)
    (w expt) (l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep w))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) key)))
  (neq (w (one)) (eA eB) ((mul (rec eA) eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1387)
  (parent 1339)
  (unrealized (0 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1388)
  (parent 1340)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1389)
  (parent 1340)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1390)
  (parent 1340)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1391)
  (parent 1341)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1392)
  (parent 1341)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1393)
  (parent 1341)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1394)
  (parent 1341)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep e))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1395)
  (parent 1342)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1396)
  (parent 1342)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1397)
  (parent 1342)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1398)
  (parent 1342)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1399)
  (parent 1342)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1400)
  (parent 1343)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1401)
  (parent 1343)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w))
    (mul eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1402)
  (parent 1343)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1403)
  (parent 1343)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l-0)))
        key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1404)
  (parent 1343)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec eB))))
      (send (cat (gen) (mul l-0 eA (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1405)
  (parent 1344)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec eB) l-1)))
      (send (cat (gen) (mul l-0 eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1406)
  (parent 1344)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1407)
  (parent 1344)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1408)
  (parent 1344)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1409)
  (parent 1345)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0 (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e l-0 (rec eB) l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1410)
  (parent 1345)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1411)
  (parent 1345)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (3 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-0))) key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener eB) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1412)
  (parent 1345)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1413)
  (parent 1346)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul l-0 eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 eA (rec e) l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1414)
  (parent 1346)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0 eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1415)
  (parent 1346)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1416)
  (parent 1346)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1417)
  (parent 1347)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-1 eA l-2))) key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1418)
  (parent 1347)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1419)
  (parent 1347)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1420)
  (parent 1347)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1421)
  (parent 1347)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep l))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul l eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA))) key)))
  (neq ((mul l eA (rec eB)) (one)) (eA eB) (l (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1422)
  (parent 1348)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul l eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul l eA l-1)))
        key)))
  (neq ((mul l eA (rec eB) l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1423)
  (parent 1348)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1424)
  (parent 1348)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1425)
  (parent 1348)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l-0) (e eB)
    (lp l-0) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1426)
  (parent 1348)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1427)
  (parent 1349)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-1))) key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1428)
  (parent 1349)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2))) key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1429)
  (parent 1349)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1430)
  (parent 1349)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1431)
  (parent 1349)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1432)
  (parent 1349)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1433)
  (parent 1350)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-1))) key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1434)
  (parent 1350)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eA l-1 l-2))) key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1435)
  (parent 1350)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1436)
  (parent 1350)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1437)
  (parent 1350)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1438)
  (parent 1350)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1439)
  (parent 1351)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1440)
  (parent 1351)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1441)
  (parent 1351)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1442)
  (parent 1352)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1443)
  (parent 1352)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1444)
  (parent 1352)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1445)
  (parent 1352)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1446)
  (parent 1353)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1447)
  (parent 1353)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1448)
  (parent 1353)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1449)
  (parent 1353)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1450)
  (parent 1354)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul (rec eB) e e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e l-1))) key)))
  (neq ((mul (rec eB) e e l-1) (one)) (e eB) ((mul e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eB) e e l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1451)
  (parent 1354)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1452)
  (parent 1354)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1453)
  (parent 1354)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1454)
  (parent 1354)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep e))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1455)
  (parent 1355)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1456)
  (parent 1355)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1457)
  (parent 1355)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1458)
  (parent 1355)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1459)
  (parent 1355)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1460)
  (parent 1356)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) e l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-2)) (eA (exp (gen) l))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA e l-2))) key)))
  (neq ((mul eA (rec eB) e l-2) (one)) (eA eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1461)
  (parent 1356)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1462)
  (parent 1356)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1463)
  (parent 1356)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1464)
  (parent 1356)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1465)
  (parent 1356)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) key)))
  (neq ((mul eA (rec eB) l-0) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul eA (rec eB) l-0)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1466)
  (parent 1357)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1467)
  (parent 1357)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l) w))
    (mul eB (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1468)
  (parent 1357)
  (unrealized (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l) w))
    (mul (rec eA) (rec l) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1469)
  (parent 1357)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l) (mul w (rec l) l-0)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l) (mul w (rec l) l-0)))
      (send (cat (exp (gen) l) (mul w (rec l) l-0))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1470)
  (parent 1357)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1471)
  (parent 1358)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul eA (rec eB) l-0 l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-0 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-0 l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1)))
      (send (cat (exp (gen) l) (mul (rec l) eA (rec eB) l-0 l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1472)
  (parent 1358)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1473)
  (parent 1358)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1474)
  (parent 1358)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1475)
  (parent 1358)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1476)
  (parent 1358)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul l-0 eA (rec eB))))
      (send (cat (gen) (mul l-0 eA (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1477)
  (parent 1359)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec eB) l-1)))
      (send (cat (gen) (mul l-0 eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1478)
  (parent 1359)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1479)
  (parent 1359)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1480)
  (parent 1359)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1481)
  (parent 1360)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0 (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e l-0 (rec eB) l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1482)
  (parent 1360)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-absence eB w) w (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1483)
  (parent 1360)
  (unrealized (1 3) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (3 0)))
  (absent (l-0 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-0)))
        key)))
  (neq ((mul w e l-0) (one)) (e eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l e l-0)
  (operation nonce-test (added-listener eB) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1484)
  (parent 1360)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l l-0 eA e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec e)) (one)) (eA e) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1485)
  (parent 1361)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul l-0 eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1486)
  (parent 1361)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1487)
  (parent 1361)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1488)
  (parent 1361)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (l-1 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1489)
  (parent 1362)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1490)
  (parent 1362)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1491)
  (parent 1362)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1492)
  (parent 1362)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1493)
  (parent 1362)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA eB l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep l))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul l eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA))) key)))
  (neq ((mul l eA (rec eB)) (one)) (eA eB) (l (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-0)))
      (l-1 l-0)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0))))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1494)
  (parent 1363)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul l eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l eA l-1))) key)))
  (neq ((mul l eA (rec eB) l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1495)
  (parent 1363)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-absence eA (mul eB w l)) (mul eB w l)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1496)
  (parent 1363)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l))
    (mul (rec eA) w l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1497)
  (parent 1363)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) eB w l)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l-0) (e eB)
    (lp l) (ep (mul w l)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w l (rec l-0))))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0)))
  (absent (l-0 (mul (rec eA) eB w)) (l (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l self) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) key)))
  (neq ((mul w l) (one)) (eA eB) ((mul (rec eA) eB w l) (one)))
  (non-orig (privk B) (privk self) l-0)
  (precur (4 0) (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-absence l-0 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w l (rec l-0)))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l)))
  (label 1498)
  (parent 1363)
  (unrealized (0 3) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1499)
  (parent 1364)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1500)
  (parent 1364)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1501)
  (parent 1364)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1502)
  (parent 1364)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1503)
  (parent 1364)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1504)
  (parent 1364)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1505)
  (parent 1365)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1506)
  (parent 1365)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1507)
  (parent 1365)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1508)
  (parent 1365)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1509)
  (parent 1365)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1510)
  (parent 1365)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec eB))))
      (send (cat (gen) (mul l-1 eA (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1511)
  (parent 1366)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul l-0 eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul l-0 eA (rec eB) l-1)))
      (send (cat (gen) (mul l-0 eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1512)
  (parent 1366)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec eB) l-2)))
      (send (cat (gen) (mul l-1 eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1513)
  (parent 1366)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1514)
  (parent 1366)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1515)
  (parent 1366)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1516)
  (parent 1367)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l e l-0 eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul e l-0 (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e l-0 l-1))) key)))
  (neq ((mul e l-0 (rec eB) l-1) (one)) (e eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB) l-1)))
      (send (cat (exp (gen) e) (mul l-0 (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1517)
  (parent 1367)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e l-1 eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1 (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e l-1 (rec eB) l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1518)
  (parent 1367)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1519)
  (parent 1367)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1520)
  (parent 1367)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1521)
  (parent 1368)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e e)
    (lp l-1) (ep (mul l-0 eA (rec e) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (e (exp (gen) l)) (e (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec e) l-1) (one)) (eA e) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 eA (rec e) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul l-0 eA (rec e) (rec e) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1522)
  (parent 1368)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1523)
  (parent 1368)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1524)
  (parent 1368)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1525)
  (parent 1368)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-2 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA))) key)))
  (neq ((mul l-2 eA (rec eB)) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1526)
  (parent 1369)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 self-1 name) (l l-0 rndx)
    (lp expt) (l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-1 l-2) (e (exp (gen) l-0)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-2)) (eA (exp (gen) l))
    (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-1 eA eB e l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1527)
  (parent 1369)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-2 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))
        key)))
  (neq ((mul l-2 eA (rec eB) l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1528)
  (parent 1369)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1529)
  (parent 1369)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1530)
  (parent 1369)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1531)
  (parent 1369)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1532)
  (parent 1370)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eA) (lp l-1) (ep (mul l l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l-0)
    (e eB) (lp l-1) (ep (mul l eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
  (deflistener l)
  (defstrand ltx-gen 2 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l l-1) (eB (exp (gen) l-0)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-0)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l self) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul l eA l-1)))
        key)))
  (neq ((mul l eA (rec eB) l-1) (one)) (eA eB) ((mul l l-1) (one)))
  (non-orig (privk B) (privk self-0) l-0)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul l eA (rec eB) (rec l-0) l-1))))
    ((recv l) (send l))
    ((send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1533)
  (parent 1370)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1534)
  (parent 1370)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1535)
  (parent 1370)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1536)
  (parent 1370)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1537)
  (parent 1370)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1538)
  (parent 1371)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-1)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-1) (one)) (eA eB) ((mul l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 3 8 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1539)
  (parent 1371)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1540)
  (parent 1371)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1541)
  (parent 1371)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1542)
  (parent 1371)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1543)
  (parent 1371)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1544)
  (parent 1372)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (3 0)))
  (absent (l-0 l-1) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-1)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-1) (one)) (eA eB) ((mul l-1 l-1) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 4 8 ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1545)
  (parent 1372)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1546)
  (parent 1372)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1547)
  (parent 1372)
  (unrealized (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1548)
  (parent 1372)
  (unrealized (0 3) (1 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 1549)
  (parent 1372)
  (unrealized (0 3) (3 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1550)
  (parent 1373)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1551)
  (parent 1373)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1552)
  (parent 1373)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1553)
  (parent 1373)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1554)
  (parent 1373)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1555)
  (parent 1373)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1556)
  (parent 1373)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2) (w (mul eA (rec eB) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1557)
  (parent 1374)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (3 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (5 0) (3 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1558)
  (parent 1374)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1559)
  (parent 1374)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1560)
  (parent 1374)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1561)
  (parent 1374)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1562)
  (parent 1374)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1563)
  (parent 1374)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1564)
  (parent 1375)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1565)
  (parent 1375)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1566)
  (parent 1375)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) key)))
  (neq ((mul (rec eB) e e) (one)) (e eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (e-0 e) (w (mul (rec eB) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1567)
  (parent 1376)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eB e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul (rec eB) e e l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) key)))
  (neq ((mul (rec eB) e e l-2) (one)) (e eB) ((mul e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eB) e e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1568)
  (parent 1376)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1569)
  (parent 1376)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-absence e (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1570)
  (parent 1376)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep e))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) key)))
  (neq ((mul eA (rec eB) e) (one)) (eA eB) (e (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul eA (rec eB) e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1571)
  (parent 1377)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul e l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) e l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-3))) key)))
  (neq ((mul eA (rec eB) e l-3) (one)) (eA eB) ((mul e l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) eA (rec eB) e l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1572)
  (parent 1377)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec e) w))
    (mul eB (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1573)
  (parent 1377)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec e) w))
    (mul (rec eA) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1574)
  (parent 1377)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1575)
  (parent 1377)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1576)
  (parent 1378)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1577)
  (parent 1378)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1578)
  (parent 1378)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1579)
  (parent 1378)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1580)
  (parent 1378)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) key)))
  (neq ((mul eA (rec eB) l-1) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul eA (rec eB) l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1581)
  (parent 1379)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul eA (rec eB) l-1 l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-1 l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) eA (rec eB) l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1582)
  (parent 1379)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-0) w))
    (mul eB (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1583)
  (parent 1379)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-0) w))
    (mul (rec eA) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1584)
  (parent 1379)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1585)
  (parent 1379)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 eA (rec eB))))
      (send (cat (gen) (mul l-1 eA (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1586)
  (parent 1380)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 eA (rec eB) l-2)))
      (send (cat (gen) (mul l-1 eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1587)
  (parent 1380)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1588)
  (parent 1380)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1589)
  (parent 1380)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (contracted (eB-0 eB) (w (rec eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1590)
  (parent 1381)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 e l-1 eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1 (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) key)))
  (neq ((mul e l-1 (rec eB) l-2) (one)) (e eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1591)
  (parent 1381)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1592)
  (parent 1381)
  (unrealized (1 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eB rndx) (w expt)
    (l l-0 e l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (5 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eB w)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-1)))
        key)))
  (neq ((mul w e l-1) (one)) (e eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation nonce-test (added-listener eB) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1593)
  (parent 1381)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 l-1 eA e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec e)) (one)) (eA e) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul eA (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1594)
  (parent 1382)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul l-1 eA (rec e) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec e) l-2) (one)) (eA e) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-1 eA (rec e) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eA (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1595)
  (parent 1382)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1596)
  (parent 1382)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1597)
  (parent 1382)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-2 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA))) key)))
  (neq ((mul l-2 eA (rec eB)) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e))))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1598)
  (parent 1383)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 self-1 name) (l l-0 l-1 rndx)
    (lp expt) (l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-2 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-2 l-3) (e (exp (gen) l-1)) (e (exp (gen) lp))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eA l-3)))
        key)))
  (neq ((mul l-2 eA (rec eB) l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB e l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 eA (rec eB) (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1599)
  (parent 1383)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w e)) (mul eB w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1600)
  (parent 1383)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w e))
    (mul (rec eA) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1601)
  (parent 1383)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-absence e (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1602)
  (parent 1383)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1603)
  (parent 1384)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l-1)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1604)
  (parent 1384)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1605)
  (parent 1384)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1606)
  (parent 1384)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-1) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l-1)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1607)
  (parent 1384)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1608)
  (parent 1385)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l-1)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 eA (rec eB) (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1609)
  (parent 1385)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w l-0)) (mul eB w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1610)
  (parent 1385)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-0))
    (mul (rec eA) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 1611)
  (parent 1385)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-1) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l-1))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 1612)
  (parent 1385)
  (unrealized (0 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB) (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1613)
  (parent 1386)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-2)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-2) (one)) (eA eB) ((mul l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1614)
  (parent 1386)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 eA (rec eB) (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1615)
  (parent 1386)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w l-1)) (mul eB w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1616)
  (parent 1386)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w l-1))
    (mul (rec eA) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1617)
  (parent 1386)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1618)
  (parent 1386)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-2))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) key)))
  (neq ((mul eA (rec eB) l-2) (one)) (eA eB) (l-2 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2) (w (mul eA (rec eB) l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1619)
  (parent 1387)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-2)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-2) (one)) (eA eB) ((mul l-2 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1620)
  (parent 1387)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-2 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul eA (rec eB) l-2 l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2 l-3)))
        key)))
  (neq ((mul eA (rec eB) l-2 l-3) (one)) (eA eB) ((mul l-2 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2 l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) eA (rec eB) l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1621)
  (parent 1387)
  (unrealized (1 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB (rec l-1) w))
    (mul eB (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1622)
  (parent 1387)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) (rec l-1) w))
    (mul (rec eA) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1623)
  (parent 1387)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul (rec eA) eB w))
    (mul (rec eA) eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1624)
  (parent 1387)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1625)
  (parent 1389)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1626)
  (parent 1390)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1627)
  (parent 1393)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1628)
  (parent 1394)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1629)
  (parent 1397)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1630)
  (parent 1398)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1631)
  (parent 1399)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1632)
  (parent 1402)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1633)
  (parent 1403)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1634)
  (parent 1407)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1635)
  (parent 1408)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1636)
  (parent 1409)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1637)
  (parent 1411)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0 eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1638)
  (parent 1415)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e e)
    (lp l) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-0 (rec e)))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1639)
  (parent 1416)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1640)
  (parent 1419)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1641)
  (parent 1420)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (8 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1642)
  (parent 1421)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1643)
  (parent 1424)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1644)
  (parent 1425)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1645)
  (parent 1430)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1646)
  (parent 1431)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (8 0)) ((6 1) (4 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1647)
  (parent 1432)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1648)
  (parent 1436)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1649)
  (parent 1437)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (3 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1650)
  (parent 1438)
  (seen 1739)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1651)
  (parent 1440)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1652)
  (parent 1441)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1653)
  (parent 1444)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1654)
  (parent 1445)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-0 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1655)
  (parent 1448)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (e (mul eB w)) (l-0 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (added-listener e) (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1656)
  (parent 1449)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1657)
  (parent 1453)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1658)
  (parent 1454)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec e) w)) (l-1 (mul eB (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1659)
  (parent 1457)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA e rndx) (w expt) (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-1 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1660)
  (parent 1458)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 2) (3 0))
    ((4 2) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1661)
  (parent 1459)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1662)
  (parent 1463)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1663)
  (parent 1464)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec l-2) e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1664)
  (parent 1465)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul eB (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB (rec l) w)) (l-0 (mul eB (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-0) eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul eB (rec l) w)))
      (send (cat (exp (gen) l) (mul eB (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) eA)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1665)
  (parent 1468)
  (unrealized (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) (rec l) w)) (l-0 (mul (rec eA) (rec l) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-0) (rec eB))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) (mul (rec eA) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eA) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1666)
  (parent 1469)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1667)
  (parent 1474)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1668)
  (parent 1475)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (gen) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB w)) (l-0 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (gen) (mul w l-0 eA)))
      (send (cat (gen) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1669)
  (parent 1479)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-0 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-0 (rec eB))))
      (send (cat (gen) (mul w l-0 (rec eB))))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1670)
  (parent 1480)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (l-0 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) key)))
  (neq ((mul e l-0 (rec eB)) (one)) (e eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1671)
  (parent 1481)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w e l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB w) (l-0 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-0))) key)))
  (neq ((mul w e l-0 (rec eB)) (one)) (e eB) ((mul w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1672)
  (parent 1483)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (w expt) (l e l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w e l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul w e e)) (l-0 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0 eA)))
        key)))
  (neq ((mul w e l-0 eA) (one)) (eA e) ((mul w e e l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l e l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0 eA)))
      (send (cat (exp (gen) e) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1673)
  (parent 1487)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e e)
    (lp l-0) (ep (mul w l-0 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (e (mul (rec eA) w)) (l-0 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec e)) (one)) (eA e) ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (added-listener e) (mul w l-0 (rec e) (rec e))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-0 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-0 (rec e) (rec e)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv e) (send e)))
  (label 1674)
  (parent 1488)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eB rndx) (w expt) (e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w e)) (l-1 (mul eB w e)) (e (exp (gen) l-0))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA eB) ((mul eB w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w e)))
      (send (cat (exp (gen) l-1) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1675)
  (parent 1491)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA rndx) (w expt) (e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w e l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w e)) (l-1 (mul (rec eA) w e))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1676)
  (parent 1492)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self peer self-0 name) (l l-0 rndx) (lp expt)
    (eA eB rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener e)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 2) (3 0)) ((5 2) (8 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (e (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (e (exp (gen) l-0)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv e) (send e)))
  (label 1677)
  (parent 1493)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eB rndx) (w expt) (l l-0 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (7 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eA (mul eB w l)) (l-0 (mul eB w l)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l l-0 eA)))
        key)))
  (neq ((mul w l l-0 eA) (one)) (eA eB) ((mul eB w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eB l l-0 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w l)))
      (send (cat (exp (gen) l-0) (mul eB w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 eA)))
      (send (cat (exp (gen) l) (mul w l-0 eA)))) ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eA) (send eA)))
  (label 1678)
  (parent 1496)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self name) (eA rndx) (w expt) (l l-0 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0 (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (7 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)))
  (absent (eB (mul (rec eA) w l)) (l-0 (mul (rec eA) w l))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l l-0))) key)))
  (neq ((mul w l l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen eA l l-0 eB)
  (operation nonce-test (added-listener eB) (mul w l-0 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w l)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0 (rec eB))))
      (send (cat (exp (gen) l) (mul w l-0 (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv eB) (send eB)))
  (label 1679)
  (parent 1497)
  (unrealized (0 3) (3 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1680)
  (parent 1502)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (4 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1681)
  (parent 1503)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 0) (3 0)) ((5 0) (8 0)) ((6 1) (4 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1682)
  (parent 1504)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1683)
  (parent 1508)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1684)
  (parent 1509)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (8 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 1685)
  (parent 1510)
  (seen 1740)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1686)
  (parent 1514)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1687)
  (parent 1515)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1688)
  (parent 1516)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1689)
  (parent 1519)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1690)
  (parent 1524)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1691)
  (parent 1525)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1692)
  (parent 1529)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1693)
  (parent 1530)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1694)
  (parent 1531)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1695)
  (parent 1535)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1696)
  (parent 1536)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1697)
  (parent 1541)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1698)
  (parent 1542)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((3 0) (8 0)) ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0))
    ((7 0) (4 0)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w l-0 (rec l-1))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1)))
  (label 1699)
  (parent 1543)
  (unrealized (0 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (8 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1700)
  (parent 1547)
  (unrealized (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (8 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (5 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1701)
  (parent 1548)
  (unrealized (0 3) (3 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (deflistener l-1)
  (precedes ((0 2) (3 0)) ((0 2) (5 0)) ((1 2) (3 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 0) (3 0))
    ((4 0) (8 0)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (0 1))
    ((7 0) (1 1)) ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-0 self-0) (l-1 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (5 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-listener l-1) (mul w (rec l-0) l-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)) ((recv l-1) (send l-1)))
  (label 1702)
  (parent 1549)
  (seen 1741)
  (unrealized (0 3) (3 0) (8 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1703)
  (parent 1554)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1704)
  (parent 1555)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1705)
  (parent 1556)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1706)
  (parent 1561)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1707)
  (parent 1562)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1708)
  (parent 1563)
  (seen 1742)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eB w))) (send (cat (gen) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1709)
  (parent 1565)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eA) w)))
      (send (cat (gen) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1710)
  (parent 1566)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul (rec e) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w))
    (e (exp (gen) l)) (e (exp (gen) l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (e eB) ((mul (rec e) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1711)
  (parent 1569)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eB w)) (l-1 (mul eB w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e e))) key)))
  (neq ((mul w e e) (one)) (e eB) ((mul eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w e))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB w)))
      (send (cat (exp (gen) e) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1712)
  (parent 1570)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec e) w)) (l-2 (mul eB (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eB (rec e) w)))
      (send (cat (exp (gen) e) (mul eB (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1713)
  (parent 1573)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec e) w)) (l-2 (mul (rec eA) (rec e) w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eA) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1714)
  (parent 1574)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w e)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB w)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 2) (4 0)) ((5 2) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) key)))
  (neq ((mul w e) (one)) (eA eB) ((mul (rec eA) eB w e) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w (rec l-2) e) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB w)))
      (send (cat (exp (gen) e) (mul (rec eA) eB w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1715)
  (parent 1575)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1716)
  (parent 1578)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1717)
  (parent 1579)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB (rec l-0) w)) (l-1 (mul eB (rec l-0) w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eB (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eB (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1718)
  (parent 1583)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) (rec l-0) w))
    (l-1 (mul (rec eA) (rec l-0) w)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) (rec eB))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1719)
  (parent 1584)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (gen) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (gen) (mul w l-1 eA)))
      (send (cat (gen) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1720)
  (parent 1588)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (gen) (mul w l-1 (rec eB))))
      (send (cat (gen) (mul w l-1 (rec eB))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1721)
  (parent 1589)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) key)))
  (neq ((mul e l-1 (rec eB)) (one)) (e eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1722)
  (parent 1590)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e e)
    (lp l-0) (ep (mul w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB w) (l-1 w) (e (exp (gen) l)) (e (exp (gen) l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) key)))
  (neq ((mul w e l-1 (rec eB)) (one)) (e eB) ((mul w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1723)
  (parent 1592)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (w expt) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul w e e l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w e l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul w e e)) (l-1 (mul w e e)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eA)))
        key)))
  (neq ((mul w e l-1 eA) (one)) (eA e) ((mul w e e l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eA)))
      (send (cat (exp (gen) e) (mul w l-1 eA)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1724)
  (parent 1596)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (eA rndx) (w expt)
    (l l-0 l-1 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e e)
    (lp l-0) (ep (mul w l-1 (rec e))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eA) w)) (l-1 (mul (rec eA) w)) (e (exp (gen) l))
    (e (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec e)) (one)) (eA e) ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w l-1 (rec e))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 1725)
  (parent 1597)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eA)))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w e)) (l-2 (mul eB w e)) (e (exp (gen) l-1))
    (e (exp (gen) lp)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e l-2 eA)))
        key)))
  (neq ((mul w e l-2 eA) (one)) (eA eB) ((mul eB w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w e)))
      (send (cat (exp (gen) l-2) (mul eB w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eA)))
      (send (cat (exp (gen) e) (mul w l-2 eA))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1726)
  (parent 1600)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w e l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w e l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w e)) (l-2 (mul (rec eA) w e))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) key)))
  (neq ((mul w e l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w e l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1727)
  (parent 1601)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self peer self-0 name) (l l-0 l-1 rndx)
    (lp expt) (eA eB rndx) (w expt) (l-2 e rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-1) (e e)
    (lp lp))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 2) (4 0)) ((6 2) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (e (mul (rec eA) eB w)) (l-2 (mul (rec eA) eB w))
    (e (exp (gen) l-1)) (e (exp (gen) lp)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 e)
  (operation nonce-test (added-listener e) (mul w l-2 (rec e)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec e))))
      (send (cat (exp (gen) e) (mul w l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e))) ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv e) (send e)))
  (label 1728)
  (parent 1602)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1729)
  (parent 1605)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1730)
  (parent 1606)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul eB w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul eB w l-0)) (l-1 (mul eB w l-0)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l-0)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 l-1 eA)))
        key)))
  (neq ((mul w l-0 l-1 eA) (one)) (eA eB) ((mul eB w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w l-0)))
      (send (cat (exp (gen) l-1) (mul eB w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eA)))
      (send (cat (exp (gen) l-0) (mul w l-1 eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 1731)
  (parent 1610)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l-0) (e eA)
    (lp l) (ep (mul (rec eA) w l-0 l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul (rec eA) w l-0)) (l-1 (mul (rec eA) w l-0))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l-0))
    (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) key)))
  (neq ((mul w l-0 l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0 l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0 l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eB))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 1732)
  (parent 1611)
  (unrealized (0 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eB w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eA)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul eB w l-1)) (l-2 (mul eB w l-1)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 l-2 eA)))
        key)))
  (neq ((mul w l-1 l-2 eA) (one)) (eA eB) ((mul eB w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 eA) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eB w l-1)))
      (send (cat (exp (gen) l-2) (mul eB w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eA)))
      (send (cat (exp (gen) l-1) (mul w l-2 eA))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1733)
  (parent 1616)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1 l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 l-2 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul (rec eA) w l-1)) (l-2 (mul (rec eA) w l-1))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) key)))
  (neq ((mul w l-1 l-2 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1 l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1 l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 l-2 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1734)
  (parent 1617)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 0) (4 0)) ((6 0) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1735)
  (parent 1618)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul eB (rec l-1) w)) (l-2 (mul eB (rec l-1) w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w eA))) key)))
  (neq ((mul w eA) (one)) (eA eB) ((mul eB w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) eA) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w eA))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eB (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eA))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1736)
  (parent 1622)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul (rec eA) (rec l-1) w))
    (l-2 (mul (rec eA) (rec l-1) w)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) key)))
  (neq ((mul w (rec eB)) (one)) (eA eB) ((mul (rec eA) w) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) (rec eB))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eB)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1737)
  (parent 1623)
  (unrealized (0 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-2)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-2)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 0) (4 0)) ((5 0) (9 0)) ((6 1) (1 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-2))) key)))
  (neq ((mul w l-2) (one)) (eA eB) ((mul (rec eA) eB w l-2) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-2)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-2))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 1738)
  (parent 1624)
  (seen 1743)
  (unrealized (0 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1739)
  (parent 1647)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1740)
  (parent 1682)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (l-1 (mul (rec eA) eB w)) (l-0 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) key)))
  (neq ((mul w l-0) (one)) (eA eB) ((mul (rec eA) eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 3 9 ltx-gen 2) l-1 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1741)
  (parent 1699)
  (unrealized (0 3) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1742)
  (parent 1705)
  (unrealized (0 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul (rec eA) eB w)) (l-1 (mul (rec eA) eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) key)))
  (neq ((mul w l-1) (one)) (eA eB) ((mul (rec eA) eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send key))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1743)
  (parent 1735)
  (unrealized (0 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1744)
  (parent 1739)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (4 0)) ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul l-0 eA l-2))) key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1745)
  (parent 1739)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1746)
  (parent 1739)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1747)
  (parent 1739)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1748)
  (parent 1740)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (4 0))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1749)
  (parent 1740)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1750)
  (parent 1740)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1751)
  (parent 1740)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep l-0))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (l-1 (one)) (l-0 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-1)) (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA))) key)))
  (neq ((mul l-0 eA (rec eB)) (one)) (eA eB) (l-0 (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1752)
  (parent 1741)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 0) (4 0))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1753)
  (parent 1741)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1754)
  (parent 1741)
  (unrealized (0 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 1755)
  (parent 1741)
  (unrealized (0 3) (1 3) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 l-2 eA eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (eB (exp (gen) l))
    (eB (exp (gen) l-0)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1756)
  (parent 1742)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eA)
    (lp l-2) (ep (mul l-0 l-2)))
  (defstrand participant 5 (key key) (self B) (peer self-1) (l l) (e eB)
    (lp l-2) (ep (mul l-0 eA (rec eB) l-2)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-2))
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 0) (0 1)) ((9 0) (1 1))
    ((9 1) (3 0)))
  (absent (l-1 l-2) (l-0 l-2) (eB (exp (gen) l)) (eB (exp (gen) l-2))
    (eA (exp (gen) l)) (eA (exp (gen) l-2)))
  (fn-of ("principal-of" (l-2 self-1) (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-2)) (exp (gen) (mul l-0 eA l-2)))
        key)))
  (neq ((mul l-0 eA (rec eB) l-2) (one)) (eA eB) ((mul l-0 l-2) (one)))
  (non-orig (privk B) (privk self-1) l)
  (precur (4 0) (3 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 3 10 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-2))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 eA (rec eB) l-2)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec l-1) eA (rec eB) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-1 (privk self-1)))
      (send l-2)))
  (label 1757)
  (parent 1742)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 l-2 eA eB l-3 rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1758)
  (parent 1742)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1759)
  (parent 1742)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1760)
  (parent 1742)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 l-1 l-2 eA eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep l-1))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)) (eB (exp (gen) l-0))
    (eB (exp (gen) l)) (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA))) key)))
  (neq ((mul l-1 eA (rec eB)) (one)) (eA eB) (l-1 (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul eA (rec eB)))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1761)
  (parent 1743)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 self-1 name)
    (l l-0 l-1 l-2 eA eB l-3 rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul l-1 l-3)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul l-1 eA (rec eB) l-3)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eA l-3)))
        key)))
  (neq ((mul l-1 eA (rec eB) l-3) (one)) (eA eB) ((mul l-1 l-3) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-3))) (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 eA (rec eB) l-3)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) eA (rec eB) l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 1762)
  (parent 1743)
  (unrealized (1 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul eB w)) (mul eB w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1763)
  (parent 1743)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul (rec eA) w))
    (mul (rec eA) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 1764)
  (parent 1743)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0 eA))) key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1765)
  (parent 1746)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eA)
    (lp l) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer B) (l l) (e eB)
    (lp l) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eA (exp (gen) l)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-0 (rec l-1) (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1766)
  (parent 1747)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1767)
  (parent 1750)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self) (l l) (e eB)
    (lp l-0) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (0 1)) ((6 0) (1 1)) ((6 1) (5 0))
    ((7 1) (4 0)) ((8 0) (3 0)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-0 (rec l-1) (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1768)
  (parent 1751)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul eB w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eB w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (3 0)) ((0 2) (9 0)) ((1 2) (3 0)) ((1 2) (4 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eA (mul eB w)) (l-1 (mul eB w)) (l-0 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-1)) (eA (exp (gen) l))
    (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0 eA)))
        key)))
  (neq ((mul w l-0 eA) (one)) (eA eB) ((mul eB w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-0 (rec l-1) eA)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul eB w)))
      (send (cat (exp (gen) l-0) (mul eB w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA)))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) eA))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eA) (send eA)))
  (label 1769)
  (parent 1754)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B self self-0 name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eA)
    (lp l-1) (ep (mul (rec eA) w l-0)))
  (defstrand participant 5 (key key) (self B) (peer self-0) (l l) (e eB)
    (lp l-1) (ep (mul w l-0 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (3 0)) ((0 2) (4 0)) ((1 2) (3 0)) ((1 2) (9 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 1) (0 3)) ((4 1) (1 3))
    ((5 1) (4 0)) ((6 0) (3 0)) ((6 1) (5 0)) ((7 1) (4 0))
    ((8 0) (0 1)) ((8 0) (1 1)) ((8 1) (7 0)) ((9 1) (4 0)))
  (absent (eB (mul (rec eA) w)) (l-1 (mul (rec eA) w))
    (l-0 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-1))
    (eA (exp (gen) l)) (eA (exp (gen) l-1)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))) key)))
  (neq ((mul w l-0 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-0) (one)))
  (non-orig (privk B) (privk self-0) l)
  (precur (4 0) (3 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-0 (rec l-1) (rec eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-0)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) w)))
      (send (cat (exp (gen) l-0) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1) (rec eB)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv eB) (send eB)))
  (label 1770)
  (parent 1755)
  (unrealized (0 3) (3 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l)) (eB (exp (gen) l-0)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1771)
  (parent 1759)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l) (e eB)
    (lp l-0) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (10 0))
    ((2 0) (0 0)) ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l)) (eB (exp (gen) l-0))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-1 (rec l-2) (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1772)
  (parent 1760)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul eB w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 eA)))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eB w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eA (mul eB w)) (l-2 (mul eB w)) (l-1 (mul eB w))
    (eB (exp (gen) l-0)) (eB (exp (gen) l)) (eA (exp (gen) l))
    (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1 eA)))
        key)))
  (neq ((mul w l-1 eA) (one)) (eA eB) ((mul eB w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec l-2) eA)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul eB w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 eA)))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eB w)))
      (send (cat (exp (gen) l-1) (mul eB w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eA))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 1773)
  (parent 1763)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (key key) (self D) (peer B) (l l) (e eA)
    (lp l-0) (ep (mul (rec eA) w l-1)))
  (defstrand participant 5 (key key) (self B) (peer D) (l l-0) (e eB)
    (lp l) (ep (mul w l-1 (rec eB))))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (10 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (0 3)) ((5 1) (1 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eB (mul (rec eA) w)) (l-2 (mul (rec eA) w))
    (l-1 (mul (rec eA) w)) (eB (exp (gen) l-0)) (eB (exp (gen) l))
    (eA (exp (gen) l)) (eA (exp (gen) l-0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) key)))
  (neq ((mul w l-1 (rec eB)) (one)) (eA eB)
    ((mul (rec eA) w l-1) (one)))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB)
    (mul w l-1 (rec l-2) (rec eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w l-1)))
      (send key))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1 (rec eB))))
      (send key)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) w)))
      (send (cat (exp (gen) l-1) (mul (rec eA) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eB)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 1774)
  (parent 1764)
  (unrealized (0 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (lp ep expt) (self peer name) (key data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" (exp (gen) lp) peer (privk peer)))
      (send (exp (gen) e)) (recv (exp (gen) ep)) (send key))
    (uniq-gen e)
    (fn-of
      (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e ep))) key)))
    (neq (ep (one)))
    (absent (e (exp (gen) l)) (e (exp (gen) lp))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))))
  (label 1775)
  (unrealized (0 0) (0 1) (0 3) (2 0))
  (preskeleton)
  (origs)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (precedes ((0 2) (2 0)) ((1 2) (0 3)) ((1 2) (2 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))))
  (label 1776)
  (parent 1775)
  (unrealized (0 0) (0 1) (2 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (precedes ((0 2) (1 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (operation collapsed 1 0)
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))))
  (label 1777)
  (parent 1776)
  (unrealized (0 0) (0 1) (1 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
    (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))))
  (label 1778)
  (parent 1776)
  (unrealized (0 0) (0 1) (3 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
    (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))))
  (label 1779)
  (parent 1777)
  (unrealized (0 0) (0 1) (2 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) e1) e2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (operation nonce-test (added-listener (cat (exp (gen) e1) e2))
    (exp (gen) (mul e1 e2)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) e1) e2)) (send (cat (exp (gen) e1) e2))))
  (label 1780)
  (parent 1778)
  (unrealized (0 0) (0 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (e1 e2 l rndx) (lp expt)
    (l-0 rndx) (lp-0 expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e1)
    (lp lp) (ep e2))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e2)
    (lp lp-0))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
  (deflistener (cat (exp (gen) e2) e1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((3 1) (2 0))
    ((4 1) (3 0)))
  (absent (e2 (exp (gen) l-0)) (e2 (exp (gen) lp-0)) (e1 (exp (gen) l))
    (e1 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e1 e2)
  (operation nonce-test (added-listener (cat (exp (gen) e2) e1))
    (exp (gen) (mul e1 e2)) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e1)) (recv (exp (gen) e2)) (send key))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e2)))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e1 e2)))))
    ((recv (cat (exp (gen) e2) e1)) (send (cat (exp (gen) e2) e1))))
  (label 1781)
  (parent 1778)
  (unrealized (0 0) (0 1) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx) (w expt))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul e2 e2 (rec w))) w))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul e2 e2 (rec w))) w))
    (exp (gen) (mul e2 e2)) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul e2 e2 (rec w))) w))
      (send (cat (exp (gen) (mul e2 e2 (rec w))) w))))
  (label 1782)
  (parent 1779)
  (unrealized (0 0) (0 1) (3 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (gen) (mul e2 e2)))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test (contracted (e2-0 e2) (w (mul e2 e2))) (gen)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (gen) (mul e2 e2))) (send (cat (gen) (mul e2 e2)))))
  (label 1783)
  (parent 1782)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e)
    (lp lp) (ep e))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
  (deflistener (cat (exp (gen) e) e))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (absent (e (exp (gen) l)) (e (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e e))) key)))
  (neq (e (one)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e e)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e e))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e e)))))
    ((recv (cat (exp (gen) e) e)) (send (cat (exp (gen) e) e))))
  (label 1784)
  (parent 1782)
  (unrealized (0 0) (0 1) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self peer name) (l rndx) (lp expt) (l-0 rndx)
    (lp-0 expt) (e2 e rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) e) (mul e2 e2 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (l l-0) (e e)
    (lp lp-0))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0)))
  (absent (e (exp (gen) l-0)) (e (exp (gen) lp-0)) (e2 (exp (gen) l))
    (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e2 e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) e) (mul e2 e2 (rec e))))
      (send (cat (exp (gen) e) (mul e2 e2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) lp-0) peer (privk peer)))
      (send (exp (gen) e))))
  (label 1785)
  (parent 1782)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B self name) (l rndx) (lp expt) (e2 l-0 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) l-0) (mul e2 e2 (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (3 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of ("principal-of" (l-0 self))
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (precur (3 0))
  (uniq-gen e2 l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) l-0) (mul e2 e2 (rec l-0))))
      (send (cat (exp (gen) l-0) (mul e2 e2 (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 1786)
  (parent 1782)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (key data) (A B name) (l rndx) (lp expt) (e2 rndx))
  (defstrand participant 5 (key key) (self A) (peer B) (l l) (e e2)
    (lp lp) (ep e2))
  (deflistener (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
  (deflistener (cat (gen) (mul e2 e2)))
  (deflistener e2)
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0)))
  (absent (e2 (exp (gen) l)) (e2 (exp (gen) lp)))
  (fn-of
    (foo ((hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))) key)))
  (neq (e2 (one)))
  (non-orig (privk A) (privk B))
  (uniq-gen e2)
  (precur (3 0))
  (operation nonce-test (added-listener e2) (mul e2 e2) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lp) B (privk B)))
      (send (exp (gen) e2)) (recv (exp (gen) e2)) (send key))
    ((recv (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (hash (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2))))
      (send (cat (exp (gen) (mul l lp)) (exp (gen) (mul e2 e2)))))
    ((recv (cat (gen) (mul e2 e2))) (send (cat (gen) (mul e2 e2))))
    ((recv e2) (send e2)))
  (label 1787)
  (parent 1783)
  (unrealized (0 0) (0 1) (4 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")
