(herald "IADH: unified model (UM)" (bound 20) (limit 2000)
  (algebra diffie-hellman))

(comment "CPSA 3.6.11")
(comment "All input read from tst/iadh_um_joshua.scm")
(comment "Strand count bounded at 20")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (hl he base) (self peer name) (kh data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))
      (recv he) (send kh))
    (fn-of (foo ((hash (exp hl l) (exp he e)) kh)))
    (neq (he (gen))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (hl he hl-0 he-0 base)
    (eA lA eB lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl hl-0) (he he-0)
    (l lB) (e eB))
  (fn-of
    (foo ((hash (exp hl-0 lB) (exp he-0 eB)) kh)
      ((hash (exp hl lA) (exp he eA)) kh)))
  (neq (he-0 (gen)) (he (gen)) (A B) (eA eB) (lA lB) (eA lA) (eB lB)
    (eA lB) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send kh))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" hl-0 D (privk D))) (send (exp (gen) eB))
      (recv he-0) (send kh)))
  (label 0)
  (unrealized (0 0) (0 1) (1 0) (1 1))
  (preskeleton)
  (origs)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (hl he base) (lA lB eA eB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp hl (mul lA (rec lB)))) (he (exp he (mul eA (rec eB))))
    (l lB) (e eB))
  (precedes ((0 2) (1 3)))
  (fn-of (foo ((hash (exp hl lA) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA lB) (eA lA) (eB lB) (eA lB) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send kh))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp hl (mul lA (rec lB))) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB))))
      (send kh)))
  (label 1)
  (parent 0)
  (unrealized (0 0) (0 1) (1 0) (1 1) (1 3))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (hl he base) (lA eA eB l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp hl (mul lA (rec l)))) (he (exp he (mul eA (rec eB)))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((2 0) (1 0)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp hl lA) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l) B (privk B)) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp hl (mul lA (rec l))) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB)))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 2)
  (parent 1)
  (unrealized (0 0) (0 1) (1 1) (1 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (he base) (lA eA eB l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l))) (he he) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (operation encryption-test (displaced 3 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB)))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 3)
  (parent 2)
  (unrealized (0 0) (0 1) (1 3))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (he base) (lA eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0))) (he he) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 1))
    ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) D (privk D)) (1 1))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB)))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 4)
  (parent 2)
  (unrealized (0 0) (0 1) (1 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eB l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) eB)) (l l-0)
    (e e))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) eB) (gen)) (A B) (e eB) (l-0 l)
    (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eB l e)
  (operation nonce-test (displaced 3 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 5)
  (parent 3)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (lA eA l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) e e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (A B) (eA e) (lA l) (eA lA) (e l) (eA l) (e lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA l e)
  (operation nonce-test (displaced 3 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 6)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C self peer name) (hl base)
    (lA eA eB l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 2) (1 3)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 7)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (lA eA eB l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB l))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 3 2 ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B)))))
  (label 8)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C self name) (lA eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 9)
  (parent 3)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (he base) (lA eA eB l rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l))) (he he) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp he (mul eA (rec eB) (rec w))) w))
  (precedes ((0 2) (3 0)) ((1 2) (3 0)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 1) (1 3)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec eB) (rec w))) w))
    (exp he (mul eA (rec eB))) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB)))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp he (mul eA (rec eB) (rec w))) w))
      (send (cat (exp he (mul eA (rec eB) (rec w))) w))))
  (label 10)
  (parent 3)
  (unrealized (0 0) (0 1) (3 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eB l l-0 l-1 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) eB)) (l l-1)
    (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) eB) (gen)) (A B) (e eB) (l-1 l)
    (e l-1) (eB l) (e l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 11)
  (parent 4)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (lA eA l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) e e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (A B) (eA e) (lA l-0) (eA lA) (e l-0) (eA l-0) (e lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 12)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self peer name) (hl base)
    (lA eA eB l l-0 l-1 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 13)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (lA eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (A B) (eA eB) (lA l-0) (eA lA) (eB l-0) (eA l-0) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 14)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (lA eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 4 3 ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 15)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self name) (lA eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 16)
  (parent 4)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (he base) (lA eA eB l l-0 rndx)
    (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0))) (he he) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp he (mul eA (rec eB)))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp he (mul eA (rec eB) (rec w))) w))
  (precedes ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec eB) (rec w))) w))
    (exp he (mul eA (rec eB))) (1 3))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB)))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp he (mul eA (rec eB) (rec w))) w))
      (send (cat (exp he (mul eA (rec eB) (rec w))) w))))
  (label 17)
  (parent 4)
  (unrealized (0 0) (0 1) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eB l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) eB)) (l l-0)
    (e e))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) eB) (gen)) (A B) (e eB) (l-0 l)
    (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eB l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 18)
  (parent 5)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) e e))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (A B) (eA e) (l-0 l) (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eA l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 19)
  (parent 6)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C self peer name) (hl base)
    (eA eB l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB e))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 2) (1 3)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (A B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (uniq-gen eA eB l l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 20)
  (parent 7)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB l))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (A B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 21)
  (parent 8)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C self name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (self B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk C) (privk self) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 22)
  (parent 9)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 3)) ((3 0) (1 3)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (A B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 23)
  (parent 9)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (lA eA eB l rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec eA) eB w)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (uniq-gen eA eB l)
  (precur (3 0))
  (operation nonce-test
    (contracted (he (exp (gen) (mul (rec eA) eB w)))) (gen) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 24)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eB l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) (mul eB w)))
    (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen)) ((exp (gen) (mul eB w)) (gen))
    (A B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eB l e)
  (operation nonce-test (displaced 4 0 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 25)
  (parent 10)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (lA eA rndx) (w expt) (l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) w e e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) w e e)) (gen)) (A B) (eA e) (lA l) (eA lA)
    (e l) (eA l) (e lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA l e)
  (operation nonce-test (displaced 4 1 participant 3) (exp (gen) e-0)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 26)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C self peer name) (hl base) (lA eA eB l rndx)
    (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) eB w e)) (gen)) (A B) (eA eB) (lA l)
    (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 27)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (lA eA eB rndx) (w expt) (l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w l))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)))
  (fn-of ("principal-of" (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) kh)))
  (neq ((exp (gen) (mul w l)) (gen))
    ((exp (gen) (mul (rec eA) eB w l)) (gen)) (A B) (eA eB) (lA l)
    (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA eB l)
  (operation nonce-test (displaced 4 2 ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 28)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C self name) (lA eA eB l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l)))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (A B) (eA eB) (lA l)
    (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) lA l)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 29)
  (parent 10)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eB l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) eB)) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) eB) (gen)) (D B) (e eB) (l-0 l)
    (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eB l e l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) eB)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 30)
  (parent 11)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eB l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) eB)) (l l-1)
    (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) eB) (gen)) (A B) (e eB) (l-1 l)
    (e l-1) (eB l) (e l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eB l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 31)
  (parent 11)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) e e))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l-0 l) (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA l e l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 32)
  (parent 12)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) e e))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (A B) (eA e) (l-1 l-0) (eA l-1) (e l-0) (eA l-0) (e l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (uniq-gen eA l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 33)
  (parent 12)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D self peer name) (hl base)
    (eA eB l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB e))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l l-1)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 34)
  (parent 13)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self peer name) (hl base)
    (eA eB l l-0 l-1 e l-2 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB e))) (l l-2) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 2) (1 3)) ((5 0) (0 0)))
  (fn-of ("principal-of" (l-2 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (A B) (eA eB) (l-2 l) (eA l-2) (eB l) (eA l) (eB l-2))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (uniq-gen eA eB l l-0 l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 35)
  (parent 13)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 36)
  (parent 14)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (A B) (eA eB) (l-1 l-0) (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 37)
  (parent 14)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D)))))
  (label 38)
  (parent 15)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (A B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 39)
  (parent 15)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 40)
  (parent 16)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B C D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (self B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk C) (privk D) (privk self) l l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 41)
  (parent 16)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self name) (eA eB l l-0 l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l l-2) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 1)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-2 A) (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (A B) (eA eB) (l-2 l) (eA l-2) (eB l) (eA l) (eB l-2))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (uniq-gen eA eB l l-0 l-1 l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 42)
  (parent 16)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (lA eA eB l l-0 rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec eA) eB w)) (gen))
    (A B) (eA eB) (lA l) (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (uniq-gen eA eB l l-0)
  (precur (4 0))
  (operation nonce-test
    (contracted (he (exp (gen) (mul (rec eA) eB w)))) (gen) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 43)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eB l l-0 rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) (mul eB w)))
    (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen)) ((exp (gen) (mul eB w)) (gen))
    (A B) (e eB) (l-1 l) (e l-1) (eB l) (e l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 44)
  (parent 17)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (lA eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) w e e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) w e e)) (gen)) (A B) (eA e) (lA l-0)
    (eA lA) (e l-0) (eA l-0) (e lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 45)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self peer name) (hl base)
    (lA eA eB l l-0 rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w e))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) eB w e)) (gen)) (A B) (eA eB) (lA l)
    (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 46)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (lA eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (A B) (eA eB) (lA l-0)
    (eA lA) (eB l-0) (eA l-0) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 47)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (lA eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (A B) (eA eB) (lA l)
    (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 48)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self name) (lA eA eB l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul (rec lA) l l-0)))
    (he (exp (gen) (mul (rec eA) eB w l-1))) (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) kh)))
  (neq ((exp (gen) (mul w l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-1)) (gen)) (A B) (eA eB) (lA l)
    (eA lA) (eB l) (eA l) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA l)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul (rec lA) l l-0)) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 49)
  (parent 17)
  (unrealized (0 0) (0 1) (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB w))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec eA) eB w)) (gen))
    (A B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 50)
  (parent 24)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eB l rndx) (w expt) (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0)))) (he (exp (gen) (mul eB w)))
    (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen)) ((exp (gen) (mul eB w)) (gen))
    (A B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eB l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 51)
  (parent 25)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) w e e))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) w e e)) (gen)) (A B) (eA e) (l-0 l)
    (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eA l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 52)
  (parent 26)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C self peer name) (hl base) (eA eB l rndx)
    (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w e))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 2) (3 0)) ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) eB w e)) (gen)) (A B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (precur (3 0))
  (uniq-gen eA eB l l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 53)
  (parent 27)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB w l))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-0 A) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l))) kh)))
  (neq ((exp (gen) (mul w l)) (gen))
    ((exp (gen) (mul (rec eA) eB w l)) (gen)) (A B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk A) (privk B) (privk C) l l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 54)
  (parent 28)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C self name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self) (peer C)
    (hl (exp (gen) (mul l l (rec l-0))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 0)))
  (fn-of ("principal-of" (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (self B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk C) (privk self) l l-0)
  (precur (3 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-0))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 55)
  (parent 29)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C self name) (eA eB l rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (3 0)) ((1 2) (0 3)) ((1 2) (3 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 1) (1 3)) ((4 0) (0 3)) ((4 0) (3 0))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self) (l B))
    (foo ((hash (exp (gen) (mul l l)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (A B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) l l-1)
  (precur (3 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 56)
  (parent 29)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) eB)) (l l) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) eB) (gen)) (D B) (e eB) (l l-0)
    (e l) (eB l-0) (e l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB e l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) eB)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 57)
  (parent 30)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (A D) (B B) (C B) (D D) (hl (exp (gen) l-0))
        (he (exp (gen) eB)) (lA l) (lB l-0) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA e l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 58)
  (parent 32)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l-0 l-1)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 59)
  (parent 34)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 60)
  (parent 36)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 4 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 61)
  (parent 38)
  (unrealized (0 3))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (0 3)) ((4 0) (1 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 62)
  (parent 40)
  (unrealized (0 3))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB w))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec eA) eB w)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 63)
  (parent 43)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eA eB l l-0 rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec eA) eB w)) (gen))
    (A B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 64)
  (parent 43)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eB l rndx) (w expt) (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul eB w))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen)) ((exp (gen) (mul eB w)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l e l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB w))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 65)
  (parent 44)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eB l l-0 rndx) (w expt) (e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1)))) (he (exp (gen) (mul eB w)))
    (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen)) ((exp (gen) (mul eB w)) (gen))
    (A B) (e eB) (l-1 l) (e l-1) (eB l) (e l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eB l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 66)
  (parent 44)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) w e e))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) w e e)) (gen)) (D B) (eA e) (l-0 l)
    (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l e l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 67)
  (parent 45)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eA l rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) w e e))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l D) (l-0 B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) w e e)) (gen)) (A B) (eA e) (l-1 l-0)
    (eA l-1) (e l-0) (eA l-0) (e l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 e l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 68)
  (parent 45)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D self peer name) (hl base) (eA eB l rndx)
    (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB w e))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (fn-of ("principal-of" (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) eB w e)) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-1)
  (operation encryption-test (displaced 6 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 69)
  (parent 46)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self peer name) (hl base) (eA eB l l-0 rndx)
    (w expt) (l-1 e l-2 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB w e))) (l l-2) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)) ((6 0) (0 0)))
  (fn-of ("principal-of" (l-2 A) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) eB w e)) (gen)) (A B) (eA eB) (l-2 l)
    (eA l-2) (eB l) (eA l) (eB l-2))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 70)
  (parent 46)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB w l))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) kh)))
  (neq ((exp (gen) (mul w l)) (gen))
    ((exp (gen) (mul (rec eA) eB w l)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 71)
  (parent 47)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eA eB l rndx) (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (A B) (eA eB) (l-1 l-0)
    (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 72)
  (parent 47)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk C) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 73)
  (parent 48)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (eA eB l rndx) (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (A B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 74)
  (parent 48)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B C D self name) (eA eB l rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer C) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 D) (l-0 self) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk C) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 3 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 75)
  (parent 49)
  (unrealized (0 1) (0 3))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B C D self name) (eA eB l l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self self) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-1))))
    (he (exp (gen) (mul (rec eA) eB w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) kh)))
  (neq ((exp (gen) (mul w l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-1)) (gen)) (self B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk C) (privk D) (privk self) l l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-1))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 76)
  (parent 49)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A B C D self name) (eA eB l l-0 rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C)
    (hl (exp (gen) (mul l l-0 (rec l-2))))
    (he (exp (gen) (mul (rec eA) eB w l-1))) (l l-2) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self A) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 1)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 0) (0 0)))
  (fn-of ("principal-of" (l-2 A) (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-1))) kh)))
  (neq ((exp (gen) (mul w l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-1)) (gen)) (A B) (eA eB) (l-2 l)
    (eA l-2) (eB l) (eA l) (eB l-2))
  (non-orig (privk A) (privk B) (privk C) (privk D) l l-2)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1 l-2)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-2) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-2) A (privk A)))
      (recv (enc "cert" (exp (gen) (mul l l-0 (rec l-2))) C (privk C)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) A (privk A)))))
  (label 77)
  (parent 49)
  (unrealized (0 1) (0 3))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (rec eA)) e))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA e l l-0)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eA)) e))
    (exp (gen) (mul (rec eA) e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (rec eA)) e))
      (send (cat (exp (gen) (rec eA)) e))))
  (label 78)
  (parent 58)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul e e)) eA))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA e l l-0)
  (operation nonce-test (added-listener (cat (exp (gen) (mul e e)) eA))
    (exp (gen) (mul (rec eA) e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul e e)) eA))
      (send (cat (exp (gen) (mul e e)) eA))))
  (label 79)
  (parent 58)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB)) e))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((5 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l-0 l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB)) e))
    (exp (gen) (mul (rec eA) eB e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB)) e))
      (send (cat (exp (gen) (mul (rec eA) eB)) e))))
  (label 80)
  (parent 59)
  (unrealized (5 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) e)) eB))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((5 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l-0 l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) e)) eB))
    (exp (gen) (mul (rec eA) eB e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) e)) eB))
      (send (cat (exp (gen) (mul (rec eA) e)) eB))))
  (label 81)
  (parent 59)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul eB e)) eA))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((5 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l-0 l-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eB e)) eA))
    (exp (gen) (mul (rec eA) eB e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul eB e)) eA))
      (send (cat (exp (gen) (mul eB e)) eA))))
  (label 82)
  (parent 59)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) l-0)) eB))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) l-0)) eB))
    (exp (gen) (mul (rec eA) eB l-0)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) l-0)) eB))
      (send (cat (exp (gen) (mul (rec eA) l-0)) eB))))
  (label 83)
  (parent 60)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eB l-0)) eA))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul eB l-0)) eA))
    (exp (gen) (mul (rec eA) eB l-0)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eB l-0)) eA))
      (send (cat (exp (gen) (mul eB l-0)) eA))))
  (label 84)
  (parent 60)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) l)) eB))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) l)) eB))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) l)) eB))
      (send (cat (exp (gen) (mul (rec eA) l)) eB))))
  (label 85)
  (parent 61)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul eB l)) eA))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eB l)) eA))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul eB l)) eA))
      (send (cat (exp (gen) (mul eB l)) eA))))
  (label 86)
  (parent 61)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB)) l))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB)) l))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB)) l))
      (send (cat (exp (gen) (mul (rec eA) eB)) l))))
  (label 87)
  (parent 62)
  (unrealized (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) l)) eB))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) l)) eB))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) l)) eB))
      (send (cat (exp (gen) (mul (rec eA) l)) eB))))
  (label 88)
  (parent 62)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul eB l)) eA))
  (precedes ((0 2) (1 3)) ((0 2) (5 0)) ((1 2) (5 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 0) (1 3))
    ((4 0) (5 0)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-listener (cat (exp (gen) (mul eB l)) eA))
    (exp (gen) (mul (rec eA) eB l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul eB l)) eA))
      (send (cat (exp (gen) (mul eB l)) eA))))
  (label 89)
  (parent 62)
  (unrealized (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec eA) eB w)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 90)
  (parent 63)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB rndx) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen)) ((exp (gen) (mul eB w)) (gen))
    (D B) (e eB) (l l-0) (e l) (eB l-0) (e l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB e l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 91)
  (parent 65)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA rndx) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) w e e)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA e l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 92)
  (parent 67)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (eA eB rndx) (w expt)
    (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) eB w e)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA eB l-0 l-1)
  (operation encryption-test (displaced 6 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 93)
  (parent 69)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 94)
  (parent 71)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) kh)))
  (neq ((exp (gen) (mul w l)) (gen))
    ((exp (gen) (mul (rec eA) eB w l)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation encryption-test (displaced 5 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 95)
  (parent 73)
  (unrealized (0 3))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w l))) kh)))
  (neq ((exp (gen) (mul w l)) (gen))
    ((exp (gen) (mul (rec eA) eB w l)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation encryption-test (displaced 6 2 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 96)
  (parent 75)
  (unrealized (0 3))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB)) e))
  (deflistener (cat (exp (gen) (rec eA)) eB))
  (precedes ((0 2) (1 3)) ((0 2) (6 0)) ((1 2) (6 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l-0 l-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eA)) eB))
    (exp (gen) (mul (rec eA) eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB)) e))
      (send (cat (exp (gen) (mul (rec eA) eB)) e)))
    ((recv (cat (exp (gen) (rec eA)) eB))
      (send (cat (exp (gen) (rec eA)) eB))))
  (label 97)
  (parent 80)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (eA eB l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB)) e))
  (deflistener (cat (exp (gen) eB) eA))
  (precedes ((0 2) (1 3)) ((0 2) (6 0)) ((1 2) (6 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 2) (1 3))
    ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (uniq-gen eA eB l-0 l-1)
  (operation nonce-test (added-listener (cat (exp (gen) eB) eA))
    (exp (gen) (mul (rec eA) eB)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB)) e))
      (send (cat (exp (gen) (mul (rec eA) eB)) e)))
    ((recv (cat (exp (gen) eB) eA)) (send (cat (exp (gen) eB) eA))))
  (label 98)
  (parent 80)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB)) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (1 3)) ((0 2) (4 0)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (displaced 4 6 ltx-gen 2) l-1 (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB)) l-1))
      (send (cat (exp (gen) (mul (rec eA) eB)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 99)
  (parent 87)
  (unrealized (4 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) ((exp (gen) e) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) e e)))
      (send (cat (gen) (mul (rec eB) e e)))))
  (label 100)
  (parent 90)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) eA))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) eA) (gen)) ((exp (gen) e) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) eA)) (send (cat (gen) eA))))
  (label 101)
  (parent 90)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) ((exp (gen) e) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) e)))
      (send (cat (gen) (mul eA (rec eB) e))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 102)
  (parent 90)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 103)
  (parent 90)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-0)))
      (send (cat (gen) (mul eA (rec eB) l-0)))))
  (label 104)
  (parent 90)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) l-1)))
      (send (cat (gen) (mul eA (rec eB) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 105)
  (parent 90)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul (rec eA) eB w)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w (rec w-0))) w-0))))
  (label 106)
  (parent 90)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB rndx) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB w))) (l l) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (precedes ((0 2) (1 3)) ((1 2) (0 3)) ((2 0) (0 1)) ((2 0) (1 0))
    ((3 0) (0 0)) ((3 0) (1 1)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen)) ((exp (gen) (mul eB w)) (gen))
    (D B) (e eB) (l l-0) (e l) (eB l-0) (e l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eB e l l-0)
  (operation generalization deleted (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB w))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D)))))
  (label 107)
  (parent 91)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (A D) (B B) (C B) (D D) (hl (exp (gen) l-0))
        (he (exp (gen) (mul eB w))) (lA l) (lB l-0) (eA e) (eB eB))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e-0)) (l l-0) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) e-0 e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e-0 e-0))) kh)))
  (neq ((exp (gen) (mul (rec e) e-0 e-0)) (gen)) ((exp (gen) e-0) (gen))
    (D B) (e-0 e) (l-0 l) (e-0 l-0) (e l) (e-0 l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) e-0 e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) e-0 e-0)))))
  (label 108)
  (parent 92)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) eA) (gen)) ((exp (gen) e) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e))))))
  (label 109)
  (parent 92)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) e-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e-0))) kh)))
  (neq ((exp (gen) (mul eA (rec e) e-0)) (gen)) ((exp (gen) e-0) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) e-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 110)
  (parent 92)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) l-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec e) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))))
  (label 111)
  (parent 92)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec e) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA e) (l-0 l) (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-0)))))
  (label 112)
  (parent 92)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec e) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 113)
  (parent 92)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA rndx) (w expt) (e l l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) w e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) w))
  (deflistener (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) w e e)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA e l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) w e e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) w e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) w e e (rec w-0))) w-0))))
  (label 114)
  (parent 92)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eB e l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e-0 e-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) e-0 e-0)) (gen))
    ((exp (gen) e-0) (gen)) (D B) (e-0 eB) (l-1 l-0) (e-0 l-1) (eB l-0)
    (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eB l-1 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) e-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e-0 e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 115)
  (parent 93)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA e l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) eA)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e-0))) kh)))
  (neq ((exp (gen) eA) (gen)) ((exp (gen) e-0) (gen)) (D B) (eA e-0)
    (l-0 l-1) (eA l-0) (e-0 l-1) (eA l-1) (e-0 l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eA l-1 e-0)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-1)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) eA)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 116)
  (parent 93)
  (unrealized (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)) ((5 2) (4 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) ((exp (gen) e) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB))))
      (send (cat (exp (gen) e) (mul eA (rec eB)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 117)
  (parent 93)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)) ((6 2) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) e-0)) (gen)) ((exp (gen) e-0) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 118)
  (parent 93)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eA eB l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 119)
  (parent 93)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l-1 l-0) (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eA eB l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 120)
  (parent 93)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) l-2)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-2))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)) ((6 0) (0 3)) ((6 0) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-2)) (gen)) ((exp (gen) l-2) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 121)
  (parent 93)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (eA eB rndx) (w expt)
    (l e l-0 l-1 rndx) (w-0 expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w e))) kh)))
  (neq ((exp (gen) (mul w e)) (gen))
    ((exp (gen) (mul (rec eA) eB w e)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA eB l-0 l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w e)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w e (rec w-0))) w-0))))
  (label 122)
  (parent 93)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) ((exp (gen) e) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) e e)))))
  (label 123)
  (parent 94)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) eA) (gen)) ((exp (gen) e) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0))))))
  (label 124)
  (parent 94)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) ((exp (gen) e) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 125)
  (parent 94)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 126)
  (parent 94)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))))
  (label 127)
  (parent 94)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 128)
  (parent 94)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l-0))) kh)))
  (neq ((exp (gen) (mul w l-0)) (gen))
    ((exp (gen) (mul (rec eA) eB w l-0)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l-0)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l-0 (rec w-0))) w-0))))
  (label 129)
  (parent 94)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) ((exp (gen) e) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))))
  (label 130)
  (parent 95)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) eA)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) eA) (gen)) ((exp (gen) e) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 5 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l))))
      (send (cat (exp (gen) l) (mul eA (rec l))))))
  (label 131)
  (parent 95)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (0 3)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) ((exp (gen) e) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-1)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) e))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 132)
  (parent 95)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 5 2 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) l-0)))))
  (label 133)
  (parent 95)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB))))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0)) (gen)) ((exp (gen) l-0) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 5 3 ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB))))
      (send (cat (exp (gen) l-0) (mul eA (rec eB))))))
  (label 134)
  (parent 95)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 135)
  (parent 95)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx)
    (w-0 expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB w l))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) w))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB w l))) kh)))
  (neq ((exp (gen) (mul w l)) (gen))
    ((exp (gen) (mul (rec eA) eB w l)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 136)
  (parent 95)
  (unrealized (5 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) e e)) (gen)) ((exp (gen) e) (gen))
    (D B) (e eB) (l-1 l) (e l-1) (eB l) (e l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 137)
  (parent 96)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) eA)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) eA) (gen)) ((exp (gen) e) (gen)) (D B) (eA e) (l l-1)
    (eA l) (e l-1) (eA l-1) (e l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) eA)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 138)
  (parent 96)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 2) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) e)) (gen)) ((exp (gen) e) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 139)
  (parent 96)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 140)
  (parent 96)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 141)
  (parent 96)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 142)
  (parent 96)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-2)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 0) (0 3)) ((5 0) (4 0)) ((6 0) (0 3)) ((6 0) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-2)) (gen)) ((exp (gen) l-2) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-2)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 143)
  (parent 96)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB rndx) (w expt) (l l-0 l-1 rndx)
    (w-0 expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB w l))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D) (l self))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB w l))) kh)))
  (neq ((exp (gen) (mul w l)) (gen))
    ((exp (gen) (mul (rec eA) eB w l)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test
    (added-listener
      (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
    (exp (gen) (mul (rec eA) eB w l)) (0 3))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB w l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w l))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))
      (send (cat (exp (gen) (mul (rec eA) eB w l (rec w-0))) w-0))))
  (label 144)
  (parent 96)
  (unrealized (6 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB)) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) (rec eA)) eB))
  (precedes ((0 2) (1 3)) ((0 2) (6 0)) ((1 2) (6 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-listener (cat (exp (gen) (rec eA)) eB))
    (exp (gen) (mul (rec eA) eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB)) l-1))
      (send (cat (exp (gen) (mul (rec eA) eB)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) (rec eA)) eB))
      (send (cat (exp (gen) (rec eA)) eB))))
  (label 145)
  (parent 99)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) (mul (rec eA) eB)) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener (cat (exp (gen) eB) eA))
  (precedes ((0 2) (1 3)) ((0 2) (6 0)) ((1 2) (6 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (0 3))
    ((5 0) (1 3)) ((5 1) (4 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-listener (cat (exp (gen) eB) eA))
    (exp (gen) (mul (rec eA) eB)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) (mul (rec eA) eB)) l-1))
      (send (cat (exp (gen) (mul (rec eA) eB)) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv (cat (exp (gen) eB) eA)) (send (cat (exp (gen) eB) eA))))
  (label 146)
  (parent 99)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA eB rndx) (w expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) w)) (w-1 w))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 147)
  (parent 106)
  (unrealized (1 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (e eB) (l-0 l) (e l-0) (eB l)
    (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e e)))
      (send (cat (gen) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 148)
  (parent 106)
  (unrealized (1 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 149)
  (parent 106)
  (unrealized (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA eB rndx)
    (w expt) (e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (eA eB) (l l-0) (eA l) (eB l-0)
    (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w e)))
      (send (cat (gen) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 150)
  (parent 106)
  (unrealized (1 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-0)))
      (send (cat (gen) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 151)
  (parent 106)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-0)))
      (send (cat (gen) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 152)
  (parent 106)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w l-1)))
      (send (cat (gen) (mul eA (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 153)
  (parent 106)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA e rndx) (w expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w-0 (mul eA (rec e) (rec e) w)) (e-0 e)
      (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 154)
  (parent 114)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e-0))) (l l-0) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w e-0 e-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0 e-0))) kh)))
  (neq ((exp (gen) (mul (rec e) w e-0 e-0)) (gen))
    ((exp (gen) (mul w e-0)) (gen)) (D B) (e-0 e) (l-0 l) (e-0 l-0)
    (e l) (e-0 l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w e-0 e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0 e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 155)
  (parent 114)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w (rec e))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w (rec e))))
      (send (cat (exp (gen) e) (mul eA w (rec e)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 156)
  (parent 114)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA e rndx)
    (w expt) (e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w e-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e-0))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w e-0)) (gen))
    ((exp (gen) (mul w e-0)) (gen)) (D B) (eA e) (l l-0) (eA l) (e l-0)
    (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w e-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 157)
  (parent 114)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w l-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA e) (l l-0) (eA l) (e l-0)
    (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 158)
  (parent 114)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA e) (l-0 l) (eA l-0) (e l)
    (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 159)
  (parent 114)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA e) (l l-0) (eA l) (e l-0)
    (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 160)
  (parent 114)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA e l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) e-0)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) eA)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((1 2) (0 3)) ((1 2) (4 0)) ((2 0) (0 1))
    ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1)) ((4 1) (1 3))
    ((5 2) (4 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA e-0))) kh)))
  (neq ((exp (gen) eA) (gen)) ((exp (gen) e-0) (gen)) (D B) (eA e-0)
    (l-0 l-1) (eA l-0) (e-0 l-1) (eA l-1) (e-0 l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (4 0))
  (uniq-gen l-0 eA l-1 e-0)
  (operation nonce-test (added-listener eA) (mul eA (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) e-0)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) eA)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e))))
      (send (cat (exp (gen) e) (mul eA (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 161)
  (parent 116)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e rndx) (w expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec e) w))
      (e-0 e) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 162)
  (parent 122)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB e rndx)
    (w expt) (l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0))) (l l-1) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e-0 e-0)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0)))
        kh)))
  (neq ((exp (gen) (mul (rec eB) w e-0 e-0)) (gen))
    ((exp (gen) (mul w e-0)) (gen)) (D B) (e-0 eB) (l-1 l-0) (e-0 l-1)
    (eB l-0) (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0 e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 163)
  (parent 122)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA e rndx)
    (w expt) (l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e-0)) (gen))
    (D B) (eA e-0) (l-0 l-1) (eA l-0) (e-0 l-1) (eA l-1) (e-0 l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 e-0)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 164)
  (parent 122)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB rndx)
    (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (eA eB) (l l-0) (eA l) (eB l-0)
    (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 165)
  (parent 122)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e rndx) (w expt) (e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w e-0)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e-0)) (gen))
    ((exp (gen) (mul w e-0)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 166)
  (parent 122)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB e rndx)
    (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 167)
  (parent 122)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB e rndx)
    (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1)))
    (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l-1 l-0) (eA l-1)
    (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 168)
  (parent 122)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-2)))
    (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 169)
  (parent 122)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 170)
  (parent 129)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (e eB) (l-0 l) (e l-0) (eB l)
    (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 171)
  (parent 129)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w (rec l-0))))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul eA w (rec l-0)))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 172)
  (parent 129)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB l-1 rndx)
    (w expt) (e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (eA eB) (l l-1) (eA l) (eB l-1)
    (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 173)
  (parent 129)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 174)
  (parent 129)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 175)
  (parent 129)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 176)
  (parent 129)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx) (w expt))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-0) w))
      (l-1 l-0) (w-1 w)) (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 177)
  (parent 136)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (e eB) (l-0 l) (e l-0) (eB l)
    (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 178)
  (parent 136)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w)))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 e)
  (operation nonce-test (displaced 6 1 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec l) w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 179)
  (parent 136)
  (unrealized (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB l-1 rndx)
    (w expt) (e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1) (eB l)
    (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 180)
  (parent 136)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test (displaced 6 2 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 181)
  (parent 136)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) w))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test (displaced 6 3 ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w))))
  (label 182)
  (parent 136)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) w))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-1) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 183)
  (parent 136)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx) (w expt))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w-0 (mul eA (rec eB) (rec l-1) w))
      (l-2 l-1) (w-1 w)) (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 184)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e))) (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (e eB) (l-1 l) (e l-1) (eB l)
    (e l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 185)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e)) (gen))
    (D B) (eA e) (l l-1) (eA l) (e l-1) (eA l-1) (e l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 e)
  (operation nonce-test (displaced 7 1 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 186)
  (parent 144)
  (unrealized (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 rndx) (w expt) (e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e)) (gen))
    ((exp (gen) (mul w e)) (gen)) (D B) (eA eB) (l l-0) (eA l) (eB l-0)
    (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 187)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 2 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 188)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1)
    (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 7 3 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 189)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "7 in cohort - 7 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) w))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w))))
  (label 190)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 rndx) (w expt)
    (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) w))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) w)) (send (cat (exp (gen) l-2) w)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 191)
  (parent 144)
  (unrealized (1 3) (4 0))
  (comment "8 in cohort - 8 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 192)
  (parent 147)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 193)
  (parent 147)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 194)
  (parent 147)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul eB (rec e) l-1))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul eB (rec e) l-1)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 195)
  (parent 148)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (absent (eB (mul w e e)))
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 196)
  (parent 148)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (absent (e (mul (rec eB) w)))
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 197)
  (parent 148)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e eA l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul e (rec eA) l-1)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 198)
  (parent 149)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA w))
  (operation nonce-test (added-absence eA w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 199)
  (parent 149)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) e) w))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (operation nonce-test (added-listener eA) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv eA) (send eA)))
  (label 200)
  (parent 149)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 0) (5 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) kh)))
  (neq ((exp (gen) l-2) (gen)) ((exp (gen) (mul (rec eA) eB l-2)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 201)
  (parent 150)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eB rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (absent (eA (mul (rec eB) w e)))
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 202)
  (parent 150)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (absent (eB (mul eA w e)))
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 203)
  (parent 150)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 204)
  (parent 151)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w l-0)))
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 205)
  (parent 151)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w l-0)))
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 206)
  (parent 151)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (absent (l-0 (mul eA (rec eB) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 207)
  (parent 151)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 208)
  (parent 152)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w l-0)))
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 209)
  (parent 152)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w l-0)))
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 210)
  (parent 152)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (absent (l-0 (mul eA (rec eB) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 211)
  (parent 152)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 212)
  (parent 153)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) kh)))
  (neq ((exp (gen) l-2) (gen)) ((exp (gen) (mul (rec eA) eB l-2)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 213)
  (parent 153)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) w l-1)))
  (operation nonce-test (added-absence eA (mul (rec eB) w l-1))
    (mul (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-1)))
      (send (cat (gen) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 214)
  (parent 153)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (absent (eB (mul eA w l-1)))
  (operation nonce-test (added-absence eB (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-1)))
      (send (cat (gen) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 215)
  (parent 153)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (absent (l-1 (mul eA (rec eB) w)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 216)
  (parent 153)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (gen) (mul (rec eA) e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (gen) (mul (rec eA) e e)))
      (send (cat (gen) (mul (rec eA) e e)))))
  (label 217)
  (parent 154)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (gen) (mul (rec eA) e e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) kh)))
  (neq ((exp (gen) (mul e l-1)) (gen))
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (gen) (mul (rec eA) e e l-1)))
      (send (cat (gen) (mul (rec eA) e e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 218)
  (parent 154)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec e) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul (rec e) (rec e) w)))
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 219)
  (parent 154)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (absent (e (mul eA w)))
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e)))))
  (label 220)
  (parent 154)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul e e (rec e-0)))) (l l-0) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul e e (rec e-0))) (gen))
    (D B) (e-0 e) (l-0 l) (e-0 l-0) (e l) (e-0 l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (operation nonce-test
    (contracted (e-1 e) (w (mul e e (rec e-0) (rec e-0))) (e-2 e-0))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0))))))
  (label 221)
  (parent 155)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul e e (rec e-0) l-1))) (l l-0) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) kh)))
  (neq ((exp (gen) (mul e l-1)) (gen))
    ((exp (gen) (mul e e (rec e-0) l-1)) (gen)) (D B) (e-0 e) (l-0 l)
    (e-0 l-0) (e l) (e-0 l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul e e (rec e-0) l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1)))
      (send (cat (exp (gen) e-0) (mul e e (rec e-0) (rec e-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 222)
  (parent 155)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e e-0 e-0))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        kh)))
  (neq ((exp (gen) (mul w e e e-0)) (gen))
    ((exp (gen) (mul w e e-0 e-0)) (gen)) (D B) (e e-0) (l-0 l) (e l-0)
    (e-0 l) (e l) (e-0 l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (absent (e-0 (mul w e e)))
  (operation nonce-test (added-absence e-0 (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0)))))
  (label 223)
  (parent 155)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e-0)))) (l l-0) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec e) w)) (gen))
    ((exp (gen) (mul w (rec e-0))) (gen)) (D B) (e-0 e) (l-0 l)
    (e-0 l-0) (e l) (e-0 l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (absent (e-0 (mul (rec e) (rec e) w)))
  (operation nonce-test (added-absence e-0 (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 224)
  (parent 155)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) e)) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) e)))
      (send (cat (exp (gen) e) (mul (rec eA) e)))))
  (label 225)
  (parent 156)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) kh)))
  (neq ((exp (gen) (mul e l-1)) (gen))
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) e l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 226)
  (parent 156)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul w (rec e))))
  (operation nonce-test (added-absence eA (mul w (rec e)))
    (mul w (rec e)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 227)
  (parent 156)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (absent (e (mul eA w)))
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))))
  (label 228)
  (parent 156)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 eA e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e) (w (mul (rec eA) e e (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 229)
  (parent 157)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e e-0 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 0) (5 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul (rec eA) e e (rec e-0) l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 230)
  (parent 157)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 e rndx)
    (w expt) (e-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w e-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) kh)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen))
    ((exp (gen) (mul w e-0 (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul (rec e) (rec e) w e-0)))
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w e-0))
    (mul (rec e) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0))))
  (label 231)
  (parent 157)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA rndx)
    (w expt) (e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0 e-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e e-0))) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e-0 e-0)))
        kh)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen))
    ((exp (gen) (mul w e e-0 e-0)) (gen)) (D B) (eA e-0) (l l-0) (eA l)
    (e-0 l-0) (eA l-0) (e-0 l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e-0)
  (absent (e-0 (mul eA w e)))
  (operation nonce-test (added-absence e-0 (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 232)
  (parent 157)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 233)
  (parent 158)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) kh)))
  (neq ((exp (gen) (mul e l-1)) (gen))
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 234)
  (parent 158)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w l-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (absent (eA (mul (rec e) (rec e) w l-0)))
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 235)
  (parent 158)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen))
    ((exp (gen) (mul w l-0 e e)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (absent (e (mul eA w l-0)))
  (operation nonce-test (added-absence e (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 236)
  (parent 158)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (absent (l-0 (mul eA (rec e) (rec e) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 237)
  (parent 158)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) e e))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l-0 l) (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-0)))
      (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0))))))
  (label 238)
  (parent 159)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) e e l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) kh)))
  (neq ((exp (gen) (mul e l-1)) (gen))
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)) (D B) (eA e) (l-0 l)
    (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) e e (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 239)
  (parent 159)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA e) (l-0 l)
    (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (absent (eA (mul (rec e) (rec e) w l-0)))
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-0))
    (mul (rec e) (rec e) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 240)
  (parent 159)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 e e))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen))
    ((exp (gen) (mul w l-0 e e)) (gen)) (D B) (eA e) (l-0 l) (eA l-0)
    (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (absent (e (mul eA w l-0)))
  (operation nonce-test (added-absence e (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0 e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))))
  (label 241)
  (parent 159)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec e) w))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA e) (l-0 l) (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (absent (l-0 (mul eA (rec e) (rec e) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA (rec e) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 242)
  (parent 159)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 243)
  (parent 160)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) kh)))
  (neq ((exp (gen) (mul e l-1)) (gen))
    ((exp (gen) (mul (rec eA) e e l-1)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 244)
  (parent 160)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA e l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) e e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 245)
  (parent 160)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec e) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (absent (eA (mul (rec e) (rec e) w l-1)))
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w l-1))
    (mul (rec e) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 246)
  (parent 160)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 e e)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1 e)) (gen))
    ((exp (gen) (mul w l-1 e e)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (absent (e (mul eA w l-1)))
  (operation nonce-test (added-absence e (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1 e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-1)))
      (send (cat (exp (gen) e) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 247)
  (parent 160)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (absent (l-1 (mul eA (rec e) (rec e) w)))
  (operation nonce-test (added-absence l-1 (mul eA (rec e) (rec e) w))
    (mul eA (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 248)
  (parent 160)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 eA eB e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e)))
      (send (cat (gen) (mul (rec eA) eB e)))))
  (label 249)
  (parent 162)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul (rec eA) eB e l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (6 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul (rec eA) eB e l-2)))
      (send (cat (gen) (mul (rec eA) eB e l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 250)
  (parent 162)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eB e rndx)
    (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB eA)
  (absent (eA (mul (rec eB) (rec e) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 251)
  (parent 162)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA e rndx)
    (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (absent (eB (mul eA (rec e) w)))
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 252)
  (parent 162)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 eB e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul eB e (rec e-0))) (gen))
    (D B) (e-0 eB) (l-1 l-0) (e-0 l-1) (eB l-0) (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e-0)
  (operation nonce-test
    (contracted (eB-0 eB) (e-1 e) (w (mul eB e (rec e-0) (rec e-0)))
      (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0))))))
  (label 253)
  (parent 163)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e e-0 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB e (rec e-0) l-2)))
    (l l-1) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (6 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul eB e (rec e-0) l-2)) (gen)) (D B) (e-0 eB)
    (l-1 l-0) (e-0 l-1) (eB l-0) (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB e-0 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eB e (rec e-0) l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2)))
      (send (cat (exp (gen) e-0) (mul eB e (rec e-0) (rec e-0) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 254)
  (parent 163)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 e rndx) (w expt)
    (l-1 e-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-1)
    (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eB)))
        kh)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen))
    ((exp (gen) (mul w e-0 eB)) (gen)) (D B) (e-0 eB) (l-1 l-0)
    (e-0 l-1) (eB l-0) (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e-0 eB)
  (absent (eB (mul (rec e) w e-0 e-0)))
  (operation nonce-test (added-absence eB (mul (rec e) w e-0 e-0))
    (mul (rec e) w e-0 e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))))
  (label 255)
  (parent 163)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB e rndx)
    (w expt) (l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e-0))) (gen)) (D B) (e-0 eB) (l-1 l-0)
    (e-0 l-1) (eB l-0) (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 e-0)
  (absent (e-0 (mul (rec eB) (rec e) w)))
  (operation nonce-test (added-absence e-0 (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))))
  (label 256)
  (parent 163)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 e eA e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) e-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) kh)))
  (neq ((exp (gen) e-0) (gen)) ((exp (gen) (mul e (rec eA) e-0)) (gen))
    (D B) (eA e) (l-0 l-1) (eA l-0) (e l-1) (eA l-1) (e l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eA)
  (operation nonce-test
    (contracted (eA-0 eA) (e-1 e-0) (w (mul (rec eA) e-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0)))))
  (label 257)
  (parent 164)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 e eA e-0 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) e-0 l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e-0 l-2))) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (6 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0 l-2)))
        kh)))
  (neq ((exp (gen) (mul e-0 l-2)) (gen))
    ((exp (gen) (mul e (rec eA) e-0 l-2)) (gen)) (D B) (eA e) (l-0 l-1)
    (eA l-0) (e l-1) (eA l-1) (e l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eA l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul e-0 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) l-2)) (send (cat (exp (gen) e-0) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 258)
  (parent 164)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 e rndx) (w expt)
    (l-1 e-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e-0 (rec eA))) (gen))
    (D B) (eA e-0) (l-0 l-1) (eA l-0) (e-0 l-1) (eA l-1) (e-0 l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e-0 eA)
  (absent (eA (mul (rec e) w)))
  (operation nonce-test (added-absence eA (mul (rec e) w))
    (mul (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA))))))
  (label 259)
  (parent 164)
  (unrealized (0 3) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA e rndx)
    (w expt) (l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (deflistener eA)
  (precedes ((0 2) (6 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w e-0)) (gen))
    (D B) (eA e-0) (l-0 l-1) (eA l-0) (e-0 l-1) (eA l-1) (e-0 l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 e-0)
  (operation nonce-test (added-listener eA) (mul eA (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv eA) (send eA)))
  (label 260)
  (parent 164)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 e eA eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eB))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul e (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB)))
      (send (cat (exp (gen) e) (mul (rec eA) eB)))))
  (label 261)
  (parent 165)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 e eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e eB l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul e (rec eA) eB l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 262)
  (parent 165)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB rndx)
    (w expt) (l-1 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 263)
  (parent 165)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA rndx)
    (w expt) (l-1 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 264)
  (parent 165)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA eB e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-1 e)
      (w (mul (rec eA) eB e (rec e-0))) (e-2 e-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0))))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 265)
  (parent 166)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 self-1 name)
    (hl hl-0 base) (l l-0 l-1 l-2 eA eB e e-0 l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-3))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0))
    ((8 0) (6 0)) ((8 1) (4 0)))
  (fn-of ("principal-of" (l-3 self-1) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3))) kh)))
  (neq ((exp (gen) (mul e l-3)) (gen))
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3)))
      (send (cat (exp (gen) e-0) (mul (rec eA) eB e (rec e-0) l-3))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 266)
  (parent 166)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen))
    ((exp (gen) (mul w e-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB eA)
  (absent (eA (mul (rec eB) (rec e) w e-0)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w e-0))
    (mul (rec eB) (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 267)
  (parent 166)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w e-0)) (gen))
    ((exp (gen) (mul w e-0 eB)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (absent (eB (mul eA (rec e) w e-0)))
  (operation nonce-test (added-absence eB (mul eA (rec e) w e-0))
    (mul eA (rec e) w e-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 268)
  (parent 166)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 269)
  (parent 167)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (6 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 270)
  (parent 167)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB e rndx)
    (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) (rec e) w l-1)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-1))
    (mul (rec eB) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 271)
  (parent 167)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA e rndx)
    (w expt) (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 eB)
  (absent (eB (mul eA (rec e) w l-1)))
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-1))
    (mul eA (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 272)
  (parent 167)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB e rndx)
    (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1)
  (absent (l-1 (mul eA (rec eB) (rec e) w)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 273)
  (parent 167)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e))) (l l-1)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) e)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-1 l-0) (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-1))) (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1))))))
  (label 274)
  (parent 168)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB e l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (6 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)) (D B) (eA eB) (l-1 l-0)
    (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB e (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 275)
  (parent 168)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB e rndx)
    (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-1 l-0)
    (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) (rec e) w l-1)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-1))
    (mul (rec eB) (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 276)
  (parent 168)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA e rndx)
    (w expt) (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-1 l-0) (eA l-1)
    (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 eB)
  (absent (eB (mul eA (rec e) w l-1)))
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-1))
    (mul eA (rec e) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 277)
  (parent 168)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB e rndx)
    (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-1 l-0) (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA eB l-1)
  (absent (l-1 (mul eA (rec eB) (rec e) w)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 278)
  (parent 168)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e)
      (w (mul (rec eA) eB e (rec l-2))) (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 279)
  (parent 169)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-2)))
    (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (6 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB e l-2)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 280)
  (parent 169)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 eA eB e l-2 l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-3))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3))) kh)))
  (neq ((exp (gen) (mul e l-3)) (gen))
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB e (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 281)
  (parent 169)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2 (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB l-2 eA)
  (absent (eA (mul (rec eB) (rec e) w l-2)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w l-2))
    (mul (rec eB) (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 282)
  (parent 169)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-2)) (gen))
    ((exp (gen) (mul w l-2 eB)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA l-2 eB)
  (absent (eB (mul eA (rec e) w l-2)))
  (operation nonce-test (added-absence eB (mul eA (rec e) w l-2))
    (mul eA (rec e) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 283)
  (parent 169)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (absent (l-2 (mul eA (rec eB) (rec e) w)))
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) (rec e) w))
    (mul eA (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 284)
  (parent 169)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 285)
  (parent 170)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 286)
  (parent 170)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) (rec l-0) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 287)
  (parent 170)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA (rec l-0) w)))
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 288)
  (parent 170)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (absent (l-0 (mul eA (rec eB) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 289)
  (parent 170)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l-0 (rec e)))) (l l)
    (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul eB l-0 (rec e))) (gen))
    (D B) (e eB) (l l-0) (e l) (eB l-0) (e l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test
    (contracted (eB-0 eB) (l-1 l-0) (w (mul eB l-0 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0 (rec e))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 290)
  (parent 171)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l)
    (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)) (D B) (e eB) (l l-0)
    (e l) (eB l-0) (e l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eB l-0 (rec e) l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 291)
  (parent 171)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (absent (eB (mul (rec l) w e e)))
  (operation nonce-test (added-absence eB (mul (rec l) w e e))
    (mul (rec l) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 292)
  (parent 171)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e e l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen))
    ((exp (gen) (mul w e l-0)) (gen)) (D B) (e eB) (l l-0) (e l)
    (eB l-0) (e l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l e l-0)
  (absent (l-0 (mul (rec eB) w e e)))
  (operation nonce-test (added-absence l-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e)) (recv (exp (gen) (mul w e l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 293)
  (parent 171)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (absent (e (mul (rec eB) (rec l) w)))
  (operation nonce-test (added-absence e (mul (rec eB) (rec l) w))
    (mul (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 294)
  (parent 171)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e eA l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul e (rec eA) l-0)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (w (mul (rec eA) l-0)) (l-1 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 295)
  (parent 172)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l e eA l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 296)
  (parent 172)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul w (rec l-0))))
  (operation nonce-test (added-absence eA (mul w (rec l-0)))
    (mul w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 297)
  (parent 172)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0))) kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w e l-0)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA e l-0)
  (absent (l-0 (mul eA w)))
  (operation nonce-test (added-absence l-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 298)
  (parent 172)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul (rec eA) eB l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 299)
  (parent 173)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 0) (5 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)) (D B) (eA eB) (l l-1)
    (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 300)
  (parent 173)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB l-1 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 eA)
  (absent (eA (mul (rec eB) (rec l-1) w e)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w e))
    (mul (rec eB) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 301)
  (parent 173)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA l-1 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 eB)
  (absent (eB (mul eA (rec l-1) w e)))
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w e))
    (mul eA (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 302)
  (parent 173)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB rndx)
    (w expt) (e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1)
  (absent (l-1 (mul eA (rec eB) w e)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 303)
  (parent 173)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul l-0 (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 304)
  (parent 174)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 (rec eA) eB l-1)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 305)
  (parent 174)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 306)
  (parent 174)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 307)
  (parent 174)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 308)
  (parent 175)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-1))) kh)))
  (neq ((exp (gen) (mul l l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 309)
  (parent 175)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (absent (eA (mul (rec eB) (rec l) w l-0)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 310)
  (parent 175)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (absent (eB (mul eA (rec l) w l-0)))
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 311)
  (parent 175)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen))
    ((exp (gen) (mul w l l-0)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (absent (l-0 (mul eA (rec eB) w l)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 312)
  (parent 175)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (absent (l-0 (mul eA (rec eB) (rec l) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 313)
  (parent 175)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 314)
  (parent 176)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 315)
  (parent 176)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-2))) kh)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 316)
  (parent 176)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 317)
  (parent 176)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 318)
  (parent 176)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0 l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 319)
  (parent 176)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 320)
  (parent 176)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (gen) (mul (rec eA) eB l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l-0) (w (mul (rec eA) eB l-0)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (gen) (mul (rec eA) eB l-0)))
      (send (cat (gen) (mul (rec eA) eB l-0)))))
  (label 321)
  (parent 177)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-0 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 322)
  (parent 177)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) (rec l-0) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 323)
  (parent 177)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA (rec l-0) w)))
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 324)
  (parent 177)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA eB rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0)
  (absent (l-0 (mul eA (rec eB) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-0))) (send (cat (gen) (mul w l-0)))))
  (label 325)
  (parent 177)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul eB l-0 (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul eB l-0 (rec e))) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB l-0 (rec e) (rec e))) (l-1 l-0)
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e))))))
  (label 326)
  (parent 178)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul eB l-0 (rec e) l-1))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul eB l-0 (rec e) l-1)) (gen)) (D B) (e eB) (l-0 l)
    (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-0 (rec e) l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1)))
      (send (cat (exp (gen) e) (mul eB l-0 (rec e) (rec e) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 327)
  (parent 178)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (absent (eB (mul w (rec l-0) e e)))
  (operation nonce-test (added-absence eB (mul w (rec l-0) e e))
    (mul w (rec l-0) e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 328)
  (parent 178)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-0))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e l-0)) (gen))
    ((exp (gen) (mul w e l-0)) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB e l-0)
  (absent (l-0 (mul (rec eB) w e e)))
  (operation nonce-test (added-absence l-0 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 329)
  (parent 178)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (absent (e (mul (rec eB) w (rec l-0))))
  (operation nonce-test (added-absence e (mul (rec eB) w (rec l-0)))
    (mul (rec eB) w (rec l-0)) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 330)
  (parent 178)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e eA l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul e (rec eA) l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul e (rec eA) l-0)) (gen))
    (D B) (eA e) (l-0 l) (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (l-1 l-0) (w (mul (rec eA) l-0))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0)))))
  (label 331)
  (parent 179)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l e eA l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul e (rec eA) l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul e (rec eA) l-0 l-1)) (gen)) (D B) (eA e) (l-0 l)
    (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e eA l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-0 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-0 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 332)
  (parent 179)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul (rec l) w)))
  (operation nonce-test (added-absence eA (mul (rec l) w))
    (mul (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 333)
  (parent 179)
  (unrealized (0 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA rndx) (w expt) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-0))) kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w e l-0)) (gen)) (D B) (eA e) (l-0 l) (eA l-0)
    (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l e l-0)
  (absent (l-0 (mul eA w)))
  (operation nonce-test (added-absence l-0 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-0)))
      (send (cat (exp (gen) e) (mul w l-0)))))
  (label 334)
  (parent 179)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 eA eB l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1)
      (w (mul (rec eA) eB l-1 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 335)
  (parent 180)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 eA eB l-1 e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-1 l-2))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 0) (5 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-1 (rec e) l-2))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 336)
  (parent 180)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB l-1 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e (rec eA)))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 eA)
  (absent (eA (mul (rec eB) (rec l-1) w e)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w e))
    (mul (rec eB) (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 337)
  (parent 180)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA l-1 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 eB)
  (absent (eB (mul eA (rec l-1) w e)))
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w e))
    (mul eA (rec l-1) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 338)
  (parent 180)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA eB rndx)
    (w expt) (e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w e l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1)
    (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-1)
  (absent (l-1 (mul eA (rec eB) w e)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 339)
  (parent 180)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l))) kh)))
  (neq ((exp (gen) l) (gen)) ((exp (gen) (mul (rec eA) eB l)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-1 l)
      (w (mul (rec eA) eB l (rec l-0))) (l-2 l-0)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0))))))
  (label 340)
  (parent 181)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (eA eB l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l l-1))) kh)))
  (neq ((exp (gen) (mul l l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l l-1)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0 l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l (rec l-0) l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 341)
  (parent 181)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (absent (eA (mul (rec eB) (rec l) w l-0)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l) w l-0))
    (mul (rec eB) (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 342)
  (parent 181)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (absent (eB (mul eA (rec l) w l-0)))
  (operation nonce-test (added-absence eB (mul eA (rec l) w l-0))
    (mul eA (rec l) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 343)
  (parent 181)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB rndx) (w expt) (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l l-0)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
  (deflistener (cat (exp (gen) l) (mul w l-0)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l l-0)) (gen))
    ((exp (gen) (mul w l l-0)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (absent (l-0 (mul eA (rec eB) w l)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) w l))
    (mul eA (rec eB) w l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w l)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w l))))
    ((recv (cat (exp (gen) l) (mul w l-0)))
      (send (cat (exp (gen) l) (mul w l-0)))))
  (label 344)
  (parent 181)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA eB l rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA eB l l-0)
  (absent (l-0 (mul eA (rec eB) (rec l) w)))
  (operation nonce-test (added-absence l-0 (mul eA (rec eB) (rec l) w))
    (mul eA (rec eB) (rec l) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul eA (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0))))))
  (label 345)
  (parent 181)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 (rec eA) eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul l-0 (rec eA) eB)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB)))))
  (label 346)
  (parent 182)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 (rec eA) eB l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-0 eB l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul l-0 (rec eA) eB l-1)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (added-strand ltx-gen 2) l-1 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 347)
  (parent 182)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA))))))
  (label 348)
  (parent 182)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))))
  (label 349)
  (parent 182)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 350)
  (parent 183)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-1))) kh)))
  (neq ((exp (gen) (mul l-0 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-1)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 351)
  (parent 183)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0 l-2))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-0 l-2))) kh)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 352)
  (parent 183)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 353)
  (parent 183)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 354)
  (parent 183)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-0 self) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1)
    (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self)))))
  (label 355)
  (parent 183)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self)))))
  (label 356)
  (parent 183)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1)))))
  (label 357)
  (parent 184)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1))) kh)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (gen) (mul (rec eA) eB l-1 l-1)))
      (send (cat (gen) (mul (rec eA) eB l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 358)
  (parent 184)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul (rec eA) eB l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul (rec eA) eB l-1 l-2)))
      (send (cat (gen) (mul (rec eA) eB l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 359)
  (parent 184)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) (rec l-1) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w))
    (mul (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))))
  (label 360)
  (parent 184)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (absent (eB (mul eA (rec l-1) w)))
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w))
    (mul eA (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB)))))
  (label 361)
  (parent 184)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (absent (l-1 (mul eA (rec eB) w)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1)))))
  (label 362)
  (parent 184)
  (unrealized (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul eB l-1 (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul eB l-1 (rec e))) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 e)
  (operation nonce-test
    (contracted (eB-0 eB) (l-2 l-1) (w (mul eB l-1 (rec e) (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-1 (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e))))))
  (label 363)
  (parent 185)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul eB (rec e) l-1 l-1))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1))) kh)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen))
    ((exp (gen) (mul eB (rec e) l-1 l-1)) (gen)) (D B) (e eB) (l-0 l)
    (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB (rec e) l-1 l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1)))
      (send (cat (exp (gen) e) (mul eB (rec e) (rec e) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 364)
  (parent 185)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB l-1 e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul eB l-1 (rec e) l-2))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l B) (l-0 D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul eB l-1 (rec e) l-2)) (gen)) (D B) (e eB) (l-0 l)
    (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul eB l-1 (rec e) l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul eB l-1 (rec e) (rec e) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 365)
  (parent 185)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 rndx) (w expt) (l-1 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-1 l) (e l-1) (eB l) (e l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (absent (eB (mul (rec l-0) w e e)))
  (operation nonce-test (added-absence eB (mul (rec l-0) w e e))
    (mul (rec l-0) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))))
  (label 366)
  (parent 185)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-0 D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (absent (l-1 (mul (rec eB) w e e)))
  (operation nonce-test (added-absence l-1 (mul (rec eB) w e e))
    (mul (rec eB) w e e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 367)
  (parent 185)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-1 l) (e l-1)
    (eB l) (e l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (absent (e (mul (rec eB) (rec l-0) w)))
  (operation nonce-test (added-absence e (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e))))))
  (label 368)
  (parent 185)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e eA l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul e (rec eA) l-1)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (l-2 l-1) (w (mul (rec eA) l-1))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1)))))
  (label 369)
  (parent 186)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e eA l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1 l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-1))) kh)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen))
    ((exp (gen) (mul e (rec eA) l-1 l-1)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1 l-1)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 370)
  (parent 186)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 e eA l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e (rec eA) l-1 l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eA) l-1 l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul e (rec eA) l-1 l-2)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 e eA l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) l-1 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) l-1 l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 371)
  (parent 186)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 rndx) (w expt) (l-1 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-1) (eA l) (e l-1) (eA l-1) (e l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (absent (eA (mul (rec l-0) w)))
  (operation nonce-test (added-absence eA (mul (rec l-0) w))
    (mul (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA))))))
  (label 372)
  (parent 186)
  (unrealized (0 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1))) kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (absent (l-1 (mul eA w)))
  (operation nonce-test (added-absence l-1 (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))))
  (label 373)
  (parent 186)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB l-2 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) kh)))
  (neq ((exp (gen) l-2) (gen)) ((exp (gen) (mul (rec eA) eB l-2)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2)
      (w (mul (rec eA) eB l-2 (rec e))) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e))))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 374)
  (parent 187)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-2 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 0) (5 0))
    ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2))) kh)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2)))
      (send (cat (exp (gen) e) (mul (rec eA) eB (rec e) l-2 l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 375)
  (parent 187)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer self-1 name) (hl base)
    (l l-0 l-1 eA eB l-2 e l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-2 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (fn-of ("principal-of" (l-3 self-1) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-3))) kh)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen))
    ((exp (gen) (mul (rec eA) eB l-2 l-3)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3)))
      (send (cat (exp (gen) e) (mul (rec eA) eB l-2 (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 376)
  (parent 187)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 eA)
  (absent (eA (mul (rec eB) (rec l-2) w e)))
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-2) w e))
    (mul (rec eB) (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 377)
  (parent 187)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 eB)
  (absent (eB (mul eA (rec l-2) w e)))
  (operation nonce-test (added-absence eB (mul eA (rec l-2) w e))
    (mul eA (rec l-2) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 378)
  (parent 187)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen))
    ((exp (gen) (mul w e l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (absent (l-2 (mul eA (rec eB) w e)))
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w e))
    (mul eA (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 379)
  (parent 187)
  (unrealized (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 380)
  (parent 188)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1))) kh)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 381)
  (parent 188)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-0 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-2))) kh)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)) (D B) (eA eB) (l l-1)
    (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 382)
  (parent 188)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-1)
    (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 383)
  (parent 188)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 384)
  (parent 188)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 385)
  (parent 188)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 386)
  (parent 188)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul (rec eA) eB l-0)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-0)
      (w (mul (rec eA) eB l-0 (rec l-1))) (l-3 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1))))))
  (label 387)
  (parent 189)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-1 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1))) kh)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eA) eB (rec l-0) l-1 l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 388)
  (parent 189)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l eA eB l-0 l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-0 l-2))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-0 l-2))) kh)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-0 l-2)) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-0 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-0 (rec l-1) l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 389)
  (parent 189)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-0) w l-1))
    (mul (rec eB) (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 390)
  (parent 189)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1)
    (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w l-1))
    (mul eA (rec l-0) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 391)
  (parent 189)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w l-0))
    (mul eA (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))))
  (label 392)
  (parent 189)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (operation nonce-test
    (added-absence l-1 (mul eA (rec eB) (rec l-0) w))
    (mul eA (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1))))))
  (label 393)
  (parent 189)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul l-1 (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB)))))
  (label 394)
  (parent 190)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-1)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (5 0)) ((6 1) (4 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-1))) kh)))
  (neq ((exp (gen) (mul l-1 l-1)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-1)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-1)) (send (cat (exp (gen) l-1) l-1)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 395)
  (parent 190)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eA) eB l-2)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB l-2))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 396)
  (parent 190)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA))))))
  (label 397)
  (parent 190)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))))
  (label 398)
  (parent 190)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-1)
      (w (mul (rec eA) eB l-1 (rec l-2))) (l-4 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 399)
  (parent 191)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-2 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (5 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2))) kh)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eA) eB (rec l-1) l-2 l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 400)
  (parent 191)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (6 0)) ((7 1) (4 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 401)
  (parent 191)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 self-1 name)
    (l l-0 eA eB l-1 l-2 l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-1 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener
    (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 0) (6 0)) ((8 1) (4 0)))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-1 l-3))) kh)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-3)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul (rec eA) eB l-1 (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 402)
  (parent 191)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (absent (eA (mul (rec eB) (rec l-1) w l-2)))
  (operation nonce-test
    (added-absence eA (mul (rec eB) (rec l-1) w l-2))
    (mul (rec eB) (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 403)
  (parent 191)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-2)) (gen))
    ((exp (gen) (mul w l-2 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (absent (eB (mul eA (rec l-1) w l-2)))
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w l-2))
    (mul eA (rec l-1) w l-2) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 404)
  (parent 191)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen))
    ((exp (gen) (mul w l-1 l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (absent (l-2 (mul eA (rec eB) w l-1)))
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w l-1))
    (mul eA (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))))
  (label 405)
  (parent 191)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 rndx) (w expt)
    (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)))
  (operation nonce-test
    (added-absence l-2 (mul eA (rec eB) (rec l-1) w))
    (mul eA (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))))
  (label 406)
  (parent 191)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eB rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 407)
  (parent 193)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA rndx) (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 408)
  (parent 194)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (absent (eB (mul w e e)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 409)
  (parent 196)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (absent (e (mul (rec eB) w)))
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 410)
  (parent 197)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) w))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA w))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 411)
  (parent 199)
  (unrealized (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eB rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (absent (eA (mul (rec eB) w e)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w e)))
      (send (cat (gen) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 412)
  (parent 202)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (absent (eB (mul eA w e)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w e))) (send (cat (gen) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 413)
  (parent 203)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w l-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 414)
  (parent 205)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w l-0)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 415)
  (parent 206)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w l-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-0)))
      (send (cat (gen) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 416)
  (parent 209)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (gen) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w l-0)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (gen) (mul eA w l-0)))
      (send (cat (gen) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 417)
  (parent 210)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w l-1)))
      (send (cat (gen) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 418)
  (parent 214)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (absent (eB (mul eA w l-1)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w l-1)))
      (send (cat (gen) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 419)
  (parent 215)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (absent (l-1 (mul eA (rec eB) w)))
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 420)
  (parent 216)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 e rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec e) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul (rec e) (rec e) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 421)
  (parent 219)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l l-0 eA rndx) (w expt) (e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (gen) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e)
  (absent (e (mul eA w)))
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (gen) (mul w e e))) (send (cat (gen) (mul w e e))))
    ((recv e) (send e)))
  (label 422)
  (parent 220)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e e-0 e-0))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e e-0))) (l l) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e e-0 e-0)))
        kh)))
  (neq ((exp (gen) (mul w e e e-0)) (gen))
    ((exp (gen) (mul w e e-0 e-0)) (gen)) (D B) (e e-0) (l-0 l) (e l-0)
    (e-0 l) (e l) (e-0 l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e e-0)
  (absent (e-0 (mul w e e)))
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e e-0 e-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e e e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul w e e)))
      (send (cat (exp (gen) e-0) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv e-0) (send e-0)))
  (label 423)
  (parent 223)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e-0)))) (l l-0) (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec e) w)) (gen))
    ((exp (gen) (mul w (rec e-0))) (gen)) (D B) (e-0 e) (l-0 l)
    (e-0 l-0) (e l) (e-0 l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 e-0)
  (absent (e-0 (mul (rec e) (rec e) w)))
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 424)
  (parent 224)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul w (rec e))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul w (rec e))))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul w (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 425)
  (parent 227)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (absent (e (mul eA w)))
  (operation nonce-test (added-listener e) (mul w e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w e)))
      (send (cat (exp (gen) e) (mul w e)))) ((recv e) (send e)))
  (label 426)
  (parent 228)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 e rndx)
    (w expt) (e-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w e-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e-0))) kh)))
  (neq ((exp (gen) (mul (rec e) w e-0)) (gen))
    ((exp (gen) (mul w e-0 (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul (rec e) (rec e) w e-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w e-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w e-0))))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv eA) (send eA)))
  (label 427)
  (parent 231)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA rndx)
    (w expt) (e e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e-0 e-0))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e e-0))) (l l-0) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e-0) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e-0 e-0)))
        kh)))
  (neq ((exp (gen) (mul eA w e e-0)) (gen))
    ((exp (gen) (mul w e e-0 e-0)) (gen)) (D B) (eA e-0) (l l-0) (eA l)
    (e-0 l-0) (eA l-0) (e-0 l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e-0)
  (absent (e-0 (mul eA w e)))
  (operation nonce-test (added-listener e-0) (mul w e-0 e-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e-0 e-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul eA w e e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e-0) (mul eA w e)))
      (send (cat (exp (gen) e-0) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w e-0 e-0)))
      (send (cat (exp (gen) e) (mul w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv e-0) (send e-0)))
  (label 428)
  (parent 232)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w l-0))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (absent (eA (mul (rec e) (rec e) w l-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 429)
  (parent 235)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen))
    ((exp (gen) (mul w l-0 e e)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (absent (e (mul eA w l-0)))
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-0 e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 430)
  (parent 236)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l e rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec e) w l-0))) (l l)
    (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec e) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA e) (l-0 l)
    (eA l-0) (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l e l-0 eA)
  (absent (eA (mul (rec e) (rec e) w l-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul (rec e) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-0)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 431)
  (parent 240)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 e e))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 e))) (l l) (e e))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w e e)))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 e e)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 e)) (gen))
    ((exp (gen) (mul w l-0 e e)) (gen)) (D B) (eA e) (l-0 l) (eA l-0)
    (e l) (eA l) (e l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e)
  (absent (e (mul eA w l-0)))
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) (mul eA w l-0 e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-0)))
      (send (cat (exp (gen) e) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w e e)))
      (send (cat (exp (gen) l-0) (mul w e e)))) ((recv e) (send e)))
  (label 432)
  (parent 241)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec e) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (absent (eA (mul (rec e) (rec e) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 433)
  (parent 246)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1 e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 e e)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1 e)) (gen))
    ((exp (gen) (mul w l-1 e e)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (absent (e (mul eA w l-1)))
  (operation nonce-test (added-listener e) (mul w e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1 e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w l-1)))
      (send (cat (exp (gen) e) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv e) (send e)))
  (label 434)
  (parent 247)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (absent (l-1 (mul eA (rec e) (rec e) w)))
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 435)
  (parent 248)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eB e rndx)
    (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB eA)
  (absent (eA (mul (rec eB) (rec e) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 436)
  (parent 251)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 l-1 eA e rndx)
    (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (absent (eB (mul eA (rec e) w)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 437)
  (parent 252)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 e rndx) (w expt)
    (l-1 e-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e-0 eB))) (l l-1)
    (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 e-0))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0 e-0 eB)))
        kh)))
  (neq ((exp (gen) (mul w e-0 e-0)) (gen))
    ((exp (gen) (mul w e-0 eB)) (gen)) (D B) (e-0 eB) (l-1 l-0)
    (e-0 l-1) (eB l-0) (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e-0 eB)
  (absent (eB (mul (rec e) w e-0 e-0)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w e-0 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e-0 e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w e-0 e-0)))
      (send (cat (exp (gen) e) (mul (rec e) w e-0 e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 438)
  (parent 255)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB e rndx)
    (w expt) (l-1 e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec e-0)))) (l l-1)
    (e e-0))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
  (deflistener e-0)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l-1 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e-0))) (gen)) (D B) (e-0 eB) (l-1 l-0)
    (e-0 l-1) (eB l-0) (e-0 l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 e-0)
  (absent (e-0 (mul (rec eB) (rec e) w)))
  (operation nonce-test (added-listener e-0) (mul w (rec e-0) (rec e-0))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) e-0)) (recv (exp (gen) (mul w (rec e-0))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0))))
      (send (cat (exp (gen) e-0) (mul w (rec e-0) (rec e-0)))))
    ((recv e-0) (send e-0)))
  (label 439)
  (parent 256)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base)
    (l l-0 l-1 e eA e-0 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul e (rec eA) e-0))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e-0)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e-0) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l)
    (e e-0))
  (deflistener (cat (exp (gen) e) (mul (rec eA) e-0)))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul e e-0))) kh)))
  (neq ((exp (gen) e-0) (gen)) ((exp (gen) (mul e (rec eA) e-0)) (gen))
    (D B) (eA e) (l-0 l-1) (eA l-0) (e l-1) (eA l-1) (e l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e eA)
  (operation nonce-test (added-listener eA) (mul (rec eA) e-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e (rec eA) e-0)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e)) (recv (exp (gen) e-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e-0) (one)))
      (send (cat (exp (gen) e-0) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e-0)))
    ((recv (cat (exp (gen) e) (mul (rec eA) e-0)))
      (send (cat (exp (gen) e) (mul (rec eA) e-0))))
    ((recv eA) (send eA)))
  (label 440)
  (parent 257)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 e rndx) (w expt)
    (l-1 e-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l-1) (e e-0))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e-0 (rec eA))) (gen))
    (D B) (eA e-0) (l-0 l-1) (eA l-0) (e-0 l-1) (eA l-1) (e-0 l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 e-0 eA)
  (absent (eA (mul (rec e) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) e-0)) (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 441)
  (parent 259)
  (unrealized (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB rndx)
    (w expt) (l-1 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec eB) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) w)))
      (send (cat (exp (gen) e) (mul (rec eB) w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 442)
  (parent 263)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA rndx)
    (w expt) (l-1 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 443)
  (parent 264)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eB e rndx) (w expt) (e-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e-0))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e-0)) (gen))
    ((exp (gen) (mul w e-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB eA)
  (absent (eA (mul (rec eB) (rec e) w e-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w (rec eA))))
      (send (cat (exp (gen) e-0) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eA) (send eA)))
  (label 444)
  (parent 267)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 peer-0 name) (hl hl-0 base)
    (l l-0 l-1 l-2 eA e rndx) (w expt) (e-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w e-0))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w e-0)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-2)
    (e e-0))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 2) (6 0))
    ((8 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w e-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w e-0)) (gen))
    ((exp (gen) (mul w e-0 eB)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB)
  (absent (eB (mul eA (rec e) w e-0)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w e-0)))
      (send (cat (exp (gen) e) (mul eA (rec e) w e-0))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) (mul w eB)))
      (send (cat (exp (gen) e-0) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))) ((recv eB) (send eB)))
  (label 445)
  (parent 268)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB e rndx)
    (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) (rec e) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 446)
  (parent 271)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA e rndx)
    (w expt) (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 eB)
  (absent (eB (mul eA (rec e) w l-1)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 447)
  (parent 272)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB e rndx)
    (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l-0)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-1 l-0)
    (eA l-1) (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) (rec e) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 448)
  (parent 276)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA e rndx)
    (w expt) (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-1)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 D) (l-0 B))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-1 l-0) (eA l-1)
    (eB l-0) (eA l-0) (eB l-1))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 eA l-1 eB)
  (absent (eB (mul eA (rec e) w l-1)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-1)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-1))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 449)
  (parent 277)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-2))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2 (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB l-2 eA)
  (absent (eA (mul (rec eB) (rec e) w l-2)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 450)
  (parent 282)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-2 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-2)) (gen))
    ((exp (gen) (mul w l-2 eB)) (gen)) (D B) (eA eB) (l-0 l-1) (eA l-0)
    (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA l-2 eB)
  (absent (eB (mul eA (rec e) w l-2)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w l-2)))
      (send (cat (exp (gen) e) (mul eA (rec e) w l-2))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 451)
  (parent 283)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 0) (4 0))
    ((7 0) (8 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (absent (l-2 (mul eA (rec eB) (rec e) w)))
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 452)
  (parent 284)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) (rec l-0) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 453)
  (parent 287)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA (rec l-0) w)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 454)
  (parent 288)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec l) w e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (absent (eB (mul (rec l) w e e)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w e e)))
      (send (cat (exp (gen) l) (mul (rec l) w e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 455)
  (parent 292)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 e)
  (absent (e (mul (rec eB) (rec l) w)))
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 456)
  (parent 294)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul w (rec l-0))))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 457)
  (parent 297)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB l-1 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 eA)
  (absent (eA (mul (rec eB) (rec l-1) w e)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 458)
  (parent 301)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA l-1 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 eB)
  (absent (eB (mul eA (rec l-1) w e)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 459)
  (parent 302)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 460)
  (parent 306)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 461)
  (parent 307)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (absent (eA (mul (rec eB) (rec l) w l-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 462)
  (parent 310)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (absent (eB (mul eA (rec l) w l-0)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 463)
  (parent 311)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 464)
  (parent 317)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 465)
  (parent 318)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 466)
  (parent 320)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB l-0 rndx) (w expt) (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) (rec l-0) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 467)
  (parent 323)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA l-0 rndx) (w expt) (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA (rec l-0) w)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 468)
  (parent 324)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eB)
  (absent (eB (mul w (rec l-0) e e)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) e e)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) e e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 469)
  (parent 328)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e)
  (absent (e (mul (rec eB) w (rec l-0))))
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0))))
      (send (cat (exp (gen) l-0) (mul (rec eB) w (rec l-0)))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 470)
  (parent 330)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l rndx) (w expt) (l-0 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec l) w)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e eA)
  (absent (eA (mul (rec l) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec l) w)))
      (send (cat (exp (gen) l) (mul (rec l) w))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 471)
  (parent 333)
  (unrealized (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eB l-1 rndx)
    (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e (rec eA)))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w e))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-1 eA)
  (absent (eA (mul (rec eB) (rec l-1) w e)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 472)
  (parent 337)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer name) (hl base) (l l-0 eA l-1 rndx)
    (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-0)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0)))
  (fn-of ("principal-of" (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-1 eB)
  (absent (eB (mul eA (rec l-1) w e)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w e)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w e))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 473)
  (parent 338)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eB l rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eB l l-0 eA)
  (absent (eA (mul (rec eB) (rec l) w l-0)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul (rec eB) (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 474)
  (parent 342)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (eA l rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l) (mul eA (rec l) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen eA l l-0 eB)
  (absent (eB (mul eA (rec l) w l-0)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l) (mul eA (rec l) w l-0)))
      (send (cat (exp (gen) l) (mul eA (rec l) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 475)
  (parent 343)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eB rndx) (w expt) (l-0 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec eA))))
      (send (cat (exp (gen) l-0) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 476)
  (parent 348)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D name) (l eA rndx) (w expt) (l-0 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA w)))
  (deflistener (cat (exp (gen) l-0) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)))
  (fn-of ("principal-of" (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0)) (gen))
    ((exp (gen) (mul w l-0 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA w)))
      (send (cat (exp (gen) l-0) (mul eA w))))
    ((recv (cat (exp (gen) l-0) (mul w eB)))
      (send (cat (exp (gen) l-0) (mul w eB)))) ((recv eB) (send eB)))
  (label 477)
  (parent 349)
  (unrealized (1 3) (4 0) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eA) (send eA)))
  (label 478)
  (parent 353)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-listener eB) (mul w eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv eB) (send eB)))
  (label 479)
  (parent 354)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (7 0))
    ((7 1) (5 0)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (operation nonce-test (added-listener l-1) (mul w (rec l-1)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-1) (send l-1)))
  (label 480)
  (parent 356)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB l-1 rndx) (w expt)
    (eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) (rec l-1) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w (rec eA))))
      (send (cat (gen) (mul w (rec eA))))) ((recv eA) (send eA)))
  (label 481)
  (parent 360)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA l-1 rndx) (w expt)
    (eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (absent (eB (mul eA (rec l-1) w)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w eB))) (send (cat (gen) (mul w eB))))
    ((recv eB) (send eB)))
  (label 482)
  (parent 361)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB))))
  (deflistener (cat (gen) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB))))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1)) (gen)) ((exp (gen) l-1) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) l-1)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB))))
      (send (cat (exp (gen) l-1) (mul eA (rec eB)))))
    ((recv (cat (gen) l-1)) (send (cat (gen) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 483)
  (parent 362)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (absent (l-1 (mul eA (rec eB) w)))
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1)))
  (label 484)
  (parent 362)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 rndx) (w expt) (l-1 e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e eB))) (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e e))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e e eB))) kh)))
  (neq ((exp (gen) (mul w e e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (e eB) (l-1 l) (e l-1) (eB l) (e l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eB)
  (absent (eB (mul (rec l-0) w e e)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w e e)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w e e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB)))) ((recv eB) (send eB)))
  (label 485)
  (parent 366)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-1))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) e e)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul (rec eB) e e)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) e e l-1)) (gen))
    ((exp (gen) (mul e l-1)) (gen)) (D B) (e eB) (l-0 l) (e l-0) (eB l)
    (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) e e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) e e))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 486)
  (parent 367)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l B) (l-0 D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (absent (l-1 (mul (rec eB) w e e)))
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 487)
  (parent 367)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec e)))) (l l-1) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec e) (rec e))))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l B) (l-1 D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec e))) (gen)) (D B) (e eB) (l-1 l) (e l-1)
    (eB l) (e l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (absent (e (mul (rec eB) (rec l-0) w)))
  (operation nonce-test (added-listener e) (mul w (rec e) (rec e))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w (rec e) (rec e)))))
    ((recv e) (send e)))
  (label 488)
  (parent 368)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 rndx) (w expt) (l-1 e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-1) (e e))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) w) (gen)) ((exp (gen) (mul w e (rec eA))) (gen))
    (D B) (eA e) (l l-1) (eA l) (e l-1) (eA l-1) (e l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 e eA)
  (absent (eA (mul (rec l-0) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) w)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 489)
  (parent 372)
  (unrealized (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) eA))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 eA))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-1))) kh)))
  (neq ((exp (gen) (mul eA l-1)) (gen)) ((exp (gen) (mul e l-1)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) eA)) (send (cat (exp (gen) l-1) eA)))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 490)
  (parent 373)
  (unrealized (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1))) kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (absent (l-1 (mul eA w)))
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 491)
  (parent 373)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eB l-2 rndx) (w expt) (e eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w (rec eA))))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e)) (gen))
    ((exp (gen) (mul w e (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-2 eA)
  (absent (eA (mul (rec eB) (rec l-2) w e)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w (rec eA))))
      (send (cat (exp (gen) e) (mul w (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eA) (send eA)))
  (label 492)
  (parent 377)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA l-2 rndx) (w expt) (e eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w eB)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e eB))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-2 eB)
  (absent (eB (mul eA (rec l-2) w e)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec l-2) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec l-2) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w eB)))
      (send (cat (exp (gen) e) (mul w eB))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv eB) (send eB)))
  (label 493)
  (parent 378)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) e)))
  (deflistener (cat (exp (gen) e) l-2))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 0) (4 0))
    ((7 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) e)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA e l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) e l-2)) (gen))
    ((exp (gen) (mul e l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) e))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 494)
  (parent 379)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 peer name) (hl base)
    (l l-0 l-1 eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (0 3))
    ((7 2) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen))
    ((exp (gen) (mul w e l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (absent (l-2 (mul eA (rec eB) w e)))
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2)))
  (label 495)
  (parent 379)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-1)
    (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 496)
  (parent 383)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 B) (l D) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 497)
  (parent 384)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen))
    ((exp (gen) (mul l-0 l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 498)
  (parent 385)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 499)
  (parent 385)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 (rec eA)))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (absent (eA (mul (rec eB) (rec l-0) w l-1)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 500)
  (parent 390)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 eB))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA w l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 D) (l B) (l-0 self))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1)
    (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (6 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (absent (eB (mul eA (rec l-0) w l-1)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w l-1))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 501)
  (parent 391)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
  (deflistener (cat (exp (gen) l-0) l-1))
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 1) (5 0)))
  (absent (l-1 (mul eA (rec eB) l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-0 l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-0 l-1)) (gen))
    ((exp (gen) (mul l-0 l-1)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 7 ltx-gen 2) l-2 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) l-0))))
    ((recv (cat (exp (gen) l-0) l-1)) (send (cat (exp (gen) l-0) l-1)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 502)
  (parent 392)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (7 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-0 D) (l B) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (operation nonce-test (added-listener l-1) (mul w l-1) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1)))
  (label 503)
  (parent 392)
  (unrealized (1 3) (4 0) (7 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w (rec eA))))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (7 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (absent (eA (mul (rec eB) w)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec eA)))))
    ((recv eA) (send eA)))
  (label 504)
  (parent 397)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul w eB)))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (7 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (absent (eB (mul eA w)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w eB)))
      (send (cat (exp (gen) l-1) (mul w eB)))) ((recv eB) (send eB)))
  (label 505)
  (parent 398)
  (unrealized (1 3) (4 0) (7 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec eA))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (absent (eA (mul (rec eB) (rec l-1) w l-2)))
  (operation nonce-test (added-listener eA) (mul w (rec eA)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec eA)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eA) (send eA)))
  (label 506)
  (parent 403)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w eB)))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-2)) (gen))
    ((exp (gen) (mul w l-2 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (absent (eB (mul eA (rec l-1) w l-2)))
  (operation nonce-test (added-listener eB) (mul w eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w eB)))
      (send (cat (exp (gen) l-2) (mul w eB))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv eB) (send eB)))
  (label 507)
  (parent 404)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
  (deflistener (cat (exp (gen) l-1) l-2))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 0) (4 0)) ((7 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA l-1 l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) l-1 l-2)) (gen))
    ((exp (gen) (mul l-1 l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-3 (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) l-1 l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) l-1))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 508)
  (parent 405)
  (unrealized (1 3) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (8 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (6 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen))
    ((exp (gen) (mul w l-1 l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (absent (l-2 (mul eA (rec eB) w l-1)))
  (operation nonce-test (added-listener l-2) (mul w l-2) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 509)
  (parent 405)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 rndx) (w expt)
    (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (defstrand ltx-gen 1 (self self-0) (l l-2))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 0) (4 0)) ((7 0) (8 0)) ((8 1) (6 0)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)))
  (operation nonce-test (added-listener l-2) (mul w (rec l-2)) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0))))
    ((recv l-2) (send l-2)))
  (label 510)
  (parent 406)
  (unrealized (1 3) (4 0) (8 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA (rec eB) w)))
      (send (cat (gen) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 511)
  (parent 420)
  (unrealized (1 3) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA e rndx) (w expt) (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec e) (rec e) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec e) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA e l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 512)
  (parent 435)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB e rndx) (w expt) (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) (rec e) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 513)
  (parent 452)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 514)
  (parent 466)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB l-0 rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) w)) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 6 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 515)
  (parent 480)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB rndx) (w expt)
    (l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (gen) (mul w l-1))) (send (cat (gen) (mul w l-1))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 516)
  (parent 484)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-1))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w e e l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul (rec eB) w e e)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e e l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w e e)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 517)
  (parent 487)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1))) kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 518)
  (parent 491)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA eB rndx) (w expt) (e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w e)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w e l-2)) (gen))
    ((exp (gen) (mul w e l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2)))
      (send (cat (exp (gen) e) (mul w l-2))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 519)
  (parent 495)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 520)
  (parent 499)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA (rec eB) w l-0 l-1)))
    (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1)) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (displaced 5 8 ltx-gen 2) l-1 (7 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-0 l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1)))
      (send (cat (exp (gen) l-0) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 521)
  (parent 503)
  (unrealized (1 3) (4 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2)))
        kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1 l-2)) (gen))
    ((exp (gen) (mul w l-1 l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB))
      (recv (exp (gen) (mul eA (rec eB) w l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2)))
      (send (cat (exp (gen) l-1) (mul w l-2))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 522)
  (parent 509)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 rndx) (w expt)
    (l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) w)) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) (rec l-1) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w)) (gen)) ((exp (gen) w) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 7 9 ltx-gen 2) l-2 (8 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) w)) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 523)
  (parent 510)
  (unrealized (1 3) (4 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) kh)))
  (neq ((exp (gen) l-2) (gen)) ((exp (gen) (mul (rec eA) eB l-2)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) l-2)) (send (cat (gen) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 524)
  (parent 511)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 525)
  (parent 511)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 526)
  (parent 511)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 l-1 eA e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) e)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (one)))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e)
  (operation nonce-test
    (contracted (eA-0 eA) (e-0 e) (w (mul (rec eA) e e))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 527)
  (parent 512)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) e e l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul e l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e e l-2))) kh)))
  (neq ((exp (gen) (mul e l-2)) (gen))
    ((exp (gen) (mul (rec eA) e e l-2)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) e e l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul e l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) l-2)) (send (cat (exp (gen) e) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) e e l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 528)
  (parent 512)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec e) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec e) (rec e) w))
    (mul (rec e) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 529)
  (parent 512)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-absence e (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 530)
  (parent 512)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 l-2 eA eB e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) e)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e))) kh)))
  (neq ((exp (gen) e) (gen)) ((exp (gen) (mul (rec eA) eB e)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (e-0 e) (w (mul (rec eA) eB e)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) e)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 531)
  (parent 513)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB e l-3)))
    (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-3))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) l-3))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0)) ((9 1) (4 0)))
  (absent (l-2 l-3))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eB e l-3))) kh)))
  (neq ((exp (gen) (mul e l-3)) (gen))
    ((exp (gen) (mul (rec eA) eB e l-3)) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB e l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul e l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) l-3)) (send (cat (exp (gen) e) l-3)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3)))
      (send (cat (exp (gen) l-2) (mul (rec l-2) (rec eA) eB e l-3))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 532)
  (parent 513)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-2 (mul (rec eB) (rec e) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec e) w))
    (mul (rec eB) (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 533)
  (parent 513)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA (rec e) w)) (l-2 (mul eA (rec e) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec e) w))
    (mul eA (rec e) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 534)
  (parent 513)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 535)
  (parent 514)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul (rec eA) eB l-1 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)) (D B) (eA eB) (l l-1)
    (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 536)
  (parent 514)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 537)
  (parent 514)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 538)
  (parent 514)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 539)
  (parent 514)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)))
  (fn-of ("principal-of" (l-0 self) (l B) (l-1 D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul (rec eA) eB l-1)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-2 l-1) (w (mul (rec eA) eB l-1)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 540)
  (parent 515)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eA) eB l-1 l-2))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener
    (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eB l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-1 l-2)) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-1 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2)))
      (send (cat (exp (gen) l-0) (mul (rec l-0) (rec eA) eB l-1 l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 541)
  (parent 515)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-0) w))
    (mul (rec eB) (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 542)
  (parent 515)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-0) w))
    (mul eA (rec l-0) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 543)
  (parent 515)
  (unrealized (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-0 self) (l B) (l-1 D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1)
    (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-0) (mul w (rec l-0) l-1)))
      (send (cat (exp (gen) l-0) (mul w (rec l-0) l-1))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 544)
  (parent 515)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 l-1 eA eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (gen) (mul l-1 (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul l-1 (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (gen) (mul l-1 (rec eA) eB)))
      (send (cat (gen) (mul l-1 (rec eA) eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 545)
  (parent 516)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (gen) (mul l-1 (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (gen) (mul l-1 (rec eA) eB l-2)))
      (send (cat (gen) (mul l-1 (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 546)
  (parent 516)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-1 (rec eA))))
      (send (cat (gen) (mul w l-1 (rec eA))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 547)
  (parent 516)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (gen) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (gen) (mul w l-1 eB)))
      (send (cat (gen) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 548)
  (parent 516)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 l-1 eB e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 eB (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) l-1)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul l-1 eB (rec e))) (gen))
    (D B) (e eB) (l-0 l) (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e)
  (operation nonce-test
    (contracted (eB-0 eB) (w (mul eB (rec e) (rec e))) (e-0 e)) (one)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eB (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 549)
  (parent 517)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eB e l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 eB (rec e) l-2))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul l-1 eB (rec e) l-2)) (gen)) (D B) (e eB) (l-0 l)
    (e l-0) (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eB e l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 eB (rec e) l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2)))
      (send (cat (exp (gen) e) (mul l-1 eB (rec e) (rec e) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 550)
  (parent 517)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l rndx) (w expt) (l-0 e l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-1 eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul w e e)) (l-1 (mul w e e)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul w e e l-1)) (gen))
    ((exp (gen) (mul w e l-1 eB)) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-absence eB (mul w e e)) (mul w e e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 551)
  (parent 517)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt) (l-0 l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (e (mul (rec eB) w)) (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-absence e (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 552)
  (parent 517)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (one)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul e l-1 (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (contracted (eA-0 eA) (w (rec eA))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 553)
  (parent 518)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 e l-1 eA l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1 (rec eA) l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1 l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul e l-1 (rec eA) l-2)) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA) l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA) l-2)))
      (send (cat (exp (gen) e) (mul l-1 (rec eA) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 554)
  (parent 518)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l rndx) (w expt) (l-0 e l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA w) (l-1 w))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) kh)))
  (neq ((exp (gen) (mul w l-1)) (gen))
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-absence eA w) w (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 555)
  (parent 518)
  (unrealized (0 3) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) e) (mul w l-1)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (5 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (4 0)))
  (absent (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-1))) kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w e l-1)) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 e l-1)
  (operation nonce-test (added-listener eA) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) e) (mul w l-1)))
      (send (cat (exp (gen) e) (mul w l-1)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 556)
  (parent 518)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 l-2 eA eB e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (one)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eB))) kh)))
  (neq ((exp (gen) l-2) (gen)) ((exp (gen) (mul l-2 (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec e)))
      (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((recv (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e))))
      (send (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 557)
  (parent 519)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 self-1 name) (hl base)
    (l l-0 l-1 l-2 eA eB e l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-2 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-2 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (deflistener (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0)) ((9 1) (4 0)))
  (absent (l-2 l-3))
  (fn-of ("principal-of" (l-3 self-1) (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-2 eB l-3))) kh)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen))
    ((exp (gen) (mul l-2 (rec eA) eB l-3)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-2 (rec eA) eB l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((recv (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3)))
      (send (cat (exp (gen) e) (mul l-2 (rec eA) eB (rec e) l-3))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 558)
  (parent 519)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w e)) (l-2 (mul (rec eB) w e)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e l-2)) (gen))
    ((exp (gen) (mul w e l-2 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w e))
    (mul (rec eB) w e) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 559)
  (parent 519)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w e)) (l-2 (mul eA w e)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w e l-2)) (gen))
    ((exp (gen) (mul w e l-2 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w e)) (mul eA w e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w e)))
      (send (cat (exp (gen) l-2) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eB)))
      (send (cat (exp (gen) e) (mul w l-2 eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 560)
  (parent 519)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-0)) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul l-0 (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-1) (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 561)
  (parent 520)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 l-2))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-2))) kh)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen))
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)) (D B) (eA eB) (l l-1)
    (eA l) (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 562)
  (parent 520)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 563)
  (parent 520)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 564)
  (parent 520)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w l-0))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-0))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-0 self) (l-1 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l l-1) (eA l)
    (eB l-1) (eA l-1) (eB l))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 565)
  (parent 520)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA eB l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 (rec eA) eB))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) l-0)) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (one)))
  (deflistener (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-0 (one)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB))) kh)))
  (neq ((exp (gen) l-0) (gen)) ((exp (gen) (mul l-0 (rec eA) eB)) (gen))
    (D B) (eA eB) (l-1 l) (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-1)))
      (l-2 l-1)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-0)) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1))))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 566)
  (parent 521)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul l-0 (rec eA) eB l-2))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul l-0 l-2))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-0 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul l-0 eB l-2))) kh)))
  (neq ((exp (gen) (mul l-0 l-2)) (gen))
    ((exp (gen) (mul l-0 (rec eA) eB l-2)) (gen)) (D B) (eA eB) (l-1 l)
    (eA l-1) (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-strand ltx-gen 2) l-2 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-0 (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-0 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) l-2)) (send (cat (exp (gen) l-0) l-2)))
    ((recv (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul l-0 (rec eA) eB (rec l-1) l-2))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 567)
  (parent 521)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-0))
    (mul (rec eB) w l-0) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 568)
  (parent 521)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-absence eB (mul eA w l-0)) (mul eA w l-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1)))
  (label 569)
  (parent 521)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA eB rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0))) (l l-1) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul eA (rec eB) w l-0))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-1))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
  (deflistener l-0)
  (defstrand ltx-gen 2 (self self) (l l-0))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)))
  (absent (l-1 (mul eA (rec eB) w)) (l-0 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-0 self) (l-1 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-1)) (exp (gen) (mul eA w l-0))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-0)) (gen))
    ((exp (gen) (mul w l-0)) (gen)) (D B) (eA eB) (l-1 l) (eA l-1)
    (eB l) (eA l) (eB l-1))
  (non-orig (privk B) (privk D) l l-1)
  (precur (5 0) (4 0))
  (uniq-gen l eA eB l-0 l-1)
  (operation nonce-test (added-absence l-1 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-1) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-0)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-1) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w l-0 (rec l-1))))
      (send (cat (exp (gen) l-1) (mul w l-0 (rec l-1)))))
    ((recv l-0) (send l-0))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))) (send l-0)))
  (label 570)
  (parent 521)
  (unrealized (1 3) (4 0) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul l-1 (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB (rec l-2)))
      (l-3 l-2)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2))))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 571)
  (parent 522)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-2))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-2))) kh)))
  (neq ((exp (gen) (mul l-1 l-2)) (gen))
    ((exp (gen) (mul l-1 (rec eA) eB l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 6 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-2)) (send (cat (exp (gen) l-1) l-2)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 572)
  (parent 522)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (5 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-3))) kh)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen))
    ((exp (gen) (mul l-1 (rec eA) eB l-3)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec eA) eB (rec l-2) l-3))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 573)
  (parent 522)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) w l-1)) (l-2 (mul (rec eB) w l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1 l-2)) (gen))
    ((exp (gen) (mul w l-1 l-2 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w l-1))
    (mul (rec eB) w l-1) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1 l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 574)
  (parent 522)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA w l-1)) (l-2 (mul eA w l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1 l-2)) (gen))
    ((exp (gen) (mul w l-1 l-2 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w l-1)) (mul eA w l-1)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1 l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w l-1)))
      (send (cat (exp (gen) l-2) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eB)))
      (send (cat (exp (gen) l-1) (mul w l-2 eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 575)
  (parent 522)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 576)
  (parent 522)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-2)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (one)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-1 (one)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2))) kh)))
  (neq ((exp (gen) l-2) (gen)) ((exp (gen) (mul (rec eA) eB l-2)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (l-3 l-2) (w (mul (rec eA) eB l-2)))
    (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-2)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (one)))
      (send (cat (exp (gen) l-2) (one))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 577)
  (parent 523)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 eA eB l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-2)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-2 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 0) (5 0)) ((8 1) (4 0)))
  (absent (l-1 l-2))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-2))) kh)))
  (neq ((exp (gen) (mul l-2 l-2)) (gen))
    ((exp (gen) (mul (rec eA) eB l-2 l-2)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2)
  (operation nonce-test (displaced 5 9 ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-2)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-2))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-2)) (send (cat (exp (gen) l-2) l-2)))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 578)
  (parent 523)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 self-1 name)
    (l l-0 l-1 eA eB l-2 l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eA) eB l-2 l-3)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-2 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) l-3))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener
    (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 0) (6 0))
    ((9 1) (4 0)))
  (absent (l-1 l-3))
  (fn-of
    ("principal-of" (l-3 self-1) (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eB l-2 l-3))) kh)))
  (neq ((exp (gen) (mul l-2 l-3)) (gen))
    ((exp (gen) (mul (rec eA) eB l-2 l-3)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 l-1 eA eB l-2 l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul (rec eA) eB l-2 l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-2 l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) l-3)) (send (cat (exp (gen) l-2) l-3)))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3)))
      (send (cat (exp (gen) l-1) (mul (rec l-1) (rec eA) eB l-2 l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 579)
  (parent 523)
  (unrealized (0 3) (6 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eA (mul (rec eB) (rec l-1) w))
    (l-2 (mul (rec eB) (rec l-1) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) (rec l-1) w))
    (mul (rec eB) (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 580)
  (parent 523)
  (unrealized (0 3) (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (eB (mul eA (rec l-1) w)) (l-2 (mul eA (rec l-1) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA (rec l-1) w))
    (mul eA (rec l-1) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 581)
  (parent 523)
  (unrealized (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-absence l-2 (mul eA (rec eB) w))
    (mul eA (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)))
  (label 582)
  (parent 523)
  (unrealized (1 3) (4 0) (6 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul (rec eB) w)))
      (send (cat (gen) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 583)
  (parent 525)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (gen) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (gen) (mul eA w))) (send (cat (gen) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 584)
  (parent 526)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e rndx) (w expt) (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec e) w))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul (rec e) (rec e) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec e) (rec e) w)) (l-1 (mul (rec e) (rec e) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec e) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA e) (l l-0) (eA l)
    (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul (rec e) w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec e) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec e) (rec e) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 585)
  (parent 529)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt) (l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) e) (mul eA w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e e))) kh)))
  (neq ((exp (gen) (mul eA w e)) (gen)) ((exp (gen) (mul w e e)) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 e)
  (operation nonce-test (added-listener e) (mul w (rec l-1) e e) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e e))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul eA w e))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA w)))
      (send (cat (exp (gen) e) (mul eA w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) e e)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) e e))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 586)
  (parent 530)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB e rndx) (w expt) (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w (rec eA)))) (l l-0)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l-1)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul (rec eB) (rec e) w)) (l-2 (mul (rec eB) (rec e) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l-1)
    (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eB l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) (rec eA))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul (rec eB) (rec e) w)))
      (send (cat (exp (gen) e) (mul (rec eB) (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 587)
  (parent 533)
  (unrealized (1 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA e rndx) (w expt) (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-1)) (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l-1) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-1))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) e) (mul eA (rec e) w)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 2) (4 0)) ((6 1) (0 3)) ((7 1) (6 0))
    ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul eA (rec e) w)) (l-2 (mul eA (rec e) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l-1) (eA l-0) (eB l-1) (eA l-1) (eB l-0))
  (non-orig (privk B) (privk D) l-0 l-1)
  (precur (6 0) (4 0))
  (uniq-gen l-0 l-1 eA l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-1) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-1) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) e) (mul eA (rec e) w)))
      (send (cat (exp (gen) e) (mul eA (rec e) w))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 588)
  (parent 534)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 589)
  (parent 537)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 590)
  (parent 538)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB l-0 rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) (rec l-0) w))
    (l-1 (mul (rec eB) (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l-0 l) (eA l-0)
    (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-1) (rec eA))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul (rec eB) (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA))))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 591)
  (parent 542)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA l-0 rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA (rec l-0) w)) (l-1 (mul eA (rec l-0) w)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l-0 l) (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-1) eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-0) (mul eA (rec l-0) w)))
      (send (cat (exp (gen) l-0) (mul eA (rec l-0) w))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) eB)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 592)
  (parent 543)
  (unrealized (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eB rndx) (w expt)
    (l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (gen) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (gen) (mul w l-1 (rec eA))))
      (send (cat (gen) (mul w l-1 (rec eA))))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 593)
  (parent 547)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 eA rndx) (w expt)
    (l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (gen) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (gen) (mul w l-1 eB)))
      (send (cat (gen) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 594)
  (parent 548)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l rndx) (w expt) (l-0 e l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w e l-1 eB))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e e l-1))) (l l) (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul w e e)))
  (deflistener (cat (exp (gen) e) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul w e e)) (l-1 (mul w e e)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e e l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul w e e l-1)) (gen))
    ((exp (gen) (mul w e l-1 eB)) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w e l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul w e e l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul w e e)))
      (send (cat (exp (gen) l-1) (mul w e e))))
    ((recv (cat (exp (gen) e) (mul w l-1 eB)))
      (send (cat (exp (gen) e) (mul w l-1 eB)))) ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 595)
  (parent 551)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt) (l-0 l-1 e rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1 (rec e)))) (l l-0) (e e))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener e)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (e (mul (rec eB) w)) (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-1 self) (l B) (l-0 D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec e))) (gen)) (D B) (e eB) (l-0 l) (e l-0)
    (eB l) (e l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 e)
  (operation nonce-test (added-listener e) (mul w l-1 (rec e) (rec e))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1 (rec e)))) (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec e) (rec e))))
      (send (cat (exp (gen) e) (mul w l-1 (rec e) (rec e)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv e) (send e)))
  (label 596)
  (parent 552)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l l-0 e l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) e) (mul l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (l-1 (one)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul e l-1))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul e l-1 (rec eA))) (gen))
    (D B) (eA e) (l l-0) (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul e l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) e) (mul l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 597)
  (parent 553)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l rndx) (w expt) (l-0 e l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-1))) (l l-0) (e e))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) w))
  (deflistener (cat (exp (gen) e) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA w) (l-1 w))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-1))) kh)))
  (neq ((exp (gen) (mul w l-1)) (gen))
    ((exp (gen) (mul w e l-1 (rec eA))) (gen)) (D B) (eA e) (l l-0)
    (eA l) (e l-0) (eA l-0) (e l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 e l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D))) (send (exp (gen) e))
      (recv (exp (gen) (mul w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) w)) (send (cat (exp (gen) l-1) w)))
    ((recv (cat (exp (gen) e) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 598)
  (parent 555)
  (unrealized (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eB rndx) (w expt) (e l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 (rec eA))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul (rec eB) w e)) (l-2 (mul (rec eB) w e)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w e l-2)) (gen))
    ((exp (gen) (mul w e l-2 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w e)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) e) (mul w l-2 (rec eA)))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 599)
  (parent 559)
  (unrealized (1 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self peer self-0 name) (hl base)
    (l l-0 l-1 eA rndx) (w expt) (e l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w e l-2 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w e l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w e)))
  (deflistener (cat (exp (gen) e) (mul w l-2 eB)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l-1)
    (e e))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 2) (5 0)) ((7 1) (5 0))
    ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul eA w e)) (l-2 (mul eA w e)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w e l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w e l-2)) (gen))
    ((exp (gen) (mul w e l-2 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w e l-2 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w e l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w e)))
      (send (cat (exp (gen) l-2) (mul eA w e))))
    ((recv (cat (exp (gen) e) (mul w l-2 eB)))
      (send (cat (exp (gen) e) (mul w l-2 eB))))
    ((recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 600)
  (parent 560)
  (unrealized (1 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 601)
  (parent 563)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-0 l-1 eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-0 l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 602)
  (parent 564)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eB rndx) (w expt)
    (l-0 l-1 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1 (rec eA)))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul (rec eB) w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (8 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eA (mul (rec eB) w l-0)) (l-1 (mul (rec eB) w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0 l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 (rec eA))) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eB l-0 l-1 eA)
  (operation nonce-test (added-listener eA) (mul w l-1 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w l-0)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 (rec eA))))
      (send (cat (exp (gen) l-0) (mul w l-1 (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eA) (send eA)))
  (label 603)
  (parent 568)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self name) (l eA rndx) (w expt)
    (l-0 l-1 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B) (hl (exp (gen) l))
    (he (exp (gen) (mul w l-0 l-1 eB))) (l l-0) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul eA w l-0 l-1))) (l l)
    (e eB))
  (defstrand ltx-gen 1 (self B) (l l))
  (defstrand ltx-gen 1 (self D) (l l-0))
  (deflistener (cat (exp (gen) l-1) (mul eA w l-0)))
  (deflistener (cat (exp (gen) l-0) (mul w l-1 eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (8 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)))
  (absent (eB (mul eA w l-0)) (l-1 (mul eA w l-0)))
  (fn-of ("principal-of" (l-1 self) (l-0 D) (l B))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-0 l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-0 l-1)) (gen))
    ((exp (gen) (mul w l-0 l-1 eB)) (gen)) (D B) (eA eB) (l-0 l)
    (eA l-0) (eB l) (eA l) (eB l-0))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l eA l-0 l-1 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (recv (enc "cert" (exp (gen) l) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-0 l-1 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l) B (privk B)))
      (recv (enc "cert" (exp (gen) l-0) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-0 l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l) B (privk B))))
    ((send (enc "cert" (exp (gen) l-0) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w l-0)))
      (send (cat (exp (gen) l-1) (mul eA w l-0))))
    ((recv (cat (exp (gen) l-0) (mul w l-1 eB)))
      (send (cat (exp (gen) l-0) (mul w l-1 eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv eB) (send eB)))
  (label 604)
  (parent 569)
  (unrealized (1 3) (4 0) (8 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eA (mul (rec eB) w l-1)) (l-2 (mul (rec eB) w l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1 l-2))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1 l-2)) (gen))
    ((exp (gen) (mul w l-1 l-2 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w l-2 (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1 l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul (rec eB) w l-1)))
      (send (cat (exp (gen) l-2) (mul (rec eB) w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 (rec eA))))
      (send (cat (exp (gen) l-1) (mul w l-2 (rec eA)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 605)
  (parent 574)
  (unrealized (1 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 l-2 eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1 l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA w l-1)))
  (deflistener (cat (exp (gen) l-1) (mul w l-2 eB)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (5 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (eB (mul eA w l-1)) (l-2 (mul eA w l-1)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 l-2 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1 l-2)) (gen))
    ((exp (gen) (mul w l-1 l-2 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-2 eB) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 l-2 eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1 l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA w l-1)))
      (send (cat (exp (gen) l-2) (mul eA w l-1))))
    ((recv (cat (exp (gen) l-1) (mul w l-2 eB)))
      (send (cat (exp (gen) l-1) (mul w l-2 eB))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 606)
  (parent 575)
  (unrealized (1 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 0) (4 0)) ((6 0) (9 0))
    ((7 1) (5 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (5 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-1 self-0) (l-2 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w l-1 (rec l-2))
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 607)
  (parent 576)
  (unrealized (1 3) (4 0) (9 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB l-1 rndx) (w expt)
    (l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w (rec eA)))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (9 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eA (mul (rec eB) (rec l-1) w))
    (l-2 (mul (rec eB) (rec l-1) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) w)) kh)))
  (neq ((exp (gen) (mul (rec eB) w)) (gen))
    ((exp (gen) (mul w (rec eA))) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA) (mul w (rec l-2) (rec eA))
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) (rec eA)))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 608)
  (parent 580)
  (unrealized (1 3) (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA l-1 rndx) (w expt)
    (l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (9 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (6 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (eB (mul eA (rec l-1) w)) (l-2 (mul eA (rec l-1) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-0 B) (l D) (l-1 self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w eB))) kh)))
  (neq ((exp (gen) (mul eA w)) (gen)) ((exp (gen) (mul w eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w (rec l-2) eB) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec l-1) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec l-1) w))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l-2) (mul w (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w (rec l-2) eB))))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 609)
  (parent 581)
  (unrealized (4 0) (9 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-2))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-2))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-2) (mul eA (rec eB) w)))
  (defstrand ltx-gen 1 (self self) (l l-2))
  (deflistener (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self-0) (l l-1))
  (deflistener l-2)
  (precedes ((0 2) (4 0)) ((0 2) (6 0)) ((1 2) (4 0)) ((1 2) (6 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 0) (4 0)) ((5 0) (9 0)) ((6 1) (0 3))
    ((7 1) (6 0)) ((8 0) (4 0)) ((8 1) (7 0)) ((9 1) (6 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-1 self-0) (l-0 B) (l D) (l-2 self))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-2))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-2)) (gen))
    ((exp (gen) (mul w l-2)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (6 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (added-listener l-2) (mul w (rec l-1) l-2)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-2))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-2)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-2) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-2) (mul eA (rec eB) w))))
    ((send (enc "cert" (exp (gen) l-2) self (privk self))))
    ((recv (cat (exp (gen) l-1) (mul w (rec l-1) l-2)))
      (send (cat (exp (gen) l-1) (mul w (rec l-1) l-2))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self-0 (privk self-0)))
      (send l-1)) ((recv l-2) (send l-2)))
  (label 610)
  (parent 582)
  (seen 611)
  (unrealized (1 3) (4 0) (9 0))
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA eB rndx) (w expt)
    (l-1 l-2 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (mul eA (rec eB) w)) (l-1 (mul eA (rec eB) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1))) kh)))
  (neq ((exp (gen) (mul eA (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA eB l-1 l-2)
  (operation nonce-test (displaced 6 10 ltx-gen 2) l-2 (9 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul eA (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 611)
  (parent 607)
  (unrealized (1 3) (4 0))
  (comment "4 in cohort - 4 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 l-1 l-2 eA eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) l-1)) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (one)))
  (deflistener (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (l-2 (one)) (l-1 (one)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB))) kh)))
  (neq ((exp (gen) l-1) (gen)) ((exp (gen) (mul l-1 (rec eA) eB)) (gen))
    (D B) (eA eB) (l l-0) (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB)
  (operation nonce-test
    (contracted (eA-0 eA) (eB-0 eB) (w (mul (rec eA) eB))) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) l-1)) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 612)
  (parent 611)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 self-1 name)
    (l l-0 l-1 l-2 eA eB l-3 rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul l-1 (rec eA) eB l-3)))
    (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul l-1 l-3))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) l-3))
  (deflistener
    (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (defstrand ltx-gen 2 (self self-1) (l l-3))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 0) (5 0)) ((10 1) (4 0)))
  (absent (l-2 l-3) (l-1 l-3))
  (fn-of
    ("principal-of" (l-3 self-1) (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul l-1 eB l-3))) kh)))
  (neq ((exp (gen) (mul l-1 l-3)) (gen))
    ((exp (gen) (mul l-1 (rec eA) eB l-3)) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 l-1 l-2 eA eB l-3)
  (operation nonce-test (added-strand ltx-gen 2) l-3 (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul l-1 (rec eA) eB l-3)))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul l-1 l-3))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) l-3)) (send (cat (exp (gen) l-1) l-3)))
    ((recv (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3)))
      (send (cat (exp (gen) l-2) (mul l-1 (rec l-2) (rec eA) eB l-3))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2))
    ((send (enc "cert" (exp (gen) l-3) self-1 (privk self-1)))
      (send l-3)))
  (label 613)
  (parent 611)
  (unrealized (0 3) (5 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eA (mul (rec eB) w)) (l-2 (mul (rec eB) w))
    (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-absence eA (mul (rec eB) w))
    (mul (rec eB) w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 614)
  (parent 611)
  (unrealized (0 3) (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0)))
  (absent (eB (mul eA w)) (l-2 (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-absence eB (mul eA w)) (mul eA w) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)))
  (label 615)
  (parent 611)
  (unrealized (1 3) (4 0) (5 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eB rndx) (w expt)
    (l-1 l-2 eA rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 (rec eA)))) (l l)
    (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul (rec eB) w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul (rec eB) w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eA)
  (precedes ((0 2) (4 0)) ((0 2) (10 0)) ((1 2) (4 0)) ((1 2) (5 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eA (mul (rec eB) w)) (l-2 (mul (rec eB) w))
    (l-1 (mul (rec eB) w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-1))) kh)))
  (neq ((exp (gen) (mul (rec eB) w l-1)) (gen))
    ((exp (gen) (mul w l-1 (rec eA))) (gen)) (D B) (eA eB) (l l-0)
    (eA l) (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eB l-1 l-2 eA)
  (operation nonce-test (added-listener eA)
    (mul w l-1 (rec l-2) (rec eA)) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 (rec eA))))
      (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul (rec eB) w l-1)))
      (send kh)) ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul (rec eB) w)))
      (send (cat (exp (gen) l-1) (mul (rec eB) w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA))))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) (rec eA)))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eA) (send eA)))
  (label 616)
  (parent 614)
  (unrealized (1 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (B D self self-0 name) (l l-0 eA rndx) (w expt)
    (l-1 l-2 eB rndx))
  (defstrand participant 5 (kh kh) (self D) (peer B)
    (hl (exp (gen) l-0)) (he (exp (gen) (mul w l-1 eB))) (l l) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl (exp (gen) l))
    (he (exp (gen) (mul eA w l-1))) (l l-0) (e eB))
  (defstrand ltx-gen 1 (self B) (l l-0))
  (defstrand ltx-gen 1 (self D) (l l))
  (deflistener (cat (exp (gen) l-1) (mul eA w)))
  (deflistener (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
  (deflistener l-1)
  (defstrand ltx-gen 2 (self self) (l l-1))
  (deflistener l-2)
  (defstrand ltx-gen 2 (self self-0) (l l-2))
  (deflistener eB)
  (precedes ((0 2) (4 0)) ((0 2) (5 0)) ((1 2) (4 0)) ((1 2) (10 0))
    ((2 0) (0 1)) ((2 0) (1 0)) ((3 0) (0 0)) ((3 0) (1 1))
    ((4 1) (1 3)) ((5 1) (0 3)) ((6 1) (5 0)) ((7 0) (4 0))
    ((7 1) (6 0)) ((8 1) (5 0)) ((9 0) (4 0)) ((9 1) (8 0))
    ((10 1) (5 0)))
  (absent (eB (mul eA w)) (l-2 (mul eA w)) (l-1 (mul eA w)))
  (fn-of ("principal-of" (l-2 self-0) (l-1 self) (l-0 B) (l D))
    (foo
      ((hash (exp (gen) (mul l l-0)) (exp (gen) (mul eA w l-1 eB)))
        kh)))
  (neq ((exp (gen) (mul eA w l-1)) (gen))
    ((exp (gen) (mul w l-1 eB)) (gen)) (D B) (eA eB) (l l-0) (eA l)
    (eB l-0) (eA l-0) (eB l))
  (non-orig (privk B) (privk D) l l-0)
  (precur (5 0) (4 0))
  (uniq-gen l l-0 eA l-1 l-2 eB)
  (operation nonce-test (added-listener eB) (mul w l-1 (rec l-2) eB)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) D (privk D)))
      (recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (send (exp (gen) eA)) (recv (exp (gen) (mul w l-1 eB))) (send kh))
    ((recv (enc "cert" (exp (gen) l-0) B (privk B)))
      (recv (enc "cert" (exp (gen) l) D (privk D)))
      (send (exp (gen) eB)) (recv (exp (gen) (mul eA w l-1))) (send kh))
    ((send (enc "cert" (exp (gen) l-0) B (privk B))))
    ((send (enc "cert" (exp (gen) l) D (privk D))))
    ((recv (cat (exp (gen) l-1) (mul eA w)))
      (send (cat (exp (gen) l-1) (mul eA w))))
    ((recv (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB)))
      (send (cat (exp (gen) l-2) (mul w l-1 (rec l-2) eB))))
    ((recv l-1) (send l-1))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))) (send l-1))
    ((recv l-2) (send l-2))
    ((send (enc "cert" (exp (gen) l-2) self-0 (privk self-0)))
      (send l-2)) ((recv eB) (send eB)))
  (label 617)
  (parent 615)
  (unrealized (1 3) (4 0) (10 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (hl he base) (self peer name) (kh data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))
      (recv he) (send kh))
    (fn-of (foo ((hash (exp hl l) (exp he e)) kh)))
    (neq (he (gen))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (hl he hl-0 he-0 base)
    (eA lA eB lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D) (hl hl-0) (he he-0)
    (l lB) (e eB))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
  (fn-of
    (foo ((hash (exp hl-0 lB) (exp he-0 eB)) kh)
      ((hash (exp hl lA) (exp he eA)) kh)))
  (neq (he-0 (gen)) (he (gen)) (A B) (eA eB) (lA lB) (eA lA) (eB lB)
    (eA lB) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication + disclosure")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send kh))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" hl-0 D (privk D))) (send (exp (gen) eB))
      (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
      (send (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))))
  (label 618)
  (unrealized (0 0) (0 1) (1 0) (1 1) (2 0))
  (preskeleton)
  (origs)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (hl he base) (lA lB eA eB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp hl (mul lA (rec lB)))) (he (exp he (mul eA (rec eB))))
    (l lB) (e eB))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
  (precedes ((0 2) (1 3)) ((0 2) (2 0)) ((1 2) (2 0)))
  (fn-of (foo ((hash (exp hl lA) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA lB) (eA lA) (eB lB) (eA lB) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (comment "Implicit authentication + disclosure")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send kh))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp hl (mul lA (rec lB))) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB)))) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
      (send (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))))
  (label 619)
  (parent 618)
  (unrealized (0 0) (0 1) (1 0) (1 1) (1 3) (2 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A B C D name) (hl he base) (lA lB eA eB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl hl) (he he)
    (l lA) (e eA))
  (defstrand participant 5 (kh kh) (self B) (peer D)
    (hl (exp hl (mul lA (rec lB)))) (he (exp he (mul eA (rec eB))))
    (l lB) (e eB))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
  (deflistener (cat (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
  (precedes ((0 2) (1 3)) ((0 2) (3 0)) ((1 2) (3 0)) ((3 1) (2 0)))
  (fn-of (foo ((hash (exp hl lA) (exp he eA)) kh)))
  (neq ((exp he (mul eA (rec eB))) (gen)) (he (gen)) (A B) (eA eB)
    (lA lB) (eA lA) (eB lB) (eA lB) (eB lA))
  (non-orig (privk A) (privk B) (privk C) (privk D) lA lB)
  (uniq-gen eA eB)
  (operation encryption-test
    (added-listener
      (cat (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
    (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" hl C (privk C))) (send (exp (gen) eA)) (recv he)
      (send kh))
    ((recv (enc "cert" (exp (gen) lB) B (privk B)))
      (recv (enc "cert" (exp hl (mul lA (rec lB))) D (privk D)))
      (send (exp (gen) eB)) (recv (exp he (mul eA (rec eB)))) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
      (send (hash (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB)))))
    ((recv (cat (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))
      (send (cat (exp (gen) (mul lA lB)) (exp (gen) (mul eA eB))))))
  (label 620)
  (parent 619)
  (unrealized (0 0) (0 1) (1 0) (1 1) (1 3) (3 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (hl he base) (self peer name) (kh data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))
      (recv he) (send kh))
    (fn-of (foo ((hash (exp hl l) (exp he e)) kh)))
    (neq (he (gen))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (eA lA lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lA lB)
  (comment "Implicit authentication + disclosure")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA)))))
  (label 621)
  (unrealized (0 0) (0 1) (1 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (eA lA lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA lB)) (exp he eA)))
  (precedes ((2 1) (1 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lA lB)
  (operation encryption-test
    (added-listener (cat (exp (gen) (mul lA lB)) (exp he eA)))
    (hash (exp (gen) (mul lA lB)) (exp he eA)) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA lB)) (exp he eA)))
      (send (cat (exp (gen) (mul lA lB)) (exp he eA)))))
  (label 622)
  (parent 621)
  (unrealized (0 0) (0 1) (2 0))
  (dead)
  (comment "empty cohort"))

(comment "Nothing left to do")

(defprotocol iadh-um diffie-hellman
  (defrole participant
    (vars (l e rndx) (hl he base) (self peer name) (kh data))
    (trace (recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))
      (recv he) (send kh))
    (fn-of (foo ((hash (exp hl l) (exp he e)) kh)))
    (neq (he (gen))))
  (defrole ltx-gen
    (vars (self name) (l rndx))
    (trace (send (enc "cert" (exp (gen) l) self (privk self))) (send l))
    (uniq-gen l)
    (fn-of ("principal-of" (l self)))))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (eA lA lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lB)
  (uniq-gen eA)
  (comment "Implicit authentication + disclosure")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA)))))
  (label 623)
  (unrealized (0 0) (0 1) (1 0))
  (preskeleton)
  (origs)
  (comment "Not a skeleton"))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (eA lA lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (precedes ((0 2) (1 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lB)
  (uniq-gen eA)
  (comment "Implicit authentication + disclosure")
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA)))))
  (label 624)
  (parent 623)
  (unrealized (0 0) (0 1) (1 0))
  (origs)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (eA lA lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA lB)) (exp he eA)))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lB)
  (uniq-gen eA)
  (operation encryption-test
    (added-listener (cat (exp (gen) (mul lA lB)) (exp he eA)))
    (hash (exp (gen) (mul lA lB)) (exp he eA)) (1 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA lB)) (exp he eA)))
      (send (cat (exp (gen) (mul lA lB)) (exp he eA)))))
  (label 625)
  (parent 624)
  (unrealized (0 0) (0 1) (2 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (eA lA lB rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) lB) lA))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lB)
  (uniq-gen eA)
  (operation nonce-test (added-listener (cat (exp (gen) lB) lA))
    (exp (gen) (mul lA lB)) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA lB)) (exp he eA)))
      (send (cat (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) lB) lA)) (send (cat (exp (gen) lB) lA))))
  (label 626)
  (parent 625)
  (unrealized (0 0) (0 1) (2 0) (3 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he he-0 hl base)
    (eA lA l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp he eA)))
      (send (hash (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA e)) (exp he eA)))
      (send (cat (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 627)
  (parent 626)
  (unrealized (0 0) (0 1) (2 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he he-0 base) (eA lA l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l) (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp he eA)))
      (send (hash (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA l)) (exp he eA)))
      (send (cat (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 628)
  (parent 626)
  (unrealized (0 0) (0 1) (2 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (eA lA lB rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) lB) lA))
  (deflistener (cat (exp (gen) (mul lB (rec w))) w))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lB)
  (uniq-gen eA)
  (precur (4 0))
  (operation nonce-test
    (added-listener (cat (exp (gen) (mul lB (rec w))) w)) (exp (gen) lB)
    (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA lB)) (exp he eA)))
      (send (cat (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) lB) lA)) (send (cat (exp (gen) lB) lA)))
    ((recv (cat (exp (gen) (mul lB (rec w))) w))
      (send (cat (exp (gen) (mul lB (rec w))) w))))
  (label 629)
  (parent 626)
  (unrealized (0 0) (0 1) (2 0) (4 0))
  (comment "3 in cohort - 3 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (gen)))
  (deflistener (cat (exp (gen) (mul lA e)) (gen)))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA)
  (operation nonce-test (contracted (he-0 (exp (gen) (rec eA)))) (gen)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (gen)))
      (send (hash (exp (gen) (mul lA e)) (gen))))
    ((recv (cat (exp (gen) (mul lA e)) (gen)))
      (send (cat (exp (gen) (mul lA e)) (gen))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 630)
  (parent 627)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0)))
  (fn-of (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen e-0)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-1)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 631)
  (parent 627)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) e)))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA)
  (operation nonce-test (displaced 5 4 participant 3) (exp (gen) e-0)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) e)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) e)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) e))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 632)
  (parent 627)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA lA l e l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 2) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 633)
  (parent 627)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base)
    (eA lA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (2 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 634)
  (parent 627)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he he-0 hl base)
    (eA lA l e rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp he (mul eA (rec w))) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec w))) w)) (exp he eA)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp he eA)))
      (send (hash (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA e)) (exp he eA)))
      (send (cat (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp he (mul eA (rec w))) w))
      (send (cat (exp he (mul eA (rec w))) w))))
  (label 635)
  (parent 627)
  (unrealized (0 0) (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (gen)))
  (deflistener (cat (exp (gen) (mul lA l)) (gen)))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l)
  (operation nonce-test (contracted (he-0 (exp (gen) (rec eA)))) (gen)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (gen)))
      (send (hash (exp (gen) (mul lA l)) (gen))))
    ((recv (cat (exp (gen) (mul lA l)) (gen)))
      (send (cat (exp (gen) (mul lA l)) (gen))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 636)
  (parent 628)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen l e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 637)
  (parent 628)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base)
    (eA lA l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 2) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) e)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) e)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 638)
  (parent 628)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l)
  (operation nonce-test (displaced 5 4 ltx-gen 1) (exp (gen) l-0) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) l)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) l)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) l))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 639)
  (parent 628)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA lA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (2 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 640)
  (parent 628)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he he-0 base) (eA lA l rndx)
    (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp he (mul eA (rec w))) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l)
  (precur (5 0))
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec w))) w)) (exp he eA)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp he eA)))
      (send (hash (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA l)) (exp he eA)))
      (send (cat (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp he (mul eA (rec w))) w))
      (send (cat (exp he (mul eA (rec w))) w))))
  (label 641)
  (parent 628)
  (unrealized (0 0) (0 1) (5 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C name) (he he-0 base) (lB l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l l) (e e))
  (deflistener (hash (exp (gen) (mul lB l)) (exp he e)))
  (deflistener (cat (exp (gen) (mul lB l)) (exp he e)))
  (deflistener (cat (exp (gen) lB) l))
  (deflistener (cat (exp (gen) e) (mul lB (rec e))))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0)))
  (fn-of (foo ((hash (exp (gen) (mul lB l)) (exp he-0 e)) kh)))
  (neq (he-0 (gen)) (A C) (l lB) (e l) (e lB))
  (non-orig (privk A) (privk C) lB)
  (precur (4 0))
  (uniq-gen e)
  (operation nonce-test (displaced 5 0 participant 3) (exp (gen) e-0)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) e)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lB l)) (exp he e)))
      (send (hash (exp (gen) (mul lB l)) (exp he e))))
    ((recv (cat (exp (gen) (mul lB l)) (exp he e)))
      (send (cat (exp (gen) (mul lB l)) (exp he e))))
    ((recv (cat (exp (gen) lB) l)) (send (cat (exp (gen) lB) l)))
    ((recv (cat (exp (gen) e) (mul lB (rec e))))
      (send (cat (exp (gen) e) (mul lB (rec e))))))
  (label 642)
  (parent 629)
  (unrealized (0 0) (0 1) (2 0) (3 0) (4 0))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he he-0 hl base)
    (eA lA l lB e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) lB) lA))
  (deflistener (cat (exp (gen) e) (mul lB (rec e))))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lB)
  (precur (4 0))
  (uniq-gen eA)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA lB)) (exp he eA)))
      (send (cat (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) lB) lA)) (send (cat (exp (gen) lB) lA)))
    ((recv (cat (exp (gen) e) (mul lB (rec e))))
      (send (cat (exp (gen) e) (mul lB (rec e)))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 643)
  (parent 629)
  (unrealized (0 0) (0 1) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he he-0 base) (eA lA lB l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) lB))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA lB)) (exp he eA)))
  (deflistener (cat (exp (gen) lB) lA))
  (deflistener (cat (exp (gen) l) (mul lB (rec l))))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (4 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA lB)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA lB) (eA lA) (eA lB))
  (non-orig (privk A) (privk C) lB)
  (precur (4 0))
  (uniq-gen eA l)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) lB) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA lB)) (exp he eA)))
      (send (hash (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA lB)) (exp he eA)))
      (send (cat (exp (gen) (mul lA lB)) (exp he eA))))
    ((recv (cat (exp (gen) lB) lA)) (send (cat (exp (gen) lB) lA)))
    ((recv (cat (exp (gen) l) (mul lB (rec l))))
      (send (cat (exp (gen) l) (mul lB (rec l)))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 644)
  (parent 629)
  (unrealized (0 0) (0 1) (2 0) (3 0) (4 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (gen)))
      (send (hash (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) (mul e l-0)) (gen)))
      (send (cat (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 645)
  (parent 630)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen e-0 l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 646)
  (parent 631)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 647)
  (parent 632)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA l e l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 2) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-1 A))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 648)
  (parent 633)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (uniq-gen eA l-0)
  (operation encryption-test (displaced 6 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 649)
  (parent 634)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 650)
  (parent 634)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA l e rndx)
    (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) w)))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (uniq-gen eA)
  (precur (5 0))
  (operation nonce-test (contracted (he-0 (exp (gen) (mul (rec eA) w))))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) w)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) w)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) w))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 651)
  (parent 635)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 652)
  (parent 635)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA rndx)
    (w expt) (l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen eA)
  (operation nonce-test (displaced 6 4 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 653)
  (parent 635)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA lA l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 2) (5 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen eA)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 654)
  (parent 635)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base)
    (eA lA l e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) lA))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen eA l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 655)
  (parent 635)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 656)
  (parent 636)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 657)
  (parent 637)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base)
    (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 2) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 658)
  (parent 638)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 659)
  (parent 639)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 6 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 660)
  (parent 640)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 661)
  (parent 640)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA l rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (uniq-gen eA l)
  (precur (5 0))
  (operation nonce-test (contracted (he-0 (exp (gen) (mul (rec eA) w))))
    (gen) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) w)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) w)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) w))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 662)
  (parent 641)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l rndx) (w expt)
    (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen l e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 663)
  (parent 641)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base)
    (eA lA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 2) (5 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen eA l)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 664)
  (parent 641)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA rndx) (w expt)
    (l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen eA l)
  (operation nonce-test (displaced 6 4 ltx-gen 1) (exp (gen) l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 665)
  (parent 641)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA lA l rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) lA))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (5 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (5 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 666)
  (parent 641)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he he-0 hl base)
    (eA lA l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA)
  (operation nonce-test (contracted (lB e) (e-0 e)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp he eA)))
      (send (hash (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA e)) (exp he eA)))
      (send (cat (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 667)
  (parent 643)
  (unrealized (0 0) (0 1) (2 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he he-0 base) (eA lA l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l)
  (operation nonce-test (contracted (lB l) (l-0 l)) (one) (4 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp he eA)))
      (send (hash (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA l)) (exp he eA)))
      (send (cat (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 668)
  (parent 644)
  (unrealized (0 0) (0 1) (2 0))
  (comment "6 in cohort - 6 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (gen)))
      (send (hash (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) (mul e l-0)) (gen)))
      (send (cat (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 669)
  (parent 645)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (l e e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk C) (privk self-0) e)
  (uniq-gen e-0 l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 670)
  (parent 646)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 671)
  (parent 647)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 peer-0 self-1 name)
    (he hl hl-0 base) (eA l e l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 2) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (uniq-gen eA l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 672)
  (parent 648)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 673)
  (parent 649)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 0) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (uniq-gen eA l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 674)
  (parent 650)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 675)
  (parent 651)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e rndx) (w expt)
    (e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen e-0 l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 676)
  (parent 652)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 677)
  (parent 653)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA l e rndx) (w expt) (l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 2) (5 0)) ((7 0) (0 0)) ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen eA l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 678)
  (parent 654)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (5 0))
  (uniq-gen eA l-0)
  (operation encryption-test (displaced 7 6 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 679)
  (parent 655)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (5 0)) ((7 0) (0 0)) ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (5 0))
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 680)
  (parent 655)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 681)
  (parent 656)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (e l-0) (e l))
  (non-orig (privk C) (privk self-0) l)
  (uniq-gen l e l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 682)
  (parent 657)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 peer self-1 name) (he hl base)
    (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 2) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (uniq-gen eA l l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 683)
  (parent 658)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 684)
  (parent 659)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 5 6 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 685)
  (parent 660)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (uniq-gen eA l l-0 l-1)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 686)
  (parent 661)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 687)
  (parent 662)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l rndx) (w expt)
    (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 688)
  (parent 663)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base) (eA l rndx)
    (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 2) (5 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen eA l l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 689)
  (parent 664)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 690)
  (parent 665)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 6 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 691)
  (parent 666)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (5 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (5 0))
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 692)
  (parent 666)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (gen)))
  (deflistener (cat (exp (gen) (mul lA e)) (gen)))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA)
  (operation nonce-test (contracted (he-0 (exp (gen) (rec eA)))) (gen)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (gen)))
      (send (hash (exp (gen) (mul lA e)) (gen))))
    ((recv (cat (exp (gen) (mul lA e)) (gen)))
      (send (cat (exp (gen) (mul lA e)) (gen))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 693)
  (parent 667)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)))
  (fn-of (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen e-0)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-1)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 694)
  (parent 667)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) e)))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA)
  (operation nonce-test (displaced 6 5 participant 3) (exp (gen) e-0)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) e)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) e)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) e))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 695)
  (parent 667)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA lA l e l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 2) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 696)
  (parent 667)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base)
    (eA lA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (2 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 697)
  (parent 667)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he he-0 hl base)
    (eA lA l e rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp he eA)))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp he (mul eA (rec w))) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA)
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec w))) w)) (exp he eA)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp he eA)))
      (send (hash (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA e)) (exp he eA)))
      (send (cat (exp (gen) (mul lA e)) (exp he eA))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp he (mul eA (rec w))) w))
      (send (cat (exp he (mul eA (rec w))) w))))
  (label 698)
  (parent 667)
  (unrealized (0 0) (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (gen)))
  (deflistener (cat (exp (gen) (mul lA l)) (gen)))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l)
  (operation nonce-test (contracted (he-0 (exp (gen) (rec eA)))) (gen)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (gen)))
      (send (hash (exp (gen) (mul lA l)) (gen))))
    ((recv (cat (exp (gen) (mul lA l)) (gen)))
      (send (cat (exp (gen) (mul lA l)) (gen))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 699)
  (parent 668)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen l e)
  (operation nonce-test (displaced 6 0 participant 3) (exp (gen) e-0)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 700)
  (parent 668)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base)
    (eA lA l l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 2) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) e)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) e)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 701)
  (parent 668)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l)
  (operation nonce-test (displaced 6 5 ltx-gen 1) (exp (gen) l-0) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) l)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) l)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) l))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self)))))
  (label 702)
  (parent 668)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA lA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (2 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 703)
  (parent 668)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he he-0 base) (eA lA l rndx)
    (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he-0) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp he eA)))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp he (mul eA (rec w))) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he-0 eA)) kh)))
  (neq (he-0 (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l)
  (operation nonce-test
    (added-listener (cat (exp he (mul eA (rec w))) w)) (exp he eA)
    (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he-0) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp he eA)))
      (send (hash (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) (mul lA l)) (exp he eA)))
      (send (cat (exp (gen) (mul lA l)) (exp he eA))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp he (mul eA (rec w))) w))
      (send (cat (exp he (mul eA (rec w))) w))))
  (label 704)
  (parent 668)
  (unrealized (0 0) (0 1) (6 0))
  (comment "5 in cohort - 5 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (5 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 705)
  (parent 675)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (l e rndx)
    (w expt) (e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk C) (privk self-0) e)
  (precur (5 0))
  (uniq-gen e-0 l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 706)
  (parent 676)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((4 2) (5 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (5 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 707)
  (parent 677)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 peer-0 self-1 name)
    (he hl hl-0 base) (eA l e rndx) (w expt) (l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 2) (5 0)) ((7 0) (0 0)) ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (precur (5 0))
  (uniq-gen eA l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 708)
  (parent 678)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (5 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 709)
  (parent 679)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 self-1 name) (he hl base)
    (eA l e rndx) (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 2) (3 0))
    ((5 1) (2 0)) ((6 0) (5 0)) ((7 0) (0 0)) ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (precur (5 0))
  (uniq-gen eA l-0 l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 710)
  (parent 680)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 6 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 711)
  (parent 681)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen e l l-0)
  (operation encryption-test (displaced 6 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 712)
  (parent 682)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 2) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (displaced 7 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 713)
  (parent 683)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 6 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 714)
  (parent 684)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 6 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 715)
  (parent 685)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (displaced 7 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 716)
  (parent 686)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 717)
  (parent 687)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (l rndx) (w expt)
    (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (e l-0) (e l))
  (non-orig (privk C) (privk self-0) l)
  (precur (5 0))
  (uniq-gen l e l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 718)
  (parent 688)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 peer self-1 name) (he hl base)
    (eA l rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 2) (5 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (precur (5 0))
  (uniq-gen eA l l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 719)
  (parent 689)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 720)
  (parent 690)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 721)
  (parent 691)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 self-1 name) (he base) (eA l rndx)
    (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (5 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (precur (5 0))
  (uniq-gen eA l l-0 l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 722)
  (parent 692)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (gen)))
      (send (hash (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) (mul e l-0)) (gen)))
      (send (cat (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 723)
  (parent 693)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen e-0 l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 724)
  (parent 694)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 725)
  (parent 695)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA l e l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 2) (2 0)) ((7 0) (0 0)) ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 726)
  (parent 696)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (4 0))
  (uniq-gen eA l-0)
  (operation encryption-test (displaced 7 6 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 727)
  (parent 697)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (2 0)) ((7 0) (0 0)) ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (4 0))
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 728)
  (parent 697)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA l e rndx)
    (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) w)))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA)
  (operation nonce-test (contracted (he-0 (exp (gen) (mul (rec eA) w))))
    (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) w)))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) w)))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) w))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 729)
  (parent 698)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e rndx) (w expt)
    (l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen e-0)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-1)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w))))
  (label 730)
  (parent 698)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA lA rndx)
    (w expt) (l e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (2 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA)
  (operation nonce-test (displaced 7 5 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 731)
  (parent 698)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA lA l e rndx) (w expt) (l-0 e-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 2) (6 0)))
  (fn-of (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA)
  (operation nonce-test (added-strand participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0))))
  (label 732)
  (parent 698)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base)
    (eA lA l e rndx) (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) lA))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul lA e)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA e) (eA lA) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul lA e)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul lA e)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul lA e)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) lA)) (send (cat (exp (gen) e) lA)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 733)
  (parent 698)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 734)
  (parent 699)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 735)
  (parent 700)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base)
    (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 2) (2 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 736)
  (parent 701)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 737)
  (parent 702)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 6 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 738)
  (parent 703)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (2 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 739)
  (parent 703)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA l rndx) (w expt))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (gen) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l)
  (operation nonce-test (contracted (he-0 (exp (gen) (mul (rec eA) w))))
    (gen) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) w)))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) w)))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) w))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w))))
  (label 740)
  (parent 704)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l rndx) (w expt)
    (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen l e)
  (operation nonce-test (displaced 7 0 participant 3) (exp (gen) e-0)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w))))
  (label 741)
  (parent 704)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base)
    (eA lA l rndx) (w expt) (l-0 e rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 2) (6 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l)
  (operation nonce-test (added-strand participant 3) (exp (gen) e)
    (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e))))
  (label 742)
  (parent 704)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA lA rndx) (w expt)
    (l rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l) w))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l)
  (operation nonce-test (displaced 7 5 ltx-gen 1) (exp (gen) l-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w))))
  (label 743)
  (parent 704)
  (unrealized (0 0) (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA lA l rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l lA) (e eA))
  (deflistener (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) lA))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (6 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul lA l)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (lA l) (eA lA) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (added-strand ltx-gen 1) (exp (gen) l-0) (6 0))
  (traces
    ((recv (enc "cert" (exp (gen) lA) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul lA l)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul lA l)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) lA)) (send (cat (exp (gen) l) lA)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 744)
  (parent 704)
  (unrealized (0 0) (0 1))
  (comment "2 in cohort - 2 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 745)
  (parent 711)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 746)
  (parent 712)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 2) (1 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 747)
  (parent 713)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 748)
  (parent 714)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 749)
  (parent 715)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (1 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 750)
  (parent 716)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 751)
  (parent 717)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen e l l-0)
  (operation encryption-test (displaced 7 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 752)
  (parent 718)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 2) (5 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (5 0))
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (displaced 8 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 753)
  (parent 719)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 754)
  (parent 720)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 755)
  (parent 721)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base) (eA rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (2 0)) ((6 0) (5 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (5 0))
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (displaced 8 4 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 756)
  (parent 722)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul e l-0)) (gen)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (gen)))
      (send (hash (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) (mul e l-0)) (gen)))
      (send (cat (exp (gen) (mul e l-0)) (gen))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 757)
  (parent 723)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (l e e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk C) (privk self-0) e)
  (precur (4 0))
  (uniq-gen e-0 l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 758)
  (parent 724)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 759)
  (parent 725)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 peer-0 self-1 name)
    (he hl hl-0 base) (eA l e l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 2) (2 0)) ((7 0) (0 0)) ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (precur (4 0))
  (uniq-gen eA l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) e-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) e-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 760)
  (parent 726)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base)
    (eA l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 761)
  (parent 727)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 0) (2 0)) ((7 0) (0 0)) ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (precur (4 0))
  (uniq-gen eA l-0 l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 762)
  (parent 728)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (0 0)) ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 763)
  (parent 729)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (l e rndx) (w expt)
    (e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (0 0)) ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen e-0 l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 764)
  (parent 730)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer name) (he hl base) (eA rndx) (w expt)
    (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 A))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 765)
  (parent 731)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 peer-0 name) (he hl hl-0 base)
    (eA l e rndx) (w expt) (l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 2) (6 0)) ((8 0) (0 0))
    ((8 0) (3 0)))
  (fn-of ("principal-of" (l-1 A))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 766)
  (parent 732)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (0 0)) ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0)
  (operation encryption-test (displaced 8 7 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 767)
  (parent 733)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) e))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (6 0)) ((8 0) (0 0))
    ((8 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk A) (privk C) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 768)
  (parent 733)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 769)
  (parent 734)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (e l-0) (e l))
  (non-orig (privk C) (privk self-0) l)
  (precur (4 0))
  (uniq-gen l e l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 770)
  (parent 735)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 peer self-1 name) (he hl base)
    (eA l l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 2) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (precur (4 0))
  (uniq-gen eA l l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 771)
  (parent 736)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 772)
  (parent 737)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 6 7 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 773)
  (parent 738)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 774)
  (parent 739)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 775)
  (parent 740)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (l rndx) (w expt)
    (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (e l-0) (e l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 776)
  (parent 741)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 peer name) (he hl base) (eA l rndx)
    (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 2) (6 0))
    ((8 0) (0 0)) ((8 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 777)
  (parent 742)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self A) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 A) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-0) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l-0) A (privk A)))))
  (label 778)
  (parent 743)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 0) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 8 7 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))))
  (label 779)
  (parent 744)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (A C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self A) (peer C) (hl (exp (gen) l))
    (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 1 (self A) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (6 0))
    ((8 0) (0 0)) ((8 0) (3 0)))
  (fn-of ("principal-of" (l-1 A) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (A C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk A) (privk C) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (added-strand ltx-gen 1)
    (enc "cert" (exp (gen) l-1) A (privk A)) (0 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) A (privk A)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) A (privk A)))))
  (label 780)
  (parent 744)
  (unrealized (0 1) (3 0))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (0 0)) ((3 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 781)
  (parent 745)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (eA eA) (lA l) (lB l-0) (he (exp (gen) (rec eA)))
        (A self-0) (C self) (he-0 he))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (0 0)) ((3 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 782)
  (parent 746)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (eA e) (lA l) (lB l-0) (he (gen)) (A self-0) (C self)
        (he-0 he))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 2) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 783)
  (parent 747)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (0 0)) ((3 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 784)
  (parent 748)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (eA eA) (lA l) (lB l-0)
        (he (exp (gen) (mul (rec eA) l-0))) (A self-0) (C self)
        (he-0 he))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (0 0)) ((3 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 785)
  (parent 749)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (eA eA) (lA l) (lB l-0) (he (exp (gen) (mul (rec eA) l)))
        (A self-0) (C self) (he-0 he))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 786)
  (parent 750)
  (seen 811)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 787)
  (parent 751)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 788)
  (parent 752)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 2) (4 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (4 0))
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 789)
  (parent 753)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 790)
  (parent 754)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-0) l))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 791)
  (parent 755)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base) (eA rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 792)
  (parent 756)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (0 0)) ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 793)
  (parent 763)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (l e rndx)
    (w expt) (e-0 l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e e-0))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (0 0)) ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he e-0)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (e-0 l-0) (e-0 e))
  (non-orig (privk C) (privk self-0) e)
  (precur (6 0) (4 0))
  (uniq-gen e-0 l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) e-0)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 794)
  (parent 764)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((5 2) (6 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 795)
  (parent 765)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 peer-0 self-1 name)
    (he hl hl-0 base) (eA l e rndx) (w expt) (l-0 e-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) e-0) w))
  (defstrand participant 3 (self self-0) (peer peer-0) (hl hl-0) (l l-0)
    (e e-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 2) (6 0)) ((8 0) (0 0))
    ((8 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-1)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w e-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) e-0) w)) (send (cat (exp (gen) e-0) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl-0 peer-0 (privk peer-0)))
      (send (exp (gen) e-0)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 796)
  (parent 766)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 name) (he hl base) (eA l e rndx)
    (w expt) (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) e)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-0))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (0 0)) ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 e) (eA l-0) (eA e))
  (non-orig (privk C) (privk self-0) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-0)) (send (cat (exp (gen) e) l-0)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 797)
  (parent 767)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (C self peer self-0 self-1 name) (he hl base)
    (eA l e rndx) (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) e)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) e) l-1))
  (deflistener (cat (exp (gen) e) (one)))
  (defstrand participant 3 (self self) (peer peer) (hl hl) (l l) (e e))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 2) (4 0)) ((6 1) (2 0)) ((7 0) (6 0)) ((8 0) (0 0))
    ((8 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0))
    (foo ((hash (exp (gen) (mul e l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 e) (eA l-1) (eA e))
  (non-orig (privk C) (privk self-1) e)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) e) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul e l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) e) l-1)) (send (cat (exp (gen) e) l-1)))
    ((recv (cat (exp (gen) e) (one))) (send (cat (exp (gen) e) (one))))
    ((recv (enc "cert" (exp (gen) l) self (privk self)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 798)
  (parent 768)
  (unrealized (0 1))
  (dead)
  (comment "empty cohort"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) (mul l l-0)) (gen)))
      (send (cat (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 799)
  (parent 769)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen e l l-0)
  (operation encryption-test (displaced 7 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 800)
  (parent 770)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 2) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (4 0))
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (displaced 8 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 801)
  (parent 771)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 802)
  (parent 772)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 7 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 803)
  (parent 773)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (2 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 0) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (4 0))
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (displaced 8 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 804)
  (parent 774)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 805)
  (parent 775)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (l rndx) (w expt)
    (e l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (e l-0) (e l))
  (non-orig (privk C) (privk self-0) l)
  (precur (6 0) (4 0))
  (uniq-gen l e l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C))) (send (exp (gen) e))
      (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 806)
  (parent 776)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 peer self-1 name) (he hl base)
    (eA l rndx) (w expt) (l-0 e l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l-0)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 2) (6 0))
    ((8 0) (0 0)) ((8 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-1)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 807)
  (parent 777)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 808)
  (parent 778)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 name) (he base) (eA l rndx) (w expt)
    (l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer C)
    (hl (exp (gen) l)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-0))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 C) (l-0 l) (eA l-0) (eA l))
  (non-orig (privk C) (privk self-0) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation nonce-test (displaced 7 8 ltx-gen 2) l-0 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-0)) (send (cat (exp (gen) l) l-0)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0)))
      (send l-0)))
  (label 809)
  (parent 779)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (C self self-0 self-1 name) (he base) (eA l rndx)
    (w expt) (l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer C)
    (hl (exp (gen) l)) (he he) (l l-1) (e eA))
  (deflistener (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l) l-1))
  (deflistener (cat (exp (gen) l) (one)))
  (defstrand ltx-gen 1 (self self) (l l))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 1 (self self-0) (l l-0))
  (defstrand ltx-gen 2 (self self-1) (l l-1))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (6 0))
    ((8 0) (0 0)) ((8 1) (3 0)))
  (fn-of ("principal-of" (l-1 self-1) (l-0 self-0) (l self))
    (foo ((hash (exp (gen) (mul l l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 C) (l-1 l) (eA l-1) (eA l))
  (non-orig (privk C) (privk self-1) l)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1)
  (operation nonce-test (displaced 8 9 ltx-gen 2) l-1 (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l) C (privk C)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-1)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l) l-1)) (send (cat (exp (gen) l) l-1)))
    ((recv (cat (exp (gen) l) (one))) (send (cat (exp (gen) l) (one))))
    ((send (enc "cert" (exp (gen) l) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l-0) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-1) self-1 (privk self-1)))
      (send l-1)))
  (label 810)
  (parent 780)
  (unrealized (0 1))
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (0 0)) ((3 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 811)
  (parent 783)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (eA eA) (lA l) (lB l-0) (he (exp (gen) (mul (rec eA) e)))
        (A self-0) (C self) (he-0 he))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((2 0) (0 1)) ((3 1) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 812)
  (parent 787)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((2 0) (0 1)) ((3 1) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 813)
  (parent 788)
  (seen 830)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((2 0) (0 1)) ((3 1) (1 0)) ((4 2) (3 0))
    ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (3 0))
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 814)
  (parent 789)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((2 0) (0 1)) ((3 1) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 815)
  (parent 790)
  (seen 830)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (3 0)) ((2 0) (0 1)) ((3 1) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 816)
  (parent 791)
  (seen 830)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base) (eA rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (3 0)) ((2 0) (0 1)) ((3 1) (1 0)) ((4 0) (3 0))
    ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (3 0))
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 817)
  (parent 792)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 818)
  (parent 799)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 819)
  (parent 800)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 2) (1 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (3 0))
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 820)
  (parent 801)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 821)
  (parent 802)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (0 0)) ((5 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 822)
  (parent 803)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 0) (1 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (3 0))
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 823)
  (parent 804)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 8 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 824)
  (parent 805)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (6 0) (4 0))
  (uniq-gen e l l-0)
  (operation encryption-test (displaced 8 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 825)
  (parent 806)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 2) (6 0))
    ((8 0) (0 0)) ((8 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l-0 l-1)
  (operation encryption-test (displaced 9 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 826)
  (parent 807)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 8 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 827)
  (parent 808)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (0 0))
    ((7 1) (3 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0)
  (operation encryption-test (displaced 8 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-0) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 828)
  (parent 809)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base) (eA rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (6 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 1) (3 0))
    ((5 0) (0 1)) ((5 0) (4 0)) ((6 1) (2 0)) ((7 0) (6 0))
    ((8 0) (0 0)) ((8 1) (3 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (6 0) (4 0))
  (uniq-gen eA l l-0 l-1)
  (operation encryption-test (displaced 9 5 ltx-gen 1)
    (enc "cert" (exp (gen) l-1) C (privk C)) (0 1))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (cat (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 829)
  (parent 810)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (0 0)) ((3 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (uniq-gen eA l l-0)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 830)
  (parent 812)
  (unrealized)
  (shape)
  (maps
    ((0 1)
      ((kh kh) (eA eA) (lA l) (lB l-0) (he (exp (gen) (mul (rec eA) w)))
        (A self-0) (C self) (he-0 he))))
  (origs))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 2) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 831)
  (parent 814)
  (seen 830)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base) (eA rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 0) (0 1)) ((3 0) (1 0)) ((4 0) (0 0))
    ((4 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (3 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 832)
  (parent 817)
  (seen 830)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (gen)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (2 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (gen)))
      (send (hash (exp (gen) (mul l l-0)) (gen))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 833)
  (parent 818)
  (seen 781)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (2 0))
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 834)
  (parent 819)
  (seen 782)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base)
    (eA l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 2) (1 0)) ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (2 0))
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) e))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 835)
  (parent 820)
  (seen 783)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (2 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l-0)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l-0))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 836)
  (parent 821)
  (seen 784)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (0 0)) ((4 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (2 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 837)
  (parent 822)
  (seen 785)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base)
    (eA l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (1 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 0) (1 0)) ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (2 0))
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l)))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) l))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 838)
  (parent 823)
  (seen 786)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 839)
  (parent 824)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 840)
  (parent 825)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (1 0)) ((6 2) (5 0)) ((7 0) (0 0))
    ((7 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (5 0) (3 0))
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 841)
  (parent 826)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 842)
  (parent 827)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-0) l))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (1 0)) ((6 0) (0 0)) ((6 1) (2 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-0) l)) (send (cat (exp (gen) l-0) l)))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 843)
  (parent 828)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base) (eA rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-1) l-0))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (5 0)) ((2 1) (1 0)) ((3 1) (2 0)) ((4 0) (0 1))
    ((4 0) (3 0)) ((5 1) (1 0)) ((6 0) (5 0)) ((7 0) (0 0))
    ((7 1) (2 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (5 0) (3 0))
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-1) l-0)) (send (cat (exp (gen) l-1) l-0)))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 844)
  (parent 829)
  (unrealized)
  (comment "1 in cohort - 1 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (gen) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0) (2 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) w)))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) w))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (gen) w)) (send (cat (gen) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 845)
  (parent 839)
  (seen 812)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (w expt) (e l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e e))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) e) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he e)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (e l) (e l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0) (2 0))
  (uniq-gen e l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) e)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 846)
  (parent 840)
  (seen 813)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 peer self-1 name) (he hl base) (eA rndx)
    (w expt) (l e l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) e) w))
  (defstrand participant 3 (self self-0) (peer peer) (hl hl) (l l)
    (e e))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 2) (4 0)) ((6 0) (0 0)) ((6 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (4 0) (2 0))
  (uniq-gen eA l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w e)))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) e) w)) (send (cat (exp (gen) e) w)))
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" hl peer (privk peer))) (send (exp (gen) e)))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 847)
  (parent 841)
  (seen 814)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l-0) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0) (2 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l-0)))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l-0) w)) (send (cat (exp (gen) l-0) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 848)
  (parent 842)
  (seen 815)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 name) (he base) (eA rndx) (w expt)
    (l l-0 rndx))
  (defstrand participant 5 (kh kh) (self self-0) (peer self)
    (hl (exp (gen) l-0)) (he he) (l l) (e eA))
  (deflistener (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-0) (one)))
  (defstrand ltx-gen 1 (self self) (l l-0))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 2 (self self-0) (l l))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (0 0)) ((5 1) (1 0)))
  (fn-of ("principal-of" (l-0 self) (l self-0))
    (foo ((hash (exp (gen) (mul l l-0)) (exp he eA)) kh)))
  (neq (he (gen)) (self-0 self) (l l-0) (eA l) (eA l-0))
  (non-orig (privk self) (privk self-0) l-0)
  (precur (4 0) (2 0))
  (uniq-gen eA l l-0)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l) self-0 (privk self-0)))
      (recv (enc "cert" (exp (gen) l-0) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l l-0)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-0) (one)))
      (send (cat (exp (gen) l-0) (one))))
    ((send (enc "cert" (exp (gen) l-0) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))) (send l)))
  (label 849)
  (parent 843)
  (seen 816)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(defskeleton iadh-um
  (vars (kh data) (self self-0 self-1 name) (he base) (eA rndx) (w expt)
    (l l-0 l-1 rndx))
  (defstrand participant 5 (kh kh) (self self-1) (peer self)
    (hl (exp (gen) l-1)) (he he) (l l-0) (e eA))
  (deflistener (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
  (deflistener (cat (exp (gen) l-1) (one)))
  (defstrand ltx-gen 1 (self self) (l l-1))
  (deflistener (cat (exp (gen) l) w))
  (defstrand ltx-gen 1 (self self-0) (l l))
  (defstrand ltx-gen 2 (self self-1) (l l-0))
  (precedes ((0 2) (4 0)) ((2 1) (1 0)) ((3 0) (0 1)) ((3 0) (2 0))
    ((4 1) (1 0)) ((5 0) (4 0)) ((6 0) (0 0)) ((6 1) (1 0)))
  (fn-of ("principal-of" (l-1 self) (l-0 self-1) (l self-0))
    (foo ((hash (exp (gen) (mul l-0 l-1)) (exp he eA)) kh)))
  (neq (he (gen)) (self-1 self) (l-0 l-1) (eA l-0) (eA l-1))
  (non-orig (privk self) (privk self-1) l-1)
  (precur (4 0) (2 0))
  (uniq-gen eA l l-0 l-1)
  (operation generalization deleted (2 0))
  (traces
    ((recv (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (recv (enc "cert" (exp (gen) l-1) self (privk self)))
      (send (exp (gen) eA)) (recv he) (send kh))
    ((recv (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l))))
      (send (hash (exp (gen) (mul l-0 l-1)) (exp (gen) (mul w l)))))
    ((recv (cat (exp (gen) l-1) (one)))
      (send (cat (exp (gen) l-1) (one))))
    ((send (enc "cert" (exp (gen) l-1) self (privk self))))
    ((recv (cat (exp (gen) l) w)) (send (cat (exp (gen) l) w)))
    ((send (enc "cert" (exp (gen) l) self-0 (privk self-0))))
    ((send (enc "cert" (exp (gen) l-0) self-1 (privk self-1)))
      (send l-0)))
  (label 850)
  (parent 844)
  (seen 817)
  (unrealized)
  (comment "1 in cohort - 0 not yet seen"))

(comment "Nothing left to do")
